/*
 *  History:
 *  Started: Aug 9 by Lawrence Foard (entropy@world.std.com),
 *           to allow user process control of SCSI devices.
 *  Development Sponsored by Killy Corp. NY NY  [1992 ??]
 *
 * Original driver (sg.c):
 *        Copyright (C) 1992 Lawrence Foard
 * Version 2 and 3 extensions to driver:
 *        Copyright (C) 1998 - 2018 Douglas Gilbert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 *
 */

static int sg_version_num = 40008;      /* [x]xyyzz where [x] empty when x=0 */
#define SG_VERSION_STR "4.0.08"         /* [x]x.[y]y.zz */
static char *sg_version_date = "20190408";

#include <linux/module.h>

#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/errno.h>
#include <linux/mtio.h>
#include <linux/ioctl.h>
#include <linux/slab.h>
#include <linux/fcntl.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/moduleparam.h>
#include <linux/cdev.h>
#include <linux/idr.h>
#include <linux/file.h>         /* for fget() and fput() */
#include <linux/seq_file.h>
#include <linux/blkdev.h>
#include <linux/delay.h>
#include <linux/blktrace_api.h>
#include <linux/mutex.h>
#include <linux/atomic.h>
#include <linux/ratelimit.h>
#include <linux/uio.h>
#include <linux/cred.h>         /* for sg_check_file_access() */
#include <linux/bsg.h>
#include <linux/timekeeping.h>
#include <linux/circ_buf.h>	/* for SG_TRC */
#include <linux/proc_fs.h>	/* used if CONFIG_SCSI_PROC_FS */

#include "scsi.h"
#include <scsi/scsi_dbg.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_driver.h>
#include <scsi/scsi_ioctl.h>
#include <scsi/sg.h>

#include "scsi_logging.h"


#define SG_ALLOW_DIO_DEF 0

#define SG_MAX_DEVS 32768
#define SG_MAX_MULTI_REQ_SZ (2 * 1024 * 1024)

/* Comment out the following line to compile out SCSI_LOGGING stuff */
#define SG_DEBUG 1

#if !IS_ENABLED(SG_DEBUG)
#if IS_ENABLED(DEBUG)    /* If SG_DEBUG not defined, check for DEBUG */
#define SG_DEBUG DEBUG
#endif
#endif

#if IS_ENABLED(CONFIG_SCSI_PROC_FS) && IS_ENABLED(SG_DEBUG)
#define SG_PROC_AND_DEBUG 1
#endif

/*
 * SG_MAX_CDB_SIZE should be 260 (spc4r37 section 3.1.30) however the type
 * of sg_io_hdr::cmd_len can only represent 255. All SCSI commands greater
 * than 16 bytes are "variable length" whose length is a multiple of 4, so
 */
#define SG_MAX_CDB_SIZE 252

#define uptr64(val) ((void __user *)(uintptr_t)(val))
#define cuptr64(val) ((const void __user *)(uintptr_t)(val))

/* Following enum contains the states of sg_request::rq_st */
enum sg_rq_state {
	SG_RS_INACTIVE = 0,	/* request not in use (e.g. on fl) */
	SG_RS_INFLIGHT,		/* active: cmd/req issued, no response yet */
	SG_RS_AWAIT_RD,		/* response received, awaiting read */
	SG_RS_DONE_RD,		/* read is ongoing or done */
	SG_RS_BUSY,		/* temporary state should rarely be seen */
	SG_RS_SHR_SWAP,		/* swap: master finished, awaiting slave */
	SG_RS_SHR_SLAVE,	/* master waits while slave inflight */
	SG_RS_SHR_SWAIT,	/* slave waiting for master to complete */
};

/* slave sets up sharing: ioctl(sl_fd,SG_SET_GET_EXTENDED(SHARE_FD(ma_fd))) */
enum sg_shr_var {
	SG_SHR_NONE = 0,	/* no sharing on owning fd */
	SG_SHR_MA_FD_NOT_RQ,	/* master sharing on fd but not this req */
	SG_SHR_MA_RQ,		/* master sharing on this req */
	SG_SHR_SL_FD_NOT_RQ,	/* slave sharing on fd but not this req */
	SG_SHR_SL_RQ,		/* slave sharing on this req */
};

/* If sum_of(dlen) of a fd exceeds this, write() will yield E2BIG */
#define SG_TOT_FD_THRESHOLD (32 * 1024 * 1024)

#define SG_TIME_UNIT_MS 0	/* milliseconds */
/* #define SG_TIME_UNIT_NS 1	   nanoseconds */
#define SG_DEF_TIME_UNIT SG_TIME_UNIT_MS
#define SG_DEFAULT_TIMEOUT mult_frac(SG_DEFAULT_TIMEOUT_USER, HZ, USER_HZ)
#define SG_FD_Q_AT_HEAD 0
#define SG_DEFAULT_Q_AT SG_FD_Q_AT_HEAD /* for backward compatibility */
#define SG_FL_MMAP_DIRECT (SG_FLAG_MMAP_IO | SG_FLAG_DIRECT_IO)

/* Only take lower 4 bits of driver byte, all host byte and sense byte */
#define SG_ML_RESULT_MSK 0x0fff00ff	/* mid-level's 32 bit result value */

#define SG_SHARE_FD_UNUSED (-1)
#define SG_SHARE_FD_MASTER (-2)		/* values >= 0 imply FD_SLAVE */

#define SG_PACK_ID_WILDCARD (-1)
#define SG_TAG_WILDCARD (-1)

#define SG_ADD_RQ_MAX_RETRIES 40	/* to stop infinite _trylock(s) */

/* Bit positions (flags) for sg_request::frq_bm bitmask follow */
#define SG_FRQ_IS_V4I		0	/* true (set) when is v4 interface */
#define SG_FRQ_IS_ORPHAN	1	/* owner of request gone */
#define SG_FRQ_SYNC_INVOC	2	/* synchronous (blocking) invocation */
#define SG_FRQ_DIO_IN_USE	3	/* false->indirect_IO,mmap; 1->dio */
#define SG_FRQ_NO_US_XFER	4	/* no user space transfer of data */
#define SG_FRQ_ABORTING		5	/* in process of aborting this cmd */
#define SG_FRQ_DEFER_XFR	6	/* slave waiting will defer transfer */
#define SG_FRQ_DEACT_ORPHAN	7	/* not keeping orphan so de-activate */

/* Bit positions (flags) for sg_fd::ffd_bm bitmask follow */
#define SG_FFD_FORCE_PACKID	0	/* receive only given pack_id/tag */
#define SG_FFD_CMD_Q		1	/* clear: only 1 active req per fd */
#define SG_FFD_KEEP_ORPHAN	2	/* policy for this fd */
#define SG_FFD_MMAP_CALLED	3	/* mmap(2) system call made on fd */
#define SG_FFD_TIME_IN_NS	4	/* set: time in nanoseconds, else ms */
#define SG_FFD_Q_AT_TAIL	5	/* set: queue reqs at tail of blk q */
#define SG_FFD_MASTER_ERR	6	/* master side of share had error */
#define SG_FFD_PREFER_TAG	7	/* prefer tag over pack_id (def) */
#define SG_FFD_RELEASE		8	/* release (close) underway */


int sg_big_buff = SG_DEF_RESERVED_SIZE;
/*
 * This variable is accessible via /proc/scsi/sg/def_reserved_size . Each
 * time sg_open() is called a sg_request of this size (or less if there is
 * not enough memory) will be reserved for use by this file descriptor.
 */
static int def_reserved_size = -1;      /* picks up init parameter */
static int sg_allow_dio = SG_ALLOW_DIO_DEF;

static int scatter_elem_sz = SG_SCATTER_SZ;

#define SG_DEF_SECTOR_SZ 512

static struct kmem_cache *sg_sense_cache;
#define SG_MEMPOOL_MIN_NR 4
static mempool_t *sg_sense_pool;

static int sg_add_device(struct device *, struct class_interface *);
static void sg_remove_device(struct device *, struct class_interface *);

static DEFINE_IDR(sg_index_idr);
static DEFINE_RWLOCK(sg_index_lock); /* Also used to lock fd list for device */

static struct class_interface sg_interface = {
	.add_dev        = sg_add_device,
	.remove_dev     = sg_remove_device,
};

/* Subset of sg_io_hdr found in <scsi/sg.h>, has only [i] and [i->o] fields */
struct sg_slice_hdr3 {
	int interface_id;
	int dxfer_direction;
	unsigned char cmd_len;
	unsigned char mx_sb_len;
	unsigned short iovec_count;
	unsigned int dxfer_len;
	void __user *dxferp;
	unsigned char __user *cmdp;
	void __user *sbp;
	unsigned int timeout;
	unsigned int flags;
	int pack_id;
	void __user *usr_ptr;
};

struct sg_slice_hdr4 {	/* parts of sg_io_v4 object needed in async usage */
	void __user *sbp;	/* derived from sg_io_v4::response */
	u64 usr_ptr;		/* hold sg_io_v4::usr_ptr as given (u64) */
	int out_resid;
	s16 dir;		/* data xfer direction; SG_DXFER_*  */
	u16 cmd_len;		/* truncated of sg_io_v4::request_len */
	u16 max_sb_len;		/* truncated of sg_io_v4::max_response_len */
	u16 mrq_ind;		/* position in parentfp->mrq_arr */
};

struct sg_scatter_hold {     /* holding area for scsi scatter gather info */
	struct page **pages;	/* num_sgat element array of struct page* */
	int buflen;		/* capacity in bytes (dlen<=buflen) */
	int dlen;		/* current valid data length of this req */
	u16 page_order;		/* byte_len = (page_size*(2**page_order)) */
	u16 num_sgat;		/* actual number of scatter-gather segments */
};

struct sg_device;		/* forward declarations */
struct sg_fd;

struct sg_request {	/* active SCSI command or inactive on free list (fl) */
	struct list_head rq_entry;	/* member of rq_list (active cmd) */
	struct list_head fl_entry;	/* member of rq_fl */
	spinlock_t req_lck;
	struct sg_scatter_hold sgat_h;	/* hold buffer, perhaps scatter list */
	struct sg_scatter_hold *sgatp;	/* ptr to prev unless slave shr req */
	union {
		struct sg_slice_hdr3 s_hdr3;  /* subset of sg_io_hdr */
		struct sg_slice_hdr4 s_hdr4; /* reduced size struct sg_io_v4 */
	};
	union {
		ktime_t start_kt;/* start time if SG_FFD_TIME_IN_NS set */
		u32 start_ms;	/* start time if SG_FFD_TIME_IN_NS clear */
	};
	u32 duration;		/* cmd duration in milli or nano seconds */
	u32 rq_flags;		/* flags given in v3 and v4 */
	u32 rq_info;		/* info supplied by v3 and v4 interfaces */
	u32 rq_result;		/* packed scsi request result from LLD */
	int in_resid;		/* requested-actual byte count on data-in */
	int pack_id;		/* v3 pack_id or in v4 request_extra field */
	int sense_len;		/* actual sense buffer length (data-in) */
	atomic_t rq_st;		/* holds a enum sg_rq_state value */
	enum sg_shr_var sh_var; /* info about sharing, if any */
	u8 cmd_opcode;		/* first byte of command */
	int tag;		/* block layer identifier of request */
	unsigned long frq_bm[1];	/* see SG_FRQ_* defines above */
	u8 *sense_bp;		/* mempool alloc-ed sense buffer, as needed */
	struct sg_fd *parentfp;	/* pointer to owning fd, even when on fl */
	struct request *rq;	/* released in sg_rq_end_io(), bio kept */
	struct bio *bio;	/* kept until this req -->SG_RS_INACTIVE */
	struct execute_work ew_orph;	/* harvest orphan request */
};

struct sg_fd {		/* holds the state of a file descriptor */
	struct list_head sfd_entry;	/* member sg_device::sfds list */
	struct sg_device *parentdp;	/* owning device */
	wait_queue_head_t read_wait;	/* queue read until command done */
	struct mutex f_mutex;	/* serialize ioctls on this fd */
	rwlock_t rq_list_lock;	/* protect sg_request lists and this object */
	struct list_head rq_list; /* head of inflight sg_request list */
	struct list_head rq_fl; /* head of sg_request free list */
	int timeout;		/* defaults to SG_DEFAULT_TIMEOUT      */
	int timeout_user;	/* defaults to SG_DEFAULT_TIMEOUT_USER */
	int tot_fd_thresh;	/* E2BIG if sum_of(dlen) > this, 0: ignore */
	int sgat_elem_sz;	/* initialized to scatter_elem_sz */
	int shr_fd;		/* init: SG_SHARE_FD_UNUSED; master: -2 */
	atomic_t sum_fd_dlens;	/* when tot_fd_thresh>0 this is sum_of(dlen) */
	atomic_t submitted;	/* number inflight or awaiting read */
	atomic_t waiting;	/* number of requests awaiting read */
	unsigned long ffd_bm[1];	/* see SG_FFD_* defines above */
	u8 next_cmd_len;	/* 0: automatic, >0: use on next write() */
	const struct file *filp;	/* my identity when sharing */
	struct sg_request *rsv_srp;/* one reserve request per fd */
	struct sg_request *slave_srp;	/* non-NULL when rsv SG_SHR_MA_RQ */
	struct sg_fd *share_sfp;/* master+slave shares set this, else NULL */
	struct fasync_struct *async_qp; /* used by asynchronous notification */
	struct kref f_ref;
	struct execute_work ew_fd;  /* harvest all fd resources and lists */
};

struct sg_device {	/* holds the state of each scsi generic device */
	struct scsi_device *device;
	wait_queue_head_t open_wait;	/* queue open() when O_EXCL present */
	struct mutex open_rel_lock;	/* held when in open() or release() */
	int max_sgat_elems;	/* adapter's max sgat number of elements */
	int max_sgat_sz;	/* max number of bytes in sgat list */
	u32 index;		/* device index number */
	struct list_head sfds;	/* head of sg_fd::sfd_entry list */
	rwlock_t sfd_llock;	/* protect access to sfds list */
	atomic_t detaching;	/* 0->device usable, 1->device detaching */
	bool exclude;		/* 1->open(O_EXCL) succeeded and is active */
	u8 sgdebug;	/* 0->off, 1->sense, 9->dump dev, 10-> all devs */
	int open_cnt;		/* count of opens (perhaps < num(sfds) ) */
	struct gendisk *disk;
	struct cdev * cdev;	/* char_dev [sysfs: /sys/cdev/major/sg<n>] */
	struct kref d_ref;
};

struct sg_comm_wr_t {	/* arguments to sg_common_write() */
	bool sync;
	bool mrq;
	bool v4i;		/* selector for union; true --> h4p */
	int timeout;
	union {
		struct sg_io_hdr *h3p;
		struct sg_io_v4 *h4p;
	};
	u8 *cmnd;
};


/* tasklet or soft irq callback */
static void sg_rq_end_io(struct request *rq, blk_status_t status);
/* Declarations of other static functions used before they are defined */
static int sg_proc_init(void);
static int sg_start_req(struct sg_request *srp, u8 *cmd, int cmd_len,
			struct sg_io_v4 *h4p, int dxfer_dir);
static void sg_finish_scsi_blk_rq(struct sg_request *srp);
static int sg_mk_sgat(struct sg_request *srp, struct sg_fd *sfp, int minlen);
static int sg_v3_receive(struct sg_fd *sfp, struct sg_request *srp,
			 void __user *p);
static int sg_v3_submit(struct file *filp, struct sg_fd *sfp,
			struct sg_io_hdr *hp, bool sync,
			struct sg_request **o_srp);
static struct sg_request *sg_common_write(struct sg_fd *sfp,
					  struct sg_comm_wr_t *cwrp);
static int sg_wait_event_srp(struct file *filp, struct sg_fd *sfp,
			     void __user *p, struct sg_io_v4 *h4p,
			     struct sg_request *srp);
static int sg_v4_receive(struct sg_fd *sfp, struct sg_request *srp,
			 void __user *p, struct sg_io_v4 *h4p);
static int sg_rd_append(struct sg_request *srp, void __user *outp,
			int num_xfer);
static void sg_remove_sgat(struct sg_request *srp);
static struct sg_fd *sg_add_sfp(struct sg_device *sdp);
static void sg_remove_sfp(struct kref *);
static struct sg_request *sg_get_rq_pack_id(struct sg_fd *sfp, int pack_id,
					    int tag, bool use_tag);
static bool sg_mrq_get_ready_srp(struct sg_fd *sfp, struct sg_request **srpp);
static struct sg_request *sg_add_request(struct sg_fd *sfp, int dxfr_len,
					 bool sync, enum sg_shr_var sh_var);
static int sg_rq_map_kern(struct sg_request *srp, struct request_queue *q,
			  struct request *rq);
static void sg_deact_request(struct sg_fd *sfp, struct sg_request *srp);
static struct sg_device *sg_get_dev(int min_dev);
static void sg_device_destroy(struct kref *kref);
static struct sg_request *sg_mk_srp(struct sg_fd *sfp, bool first);
static struct sg_request *sg_mk_srp_sgat(struct sg_fd *sfp, bool first,
					 int db_len);
static void sg_calc_sgat_param(struct sg_device *sdp);
static const char *sg_rq_st_str(enum sg_rq_state rq_st, bool long_str);
static const char *sg_shr_str(enum sg_shr_var sh_var, bool long_str);
static void sg_rep_rq_state_fail(struct sg_device *sdp,
				 enum sg_rq_state exp_old_st,
				 enum sg_rq_state want_st,
				 enum sg_rq_state act_old_st);

#define SZ_SG_HEADER ((int)sizeof(struct sg_header))	/* v1 and v2 header */
#define SZ_SG_IO_HDR ((int)sizeof(struct sg_io_hdr))	/* v3 header */
#define SZ_SG_IO_V4 ((int)sizeof(struct sg_io_v4))  /* v4 header (in bsg.h) */
#define SZ_SG_REQ_INFO ((int)sizeof(struct sg_req_info))
#define SZ_SG_EXTENDED_INFO ((int)sizeof(struct sg_extended_info))

/* There is a assert that SZ_SG_IO_V4 >= SZ_SG_IO_HDR in first function */

#define SG_RS_ACTIVE(srp) (atomic_read(&(srp)->rq_st) != SG_RS_INACTIVE)
#define SG_RS_AWAIT_READ(srp) (atomic_read(&(srp)->rq_st) == SG_RS_AWAIT_RD)
#define SG_MA_THIS_RQ(srp) ((srp)->sh_var == SG_SHR_MA_RQ)

/*
 * Kernel needs to be built with CONFIG_SCSI_LOGGING to see log messages.
 * 'depth' is a number between 1 (most severe) and 7 (most noisy, most
 * information). All messages are logged as informational (KERN_INFO). In
 * the unexpected situation where sdp is NULL the macro reverts to a pr_info
 * and ignores CONFIG_SCSI_LOGGING and always prints to the log.
 */

#define SG_PROC_DEBUG_SZ 8192
#define SG_TRC_CB_ORDER 15	/* 2^this must be > SG_PROC_DEBUG_SZ */
static int sg_trc_cb_sz = 1 << SG_TRC_CB_ORDER;
static struct circ_buf sg_trc_cb;
static DEFINE_MUTEX(sg_trc_cb_lck);
static bool sg_trc_cb_valid;
static int sg_trc_ctl_level;
static char *sg_trc_wp;
#if IS_ENABLED(SG_DEBUG)
#endif

#if IS_ENABLED(SG_PROC_AND_DEBUG)

static int sg_trc_ctl_exp = ilog2(SG_PROC_DEBUG_SZ);

/* trace is off by default; needs 'echo 1 > /proc/scsi/sg/trc_ctl' */
#define SG_TRC(from, extra, trc_type, trc_lev, vp, skip_lock, act_sl)	\
	       sg_trc_producer((from), (extra), (trc_type), (trc_lev),	\
			       (vp), (skip_lock), (act_sl))

/* Used when debugging, part of object tree to trace */
enum sg_trc_type {
	SG_TRC_ALL,
	SG_TRC_DEV,
	SG_TRC_FD,
	SG_TRC_REQ,
};

static void sg_trc_producer(const char *from, const char *extra,
			    enum sg_trc_type tt, int trc_lev, void *vp,
			    bool skip_lock, bool follow_slave);

#else

#define SG_TRC(from, extra, trc_type, trc_lev, vp, skip_lock, act_sl)

#endif	/* end of SG_PROC_AND_DEBUG conditional */

#if IS_ENABLED(CONFIG_SCSI_LOGGING)
#define SG_LOG(depth, sdp, fmt, a...)					\
	do {								\
		char _b[256];						\
		int _tid = (current ? current->pid : -1);		\
									\
		if ((sdp) && (sdp)->disk) {				\
			snprintf(_b, sizeof(_b), "%s: tid=%d",		\
				 (sdp)->disk->disk_name, _tid);		\
			SCSI_LOG_TIMEOUT(depth,				\
					 sdev_prefix_printk(KERN_INFO,	\
					 (sdp)->device, _b, fmt, ##a));	\
		} else							\
			pr_info("sg: sdp=NULL_or_ERR, " fmt, ##a);	\
	} while (0)
#else
#define SG_LOG(depth, sdp, fmt, a...)
#endif	/* end of CONFIG_SCSI_LOGGING conditional */

/*
 * Unless otherwise noted, functions that return int will return 0 for
 * good/successful or a negated errno value. A list of errno_s follows:
 *
 * E2BIG    sum(dlen) > tot_fd_thresh ; slave dxfer_len > master dxfer_len
 * EACCES   user (process) does not have sufficient privilege or capabilities
 * EADDRINUSE	sharing: slave file descriptor already in share
 * EADDRNOTAVAIL   sharing: master file descriptor already in share
 *		   slave request but no preceding master request
 * EAGAIN   'Resource temporarily unavailable'; O_NONBLOCK on open() or
 *	    SG_IORECEIVE (or read(2)) not ready and SGV4_FLAG_IMMED given
 * EBUSY    'Device or resource busy'; this uses open(O_NONBLOCK) but another
 *	    has open(O_EXCL); reserve request in use (e.g. when mmap() called)
 * EDOM     numerical error, command queueing false and second command
 *	    attempted when one already outstanding
 * EFAULT   problem moving data to or from user space
 * EINTR    interrupted system call (generated by kernel, not this driver)
 * EINVAL   flags or other input information contradicts or disallowed
 * EIO      only kept for backward compatibility, too generic to be useful
 * ELOOP    sharing: file descriptor can't share with itself
 * EMSGSIZE    cdb too long (> 252 bytes) or too short (less than 6 bytes)
 * ENODATA  sharing: no data xfer requested; mmap or direct io problem
 *          SG_IOABORT: no match on pack_id or tag; mrq: no active reqs
 * ENODEV   target (SCSI) device associated with the fd has "disappeared"
 * ENOMEM   obvious; could be some pre-allocated cache that is full
 * ENOMSG   data transfer setup needed or (direction) disallowed (sharing)
 * ENOSTR   slave request abandoned due to master error or state
 * ENOTSOCK   sharing: file descriptor for sharing unassociated with sg driver
 * ENXIO    'no such device or address' SCSI mid-level processing errors
 *          (e.g. command timeouts); also sg info not in 'file' struct
 * EPERM    not permitted (even if has ACCES); v1+2,v3,v4 interface usage
 *	    violation, opened read-only but SCSI command not listed read-only
 * EPROTO   logic error (in driver); like "shouldn't get here"
 * EPROTOTYPE    atomic state change failed unexpectedly
 * ERANGE   multiple requests: usually bad flag values
 * ERESTARTSYS   should not be seen in user space, associated with an
 *		 interruptable wait; will restart system call or give EINTR
 */

/*
 * The SCSI interfaces that use read() and write() as an asynchronous variant of
 * ioctl(..., SG_IO, ...) are fundamentally unsafe, since there are lots of ways
 * to trigger read() and write() calls from various contexts with elevated
 * privileges. This can lead to kernel memory corruption (e.g. if these
 * interfaces are called through splice()) and privilege escalation inside
 * userspace (e.g. if a process with access to such a device passes a file
 * descriptor to a SUID binary as stdin/stdout/stderr).
 *
 * This function provides protection for the legacy API by restricting the
 * calling context.
 */
static int
sg_check_file_access(struct file *filp, const char *caller)
{
	/* can't put following in declarations where it belongs */
	compiletime_assert(SZ_SG_IO_V4 >= SZ_SG_IO_HDR,
			   "struct sg_io_v4 should be larger than sg_io_hdr");

	if (filp->f_cred != current_real_cred()) {
		pr_err_once("%s: process %d (%s) changed security contexts after opening file descriptor, this is not allowed.\n",
			caller, task_tgid_vnr(current), current->comm);
		return -EPERM;
	}
	if (uaccess_kernel()) {
		pr_err_once("%s: process %d (%s) called from kernel context, this is not allowed.\n",
			caller, task_tgid_vnr(current), current->comm);
		return -EACCES;
	}
	return 0;
}

static int
wait_open_event(struct sg_device *sdp, int op_flags)
	__must_hold(&sdp->open_rel_lock)
{
	int retval = 0;

	if (op_flags & O_EXCL) {
		while (sdp->open_cnt > 0) {
			mutex_unlock(&sdp->open_rel_lock);
			retval = wait_event_interruptible(sdp->open_wait,
				    (unlikely(atomic_read(&sdp->detaching)) ||
				    !sdp->open_cnt));
			mutex_lock(&sdp->open_rel_lock);

			if (retval) /* -ERESTARTSYS */
				return retval;
			if (unlikely(atomic_read(&sdp->detaching)))
				return -ENODEV;
		}
	} else {
		while (sdp->exclude) {
			mutex_unlock(&sdp->open_rel_lock);
			retval = wait_event_interruptible(sdp->open_wait,
				    unlikely((atomic_read(&sdp->detaching)) ||
				    !sdp->exclude));
			mutex_lock(&sdp->open_rel_lock);

			if (retval) /* -ERESTARTSYS */
				return retval;
			if (unlikely(atomic_read(&sdp->detaching)))
				return -ENODEV;
		}
	}

	return retval;
}

/*
 * Corresponds to the open() system call on sg devices. Implements O_EXCL on
 * a per device basis using 'open_cnt'. If O_EXCL and O_NONBLOCK and there is
 * already a sg handle open on this device then it fails with an errno of
 * EBUSY. Without the O_NONBLOCK flag then this thread enters an interruptible
 * wait until the other handle(s) are closed.
 */
static int
sg_open(struct inode *inode, struct file *filp)
{
	int min_dev = iminor(inode);
	int op_flags = filp->f_flags;
	struct sg_device *sdp;
	struct sg_fd *sfp;
	int retval;

	nonseekable_open(inode, filp);
	if ((op_flags & O_EXCL) && (O_RDONLY == (op_flags & O_ACCMODE)))
		return -EPERM;/* not permitted, need write access for O_EXCL */
	sdp = sg_get_dev(min_dev);
	if (IS_ERR(sdp))
		return PTR_ERR(sdp);
	SG_LOG(3, sdp, "%s: minor=%d, op_flags=0x%x; %s count prior=%d%s\n",
	       __func__, min_dev, op_flags, "device open", sdp->open_cnt,
	       ((op_flags & O_NONBLOCK) ? " O_NONBLOCK" : ""));

	/* This driver's module count bumped by fops_get in <linux/fs.h> */
	/* Prevent the device driver from vanishing while we sleep */
	retval = scsi_device_get(sdp->device);
	if (retval)
		goto sg_put;

	retval = scsi_autopm_get_device(sdp->device);
	if (retval)
		goto sdp_put;

	/* scsi_block_when_processing_errors() may block so bypass
	 * check if O_NONBLOCK. Permits SCSI commands to be issued
	 * during error recovery. Tread carefully. */
	if (!((op_flags & O_NONBLOCK) ||
	      scsi_block_when_processing_errors(sdp->device))) {
		retval = -ENXIO;
		/* we are in error recovery for this device */
		goto error_out;
	}

	mutex_lock(&sdp->open_rel_lock);
	if (op_flags & O_NONBLOCK) {
		if (op_flags & O_EXCL) {
			if (sdp->open_cnt > 0) {
				retval = -EBUSY;
				goto error_mutex_locked;
			}
		} else {
			if (sdp->exclude) {
				retval = -EBUSY;
				goto error_mutex_locked;
			}
		}
	} else {
		retval = wait_open_event(sdp, op_flags);
		if (retval) /* -ERESTARTSYS or -ENODEV */
			goto error_mutex_locked;
	}

	/* N.B. at this point we are holding the open_rel_lock */
	if (op_flags & O_EXCL)
		sdp->exclude = true;

	if (sdp->open_cnt < 1) {  /* no existing opens */
		sdp->sgdebug = 0;
		/* Next done in sg_alloc(), repeat here to pick up change? */
		sg_calc_sgat_param(sdp);
	}
	sfp = sg_add_sfp(sdp);
	if (IS_ERR_OR_NULL(sfp)) {
		retval = IS_ERR(sfp) ? PTR_ERR(sfp) : -ENXIO;
		goto out_undo;
	}

	filp->private_data = sfp;
	sfp->filp = filp;
	sdp->open_cnt++;
	mutex_unlock(&sdp->open_rel_lock);

	retval = 0;
sg_put:
	kref_put(&sdp->d_ref, sg_device_destroy);
	return retval;

out_undo:
	if (op_flags & O_EXCL) {
		sdp->exclude = false;   /* undo if error */
		wake_up_interruptible(&sdp->open_wait);
	}
error_mutex_locked:
	mutex_unlock(&sdp->open_rel_lock);
error_out:
	scsi_autopm_put_device(sdp->device);
sdp_put:
	scsi_device_put(sdp->device);
	goto sg_put;
}

/*
 * Release resources associated with a prior, successful sg_open(). It can be
 * seen as the (final) close() call on a sg device file descriptor in the user
 * space. The real work releasing all resources associated with this file
 * descriptor is done by sg_remove_sfp_usercontext() which is scheduled by
 * sg_remove_sfp().
 */
static int
sg_release(struct inode *inode, struct file *filp)
{
	struct sg_device *sdp;
	struct sg_fd *sfp;

	sfp = filp->private_data;
	if (IS_ERR_OR_NULL(sfp)) {
		pr_warn("sg: %s: sfp is NULL or error\n", __func__);
		return IS_ERR(sfp) ? PTR_ERR(sfp) : -ENXIO;
	}
	sdp = sfp->parentdp;
	if (IS_ERR_OR_NULL(sdp))
		return IS_ERR(sdp) ? PTR_ERR(sdp) : -ENXIO;
	SG_LOG(3, sdp, "%s: device open count prior=%d\n", __func__,
	       sdp->open_cnt);
	set_bit(SG_FFD_RELEASE, sfp->ffd_bm);

	mutex_lock(&sdp->open_rel_lock);
	scsi_autopm_put_device(sdp->device);
	kref_put(&sfp->f_ref, sg_remove_sfp);
	sdp->open_cnt--;

	/* possibly many open()s waiting on exlude clearing, start many;
	 * only open(O_EXCL)s wait on 0==open_cnt so only start one */
	if (sdp->exclude) {
		sdp->exclude = false;
		wake_up_interruptible_all(&sdp->open_wait);
	} else if (0 == sdp->open_cnt) {
		wake_up_interruptible(&sdp->open_wait);
	}
	mutex_unlock(&sdp->open_rel_lock);
	return 0;
}

/*
 * write(2) related functions follow. They are shown before read(2) related
 * functions. That is because SCSI commands/requests are first "written" to
 * the SCSI device by using write(2), ioctl(SG_IOSUBMIT) or the first half
 * of the synchronous ioctl(SG_IO) system call.
 */

/*
 * This is the write(2) system call entry point. v4 interface disallowed.
 * Returns count or a negated errno value.
 */
static ssize_t
sg_write(struct file *filp, const char __user *p, size_t count, loff_t *ppos)
{
	int mxsize, cmd_size, input_size, res;
	u8 opcode;
	struct sg_device *sdp;
	struct sg_fd *sfp;
	struct sg_request *srp;
	u8 cmnd[SG_MAX_CDB_SIZE];
	struct sg_header ov2hdr;
	struct sg_io_hdr v3hdr;
	struct sg_header *ohp = &ov2hdr;
	struct sg_io_hdr *h3p = &v3hdr;
	struct sg_comm_wr_t cwr;

	res = sg_check_file_access(filp, __func__);
	if (res)
		return res;

	sfp = filp->private_data;
	if (IS_ERR_OR_NULL(sfp)) {
		pr_warn("sg: %s: sfp is NULL or error\n", __func__);
		return IS_ERR(sfp) ? PTR_ERR(sfp) : -ENXIO;
	}
	sdp = sfp->parentdp;
	SG_LOG(3, sdp, "%s: write(3rd arg) count=%d\n", __func__, (int)count);
	if (IS_ERR_OR_NULL(sdp))
		return IS_ERR(sdp) ? PTR_ERR(sdp) : -ENXIO;
	if (unlikely(atomic_read(&sdp->detaching)))
		return -ENODEV;
	if (!((filp->f_flags & O_NONBLOCK) ||
	      scsi_block_when_processing_errors(sdp->device)))
		return -ENXIO;

	if (!access_ok(p, count))
		return -EFAULT;
	if (count < SZ_SG_HEADER)
		return -EIO;
	if (__copy_from_user(ohp, p, SZ_SG_HEADER))
		return -EFAULT;
	if (ohp->reply_len < 0) {	/* assume this is v3 */
		struct sg_io_hdr *reinter_2p = (struct sg_io_hdr *)ohp;
		u8 *h3u8p = (u8 *)&v3hdr;

		if (count < SZ_SG_IO_HDR)
			return -EIO;
		if (reinter_2p->interface_id != 'S') {
			pr_info_once("sg: %s: v3 interface only here\n",
				     __func__);
			return -EPERM;
		}
		memcpy(h3p, ohp, SZ_SG_HEADER);
		if (__copy_from_user(h3u8p + SZ_SG_HEADER, p + SZ_SG_HEADER,
				     SZ_SG_IO_HDR - SZ_SG_HEADER))
			return -EFAULT;
		res = sg_v3_submit(filp, sfp, h3p, false, NULL);
		return res < 0 ? res : (int)count;
	}
	/* v1 and v2 interfaces processed below this point */
	if (count < (SZ_SG_HEADER + 6))
		return -EIO;    /* minimum scsi command length is 6 bytes */
	p += SZ_SG_HEADER;
	__get_user(opcode, p);
	mutex_lock(&sfp->f_mutex);
	if (sfp->next_cmd_len > 0) {
		cmd_size = sfp->next_cmd_len;
		sfp->next_cmd_len = 0;  /* reset, only this write() effected */
	} else {
		cmd_size = COMMAND_SIZE(opcode);/* old; SCSI command group */
		if (opcode >= 0xc0 && ohp->twelve_byte)
			cmd_size = 12;
	}
	mutex_unlock(&sfp->f_mutex);
	SG_LOG(4, sdp, "%s:   scsi opcode=0x%02x, cmd_size=%d\n", __func__,
	       (unsigned int)opcode, cmd_size);
	input_size = count - cmd_size;
	mxsize = max_t(int, input_size, ohp->reply_len);
	mxsize -= SZ_SG_HEADER;
	input_size -= SZ_SG_HEADER;
	if (input_size < 0)
		return -EIO; /* Insufficient bytes passed for this command. */
	memset(h3p, 0, sizeof(*h3p));
	h3p->interface_id = '\0';/* indicate v1 or v2 interface (tunnelled) */
	h3p->cmd_len = (u8)cmd_size;
	h3p->iovec_count = 0;
	h3p->mx_sb_len = 0;
	if (input_size > 0)
		h3p->dxfer_direction = (ohp->reply_len > SZ_SG_HEADER) ?
		    SG_DXFER_TO_FROM_DEV : SG_DXFER_TO_DEV;
	else
		h3p->dxfer_direction = (mxsize > 0) ? SG_DXFER_FROM_DEV :
						      SG_DXFER_NONE;
	h3p->dxfer_len = mxsize;
	if (h3p->dxfer_direction == SG_DXFER_TO_DEV ||
	    h3p->dxfer_direction == SG_DXFER_TO_FROM_DEV)
		h3p->dxferp = (u8 __user *)p + cmd_size;
	else
		h3p->dxferp = NULL;
	h3p->sbp = NULL;
	h3p->timeout = ohp->reply_len;   /* structure abuse ... */
	h3p->flags = input_size;         /* structure abuse ... */
	h3p->pack_id = ohp->pack_id;
	h3p->usr_ptr = NULL;
	cmnd[0] = opcode;
	if (__copy_from_user(cmnd + 1, p + 1, cmd_size - 1))
		return -EFAULT;
	/*
	 * SG_DXFER_TO_FROM_DEV is functionally equivalent to SG_DXFER_FROM_DEV,
	 * but it is possible that the app intended SG_DXFER_TO_DEV, because
	 * there is a non-zero input_size, so emit a warning.
	 */
	if (h3p->dxfer_direction == SG_DXFER_TO_FROM_DEV) {
		printk_ratelimited(KERN_WARNING
			"%s: data in/out %d/%d bytes for SCSI command 0x%x-- guessing data in;\n"
			"   program %s not setting count and/or reply_len properly\n",
			__func__, ohp->reply_len - (int)SZ_SG_HEADER,
			input_size, (unsigned int)cmnd[0], current->comm);
	}
	cwr.sync = false;
	cwr.mrq = false;
	cwr.v4i = false;
	cwr.h3p = h3p;
	cwr.timeout = sfp->timeout;
	cwr.cmnd = cmnd;
	srp = sg_common_write(sfp, &cwr);
	return (IS_ERR(srp)) ? PTR_ERR(srp) : (int)count;
}

static inline int
sg_chk_mmap(struct sg_fd *sfp, int rq_flags, int len)
{
	if (!list_empty(&sfp->rq_list))
		return -EBUSY;  /* already active requests on fd */
	if (len > sfp->rsv_srp->sgatp->buflen)
		return -ENOMEM; /* MMAP_IO size must fit in reserve */
	if (rq_flags & SG_FLAG_DIRECT_IO)
		return -EINVAL; /* not both MMAP_IO and DIRECT_IO */
	return 0;
}

static inline int
sg_fetch_cmnd(struct file *filp, struct sg_fd *sfp, const u8 __user *u_cdbp,
	      int len, u8 *cdbp)
{
	if (!u_cdbp || len < 6 || len > SG_MAX_CDB_SIZE)
		return -EMSGSIZE;
	if (!access_ok(u_cdbp, len))
		return -EFAULT;
	if (__copy_from_user(cdbp, u_cdbp, len))
		return -EFAULT;
	if (O_RDWR != (filp->f_flags & O_ACCMODE)) {	/* read-only */
		switch (sfp->parentdp->device->type) {
		case TYPE_DISK:
		case TYPE_RBC:
		case TYPE_ZBC:
			return blk_verify_command(cdbp, filp->f_mode);
		default:	/* SSC, SES, etc cbd_s may differ from SBC */
			break;
		}
	}
	return 0;
}

static int
sg_v3_submit(struct file *filp, struct sg_fd *sfp, struct sg_io_hdr *hp,
	     bool sync, struct sg_request **o_srp)
{
	int res, timeout;
	unsigned long ul_timeout;
	struct sg_request *srp;
	struct sg_comm_wr_t cwr;
	u8 cmnd[SG_MAX_CDB_SIZE];

	/* now doing v3 blocking (sync) or non-blocking submission */
	if (hp->flags & SG_FLAG_MMAP_IO) {
		res = sg_chk_mmap(sfp, hp->flags, hp->dxfer_len);
		if (res)
			return res;
	}
	if (hp->flags & SGV4_FLAG_MULTIPLE_REQS)
		return -ERANGE;		/* need to use v4 interface */
	/* when v3 seen, allow cmd_q on this fd (def: no cmd_q) */
	set_bit(SG_FFD_CMD_Q, sfp->ffd_bm);
	ul_timeout = msecs_to_jiffies(hp->timeout);
	timeout = min_t(unsigned long, ul_timeout, INT_MAX);
	res = sg_fetch_cmnd(filp, sfp, hp->cmdp, hp->cmd_len, cmnd);
	if (res)
		return res;
	cwr.sync = sync;
	cwr.mrq = false;
	cwr.v4i = false;
	cwr.h3p = hp;
	cwr.timeout = timeout;
	cwr.cmnd = cmnd;
	srp = sg_common_write(sfp, &cwr);
	if (IS_ERR(srp))
		return PTR_ERR(srp);
	if (o_srp)
		*o_srp = srp;
	return 0;
}

static void
sg_sgv4_out_zero(struct sg_io_v4 *h4p)
{
	h4p->driver_status = 0;
	h4p->transport_status = 0;
	h4p->device_status = 0;
	h4p->retry_delay = 0;
	h4p->info = 0;
	h4p->response_len = 0;
	h4p->duration = 0;
	h4p->din_resid = 0;
	h4p->dout_resid = 0;
	h4p->generated_tag = 0;
	h4p->spare_out = 0;
}

/*
 * Takes a pointer to the controlling multiple request (mrq) object and a
 * pointer to the command array. The command array (with tot_reqs elements)
 * is written out (flushed) to user space pointer cv4p->din_xferp. The
 * secondary error value (s_res) is placed in the array last element's
 * spare_out field iff it was zero. Returns 0 on success.
 */
static int
sg_mrq_arr_flush(struct sg_io_v4 *cv4p, struct sg_io_v4 *arr_siv4p,
		 u32 tot_reqs, int s_res)
{
	u32 sz = tot_reqs * SZ_SG_IO_V4;
	void __user *p = uptr64(cv4p->din_xferp);

	if (s_res) {
		arr_siv4p[tot_reqs - 1].spare_out = s_res;
		cv4p->spare_out = s_res;
	}
	if (!p)
		return 0;
	if (sz > cv4p->din_xfer_len)
		sz = cv4p->din_xfer_len;
	if (sz > 0) {
		if (!access_ok(p, sz))
			return -EFAULT;
		else if (__copy_to_user(p, arr_siv4p, sz))
			return -EFAULT;
	}
	return 0;
}

static int
sg_mrq_1complet(struct sg_io_v4 *cv4p, struct sg_io_v4 *arr_siv4p,
		struct sg_fd *w_sfp, struct sg_fd *sec_sfp, int tot_reqs,
		struct sg_request *srp)
{
	int s_res, indx;
	struct sg_io_v4 *siv4p;

	SG_LOG(3, w_sfp->parentdp, "%s: start\n", __func__);
	if (!srp)
		return -EPROTO;
	indx = srp->s_hdr4.mrq_ind;
	if (indx < 0 || indx >= tot_reqs)
		return -EPROTO;
	siv4p = arr_siv4p + indx;
	s_res = sg_v4_receive(w_sfp, srp, NULL, siv4p);
	if (s_res == -EFAULT)
		return s_res;
	if (siv4p->flags & SGV4_FLAG_SIG_ON_OTHER) {
		s_res = sg_mrq_arr_flush(cv4p, arr_siv4p, tot_reqs, s_res);
		if (s_res)	/* can only be -EFAULT */
			return s_res;
		kill_fasync(&sec_sfp->async_qp, SIGPOLL, POLL_IN);
	}
	return 0;
}

/*
 * This is a fair-ish algorithm for an interruptible wait on two file
 * descriptors. It favours the main fd over the secondary fd (sec_sfp).
 */
static int
sg_mrq_complets(struct sg_io_v4 *cv4p, struct sg_io_v4 *arr_siv4p,
		struct sg_fd *sfp, struct sg_fd *sec_sfp, int tot_reqs,
		int mreqs, int sec_reqs)
{
	bool done1 = false;
	int res;
	int sum_inflight = mreqs + sec_reqs;	/* may be < tot_reqs */
	struct sg_request *srp;

	SG_LOG(3, sfp->parentdp, "%s: mreqs=%d, sec_reqs=%d\n", __func__,
	       mreqs, sec_reqs);
	for ( ; sum_inflight > 0; --sum_inflight) {
		srp = NULL;
		if (mreqs > 0 && sg_mrq_get_ready_srp(sfp, &srp)) {
			if (IS_ERR(srp))	/* -ENODATA: no mrqs here */
				mreqs = 0;
			else {
				--mreqs;
				res = sg_mrq_1complet(cv4p, arr_siv4p, sfp,
						      sec_sfp, tot_reqs, srp);
				if (res)
					return res;
			}
		} else if (sec_reqs > 0 &&
			   sg_mrq_get_ready_srp(sec_sfp, &srp)) {
			if (IS_ERR(srp))
				sec_reqs = 0;
			else {
				--sec_reqs;
				res = sg_mrq_1complet(cv4p, arr_siv4p, sec_sfp,
						      sec_sfp, tot_reqs, srp);
				if (res)
					return res;
			}
		} else if (mreqs > 0) {
			res = wait_event_interruptible(sfp->read_wait,
					sg_mrq_get_ready_srp(sfp, &srp));
			if (res)  /* -ERESTARTSYS as signal hit process */
				return res;
			if (IS_ERR(srp))
				mreqs = 0;
			else {
				--mreqs;
				res = sg_mrq_1complet(cv4p, arr_siv4p, sfp,
						      sec_sfp, tot_reqs, srp);
				if (res)
					return res;
			}
		} else if (sec_reqs > 0) {
			res = wait_event_interruptible(sfp->read_wait,
					sg_mrq_get_ready_srp(sec_sfp, &srp));
			if (res)        /* -ERESTARTSYS as signal hit process */
				return res;
			if (IS_ERR(srp))
				sec_reqs = 0;
			else {
				--sec_reqs;
				res = sg_mrq_1complet(cv4p, arr_siv4p, sec_sfp,
						      sec_sfp, tot_reqs, srp);
				if (res)
					return res;
			}
		} else	/* expect one of the above conditions to be true */
			return -EPROTO;
		/* assume successful completion if we get here */
		if (!done1) {	/* set SG_INFO_MRQ_FINI if any completions */
			done1 = true;
			cv4p->info |= SG_INFO_MRQ_FINI;
		}
		if (cv4p->din_xfer_len > 0)
			--cv4p->din_resid;
	}
	return 0;
}

/*
 * Implements the multiple request functionality. When blocking is true
 * invocation was via ioctl(SG_IO), otherwise it was via ioctl(SG_IOSUBMIT).
 * Only fully non-blocking if IMMED flag given or when ioctl(SG_IOSUBMIT)
 * is used with O_NONBLOCK set on its file descriptor.
 */
static int
sg_do_multi_req(struct file *filp, struct sg_fd *sfp, struct sg_io_v4 *cv4p,
		bool blocking)
{
	bool set_this, set_other, have_mrq_sense;
	bool non_block = !!(filp->f_flags & O_NONBLOCK);
	bool immed = !!(cv4p->flags & SGV4_FLAG_IMMED);
	bool stop_if = !!(cv4p->flags & SGV4_FLAG_STOP_IF);
	int res = 0;
	int s_res = 0;	/* for partial error, some good then ... */
	int other_fp_sent = 0;
	int this_fp_sent = 0;
	int timeout;
	const int shr_complet_b4 = SGV4_FLAG_SHARE | SGV4_FLAG_COMPLETE_B4;
	unsigned long ul_timeout;
	u32 k, n, flags;
	u32 blen = cv4p->dout_xfer_len;
	u32 tot_reqs = blen / SZ_SG_IO_V4;
	struct sg_io_v4 *siv4p;
	struct sg_io_v4 *arr_siv4p;
	struct sg_fd *o_sfp = sfp->share_sfp;
	struct sg_fd *rq_sfp;
	struct sg_request *srp;
	struct sg_device *sdp = sfp->parentdp;
	__maybe_unused const char *rip = "request index";
	struct sg_comm_wr_t cwr;
	u8 cmnd[SG_MAX_CDB_SIZE];

	if (blocking) {
		if (immed) {
			SG_LOG(1, sdp, "%s: ioctl(SG_IO) %s contradicts\n",
			       __func__, "with SGV4_FLAG_IMMED");
			return -ERANGE;
		}
		if (non_block) {
			SG_LOG(6, sdp, "%s: ioctl(SG_IO) %s O_NONBLOCK\n",
			       __func__, "ignoring");
			non_block = false;
		}
	}
	if (!immed && non_block)
		immed = true;
	SG_LOG(3, sdp, "%s: %s, tot_reqs=%u\n", __func__,
	       (immed ? "IMMED" : (blocking ?
		"ordered blocking" : "variable blocking")), tot_reqs);
	sg_sgv4_out_zero(cv4p);
	if (tot_reqs > U16_MAX)
		return -ERANGE;
	else if (blen > SG_MAX_MULTI_REQ_SZ)
		return  -E2BIG;
	else if (immed && stop_if)
		return -ERANGE;
	else if (tot_reqs == 0)
		return 0;
	else if (unlikely(atomic_read(&sdp->detaching)))
		return -ENODEV;
	else if (unlikely(o_sfp && atomic_read(&o_sfp->parentdp->detaching)))
		return -ENODEV;

	have_mrq_sense = (cv4p->response && cv4p->max_response_len);
	arr_siv4p = kcalloc(tot_reqs, SZ_SG_IO_V4, GFP_KERNEL | __GFP_NOWARN);
	if (!arr_siv4p)
		return -ENOMEM;
	n = tot_reqs * SZ_SG_IO_V4;
	if (!access_ok(cuptr64(cv4p->dout_xferp), n)) {
		res = -EFAULT;
		goto fini;
	}
	if (__copy_from_user(arr_siv4p, cuptr64(cv4p->dout_xferp), n)) {
		res = -EFAULT;
		goto fini;
	}
	/* Pre-check each request for anomalies */
	for (k = 0, siv4p = arr_siv4p; k < tot_reqs; ++k, ++siv4p) {
		flags = siv4p->flags;
		sg_sgv4_out_zero(siv4p);
		if (siv4p->guard != 'Q' || siv4p->protocol != 0 ||
		    siv4p->subprotocol != 0) {
			SG_LOG(1, sdp, "%s: req index %u: %s or protocol\n",
			       __func__, k, "bad guard");
			break;
		}
		if (flags & SGV4_FLAG_MULTIPLE_REQS) {
			SG_LOG(1, sdp, "%s: %s %u: no nested multi-reqs\n",
			       __func__, rip, k);
			break;
		}
		if (immed) {	/* only accept async submits on current fd */
			if (flags & SGV4_FLAG_DO_ON_OTHER) {
				SG_LOG(1, sdp, "%s: %s %u, %s\n", __func__,
				       rip, k, "no IMMED with ON_OTHER");
				break;
			} else if (flags & SGV4_FLAG_SHARE) {
				SG_LOG(1, sdp, "%s: %s %u, %s\n", __func__,
				       rip, k, "no IMMED with FLAG_SHARE");
				break;
			} else if (flags & SGV4_FLAG_COMPLETE_B4) {
				SG_LOG(1, sdp, "%s: %s %u, %s\n", __func__,
				       rip, k, "no IMMED with COMPLETE_B4");
				break;
			}
		}
		if (!o_sfp) {
			if (flags & SGV4_FLAG_SHARE) {
				SG_LOG(1, sdp, "%s: %s %u, no share\n",
				       __func__, rip, k);
				break;
			} else if (flags & SGV4_FLAG_SIG_ON_OTHER) {
				SG_LOG(1, sdp, "%s: %s %u, %s signal on\n",
				       __func__, rip, k, "no other fd to");
				break;
			} else if (flags & SGV4_FLAG_DO_ON_OTHER) {
				SG_LOG(1, sdp, "%s: %s %u, %s do on\n",
				       __func__, rip, k, "no other fd to");
				break;
			}
		}
		if (have_mrq_sense && siv4p->response == 0 &&
		    siv4p->max_response_len == 0) {
			siv4p->response = cv4p->response;
			siv4p->max_response_len = cv4p->max_response_len;
		}
	}
	if (k < tot_reqs) {
		res = -ERANGE;
		goto fini;
	}
	set_this = false;
	set_other = false;
	/* Dispatch requests and optionally wait for response */
	for (k = 0, siv4p = arr_siv4p; k < tot_reqs; ++k, ++siv4p) {
		flags = siv4p->flags;
		if (flags & SGV4_FLAG_DO_ON_OTHER) {
			rq_sfp = o_sfp;
			if (!set_other) {
				set_other = true;
				set_bit(SG_FFD_CMD_Q, rq_sfp->ffd_bm);
			}
		} else {
			rq_sfp = sfp;
			if (!set_this) {
				set_this = true;
				set_bit(SG_FFD_CMD_Q, rq_sfp->ffd_bm);
			}
		}
		s_res = sg_fetch_cmnd(filp, rq_sfp, cuptr64(siv4p->request),
				      siv4p->request_len, cmnd);
		if (s_res)
			break;
		ul_timeout = msecs_to_jiffies(siv4p->timeout);
		timeout = min_t(unsigned long, ul_timeout, INT_MAX);
		cwr.sync = blocking;
		cwr.mrq = true;
		cwr.v4i = true;
		cwr.h4p = siv4p;
		cwr.timeout = timeout;
		cwr.cmnd = cmnd;
		srp = sg_common_write(rq_sfp, &cwr);
		if (IS_ERR(srp)) {
			s_res = PTR_ERR(srp);
			break;
		}
		srp->s_hdr4.mrq_ind = k;
		if (immed || (!(blocking || (flags & shr_complet_b4)))) {
			if (sfp == rq_sfp)
				++this_fp_sent;
			else
				++other_fp_sent;
			continue;  /* defer completion until all submitted */
		}
		s_res = sg_wait_event_srp(filp, rq_sfp, NULL, siv4p, srp);
		if (s_res) {
			if (s_res == -ERESTARTSYS) {
				res = s_res;
				goto fini;
			}
			break;
		}
		if (!srp) {
			s_res = -EPROTO;
			break;
		}
		if (stop_if && (siv4p->driver_status ||
			siv4p->transport_status || siv4p->device_status)) {
			SG_LOG(2, sdp, "%s: %s=0x%x/0x%x/0x%x] cause exit\n",
			       __func__, "STOP_IF and status [drv/tran/scsi",
			       siv4p->driver_status, siv4p->transport_status,
			       siv4p->device_status);
			break;	/* cv4p::driver_status <-- 0 in this case */
		}
		if (siv4p->flags & SGV4_FLAG_SIG_ON_OTHER) {
			res = sg_mrq_arr_flush(cv4p, arr_siv4p, tot_reqs,
					       s_res);
			if (res)
				break;
			kill_fasync(&o_sfp->async_qp, SIGPOLL, POLL_IN);
		}
	}	/* end of dispatch request and optionally wait loop */
	cv4p->dout_resid = tot_reqs - k;
	if (cv4p->din_xfer_len > 0)
		cv4p->din_resid = cv4p->din_xfer_len / SZ_SG_IO_V4;

	if (immed)
		goto fini;

	if (res == 0 && (this_fp_sent + other_fp_sent) > 0) {
		s_res = sg_mrq_complets(cv4p, arr_siv4p, sfp, o_sfp, tot_reqs,
					this_fp_sent, other_fp_sent);
		if (s_res == -EFAULT || s_res == -ERESTARTSYS)
			res = s_res;	/* this may leave orphans */
	}
fini:
	if (res == 0 && !immed)
		res = sg_mrq_arr_flush(cv4p, arr_siv4p, tot_reqs, s_res);
	kfree(arr_siv4p);
	return res;
}

static int
sg_v4_submit(struct file *filp, struct sg_fd *sfp, void __user *p,
		struct sg_io_v4 *h4p, bool sync, struct sg_request **o_srp)
{
	int timeout, res;
	unsigned long ul_timeout;
	struct sg_request *srp;
	struct sg_comm_wr_t cwr;
	u8 cmnd[SG_MAX_CDB_SIZE];

	if (h4p->flags & SGV4_FLAG_MULTIPLE_REQS) {
		/* want v4 async or sync with guard, din and dout and flags */
		if (h4p->request || h4p->request_len || !h4p->dout_xferp ||
		    h4p->din_iovec_count || h4p->dout_iovec_count ||
		    (h4p->dout_xfer_len % SZ_SG_IO_V4))
			return -ERANGE;
		if (o_srp)
			*o_srp = NULL;
		res = sg_do_multi_req(filp, sfp, h4p, sync);
		if (res)
			return res;
		if (p) {
			if (!access_ok(p, SZ_SG_IO_V4))
				return -EFAULT;
			/* Write back sg_io_v4 object for error/warning info */
			if (__copy_to_user(p, h4p, SZ_SG_IO_V4))
				return -EFAULT;
		}
		return 0;
	}
	/* once v4 (or v3) seen, allow cmd_q on this fd (def: no cmd_q) */
	set_bit(SG_FFD_CMD_Q, sfp->ffd_bm);
	ul_timeout = msecs_to_jiffies(h4p->timeout);
	timeout = min_t(unsigned long, ul_timeout, INT_MAX);
	res = sg_fetch_cmnd(filp, sfp, cuptr64(h4p->request), h4p->request_len,
			    cmnd);
	if (res)
		return res;
	cwr.sync = sync;
	cwr.mrq = false;
	cwr.v4i = true;
	cwr.h4p = h4p;
	cwr.timeout = timeout;
	cwr.cmnd = cmnd;
	srp = sg_common_write(sfp, &cwr);
	if (IS_ERR(srp))
		return PTR_ERR(srp);
	if (o_srp)
		*o_srp = srp;
	if (p && !sync && (srp->rq_flags & SGV4_FLAG_YIELD_TAG)) {
		struct sg_io_v4 __user *h4_up = (struct sg_io_v4 __user *)p;

		res = put_user(srp->tag, &h4_up->request_tag);
	}
	return res;
}

static int
sg_ctl_iosubmit(struct file *filp, struct sg_fd *sfp, void __user *p)
{
	int len;
	u8 hdr_store[SZ_SG_IO_V4];
	struct sg_io_v4 *h4p = (struct sg_io_v4 *)hdr_store;

	if (!access_ok(p, SZ_SG_IO_HDR))
		return -EFAULT;
	else if (!access_ok(p, SZ_SG_IO_V4))
		len = SZ_SG_IO_HDR;
	else
		len = SZ_SG_IO_V4;
	if (__copy_from_user(hdr_store, p, len))
		return -EFAULT;
	SG_LOG(3, sfp->parentdp, "%s:  probably %s\n", __func__,
	       (h4p->guard == 'Q' ? "v4" : "v3"));
	if (h4p->guard == 'S') {
		struct sg_io_hdr *h3p = (struct sg_io_hdr *)hdr_store;

		return sg_v3_submit(filp, sfp, h3p, false, NULL);
	}
	if (h4p->guard == 'Q') {
		if (len < SZ_SG_IO_V4)
			return -EPERM;
		return sg_v4_submit(filp, sfp, p, h4p, false, NULL);
	}
	return -EPERM;
}

/*
 * Assumes sharing has been established at the file descriptor level and now we
 * check the rq_flags of a new request/command. SGV4_FLAG_NO_DXFER may or may
 * not be used on the master side, it must be used on the slave side. Also
 * returns (via *sh_varp) the proposed sg_request::sh_var of the new request
 * yet to be built/re-used.
 */
static int
sg_share_chk_flags(struct sg_fd *sfp, u32 rq_flags, int dxfer_len, int dir,
		   enum sg_shr_var *sh_varp)
{
	bool is_master = (sfp->shr_fd == SG_SHARE_FD_MASTER);
	int result = 0;
	enum sg_shr_var sh_var = SG_SHR_NONE;

	if (rq_flags & SGV4_FLAG_SHARE) {
		if (rq_flags & SG_FLAG_DIRECT_IO)
			result = -EINVAL; /* since no control of data buffer */
		else if (dxfer_len < 1)
			result = -ENODATA;
		else if (is_master) {		/* fd is reader */
			sh_var = SG_SHR_MA_RQ;
			if (dir != SG_DXFER_FROM_DEV)
				result = -ENOMSG;
			if (rq_flags & SGV4_FLAG_NO_DXFER) {
				/* rule out some contradictions */
				if (rq_flags & SG_FL_MMAP_DIRECT)
					result = -ENODATA;
			}
		} else {			/* fd is slave, writer */
			sh_var = SG_SHR_SL_RQ;
			if (dir != SG_DXFER_TO_DEV)
				result = -ENOMSG;
			if (!(rq_flags & SGV4_FLAG_NO_DXFER))
				result = -ENOMSG;
			if (rq_flags & SG_FL_MMAP_DIRECT)
				result = -ENODATA;
		}
	} else if (is_master)
		sh_var = SG_SHR_MA_FD_NOT_RQ;
	else
		sh_var = SG_SHR_SL_FD_NOT_RQ;
	*sh_varp = sh_var;
	return result;
}

static void
sg_execute_cmd(struct sg_fd *sfp, struct sg_request *srp)
{
	bool at_head, is_v4h, sync;
	int res;
	unsigned long iflags;
	struct sg_device *sdp = sfp->parentdp;

	if (unlikely(srp->sh_var == SG_SHR_SL_RQ && sfp->share_sfp)) {
		if (test_and_clear_bit(SG_FRQ_DEFER_XFR, srp->frq_bm)) {
			res = sg_rq_map_kern(srp, sdp->device->request_queue,
					     srp->rq);
			SG_LOG(3, sdp, "%s: deferred xfer\n", __func__);
			if (res)
				pr_warn("%s: sg_rq_map_kern() --> %d\n",
					__func__, res);
		}
	}
	is_v4h = test_bit(SG_FRQ_IS_V4I, srp->frq_bm);
	sync = test_bit(SG_FRQ_SYNC_INVOC, srp->frq_bm);
	SG_LOG(3, sdp, "%s: is_v4h=%d\n", __func__, (int)is_v4h);
	if (test_bit(SG_FFD_TIME_IN_NS, sfp->ffd_bm))
		srp->start_kt = ktime_get_with_offset(TK_OFFS_BOOT);
	else
		srp->start_ms = jiffies_to_msecs(jiffies);
	srp->duration = 0;

	if (!is_v4h && srp->s_hdr3.interface_id == '\0')
		at_head = true;	/* backward compatibility: v1+v2 interfaces */
	else if (test_bit(SG_FFD_Q_AT_TAIL, sfp->ffd_bm))
		/* cmd flags can override sfd setting */
		at_head = !!(srp->rq_flags & SG_FLAG_Q_AT_HEAD);
	else            /* this sfd is defaulting to head */
		at_head = !(srp->rq_flags & SG_FLAG_Q_AT_TAIL);

	kref_get(&sfp->f_ref); /* sg_rq_end_io() does kref_put(). */

	/* >>>>>>> send cmd/req off to other levels <<<<<<<< */
	if (!sync)
		atomic_inc(&sfp->submitted);
	blk_execute_rq_nowait(sdp->device->request_queue, sdp->disk,
			      srp->rq, (int)at_head, sg_rq_end_io);

	/* Should we try to find the req's tag? Only pay if answer is yes */
	if (is_v4h && (srp->rq_flags & SGV4_FLAG_YIELD_TAG)) {
		spin_lock_irqsave(&srp->req_lck, iflags);
		{		/* we might be too late or too early! */
			struct request *rq = srp->rq;

			srp->tag = rq ? rq->tag : SG_TAG_WILDCARD;
		}
		spin_unlock_irqrestore(&srp->req_lck, iflags);
	}
}

static inline int
sg_rstate_chg(struct sg_request *srp, enum sg_rq_state old_st,
	      enum sg_rq_state new_st)
{
	enum sg_rq_state act_old_st = (enum sg_rq_state)
				atomic_cmpxchg(&srp->rq_st, old_st, new_st);

	if (act_old_st == old_st)
		return 0;	/* implies new_st --> srp->rq_st */
	else if (IS_ENABLED(CONFIG_SCSI_LOGGING))
		sg_rep_rq_state_fail(srp->parentfp->parentdp, old_st, new_st,
				     act_old_st);
	return -EPROTOTYPE;
}

/*
 * Here when share slave request is being built and SG_FRQ_DEFER_XFR is set.
 * Returns 0 for SWAIT (i.e. deferred execution), SZ_SG_IO_V4 (positive) for
 * execute now, and negated errno value for error.
 */
static int
sg_process_swait(struct sg_fd *sl_sfp, struct sg_request *sl_srp)
{
	int res = 0;
	unsigned long iflags;
	enum sg_rq_state ma_sr_st;
	struct sg_fd *ma_sfp = sl_sfp->share_sfp;
	struct sg_request *ma_rsv_srp;

	if (!ma_sfp || !ma_sfp->rsv_srp)
		return -EPROTO;
	ma_rsv_srp = ma_sfp->rsv_srp;
	spin_lock_irqsave(&ma_rsv_srp->req_lck, iflags);
	ma_sr_st = atomic_read(&ma_rsv_srp->rq_st);
	SG_LOG(3, sl_sfp->parentdp, "%s: ma_rsv_srp=0x%p, ma_sr_st: %s\n",
	       __func__, ma_rsv_srp, sg_rq_st_str(ma_sr_st, false));

	switch (ma_sr_st) {	/* master's state held constant by spinlock */
	case SG_RS_INFLIGHT:
		res = sg_rstate_chg(sl_srp, SG_RS_INFLIGHT, SG_RS_SHR_SWAIT);
		goto fini;	/* if that worked res=0 and slave now SWAIT */
	case SG_RS_AWAIT_RD:
	case SG_RS_DONE_RD:
	case SG_RS_SHR_SWAP:
		res = (ma_rsv_srp->rq_result & SG_ML_RESULT_MSK) ?
				-ENOSTR : SZ_SG_IO_V4;
		break;
	case SG_RS_BUSY:
		res = -EBUSY;
		break;
	case SG_RS_INACTIVE:
		res = -EAGAIN;
		break;
	case SG_RS_SHR_SLAVE:
		res = -EDOM;
		break;
	default:
		res = -EPROTO;
		break;
	}
fini:
	spin_unlock_irqrestore(&ma_rsv_srp->req_lck, iflags);
	return res;
}

/*
 * All writes and submits converge on this function to launch the SCSI
 * command/request (via blk_execute_rq_nowait). Returns a pointer to a
 * sg_request object holding the request just issued or a negated errno
 * value twisted by ERR_PTR.
 * N.B. pack_id placed in sg_io_v4::request_extra field.
 */
static struct sg_request *
sg_common_write(struct sg_fd *sfp, struct sg_comm_wr_t *cwrp)
{
	bool prev_v4_active;
	int res = 0;
	int dxfr_len, dir, cmd_len;
	int pack_id = SG_PACK_ID_WILDCARD;
	u32 rq_flags;
	enum sg_shr_var sh_var = SG_SHR_NONE;
	struct sg_device *sdp = sfp->parentdp;
	struct sg_request *srp;
	struct sg_io_hdr *hi_p = cwrp->v4i ? NULL : cwrp->h3p;
	struct sg_io_v4 *h4p = cwrp->v4i ? cwrp->h4p : NULL;

	if (h4p) {
		dxfr_len = 0;
		dir = SG_DXFER_NONE;
		rq_flags = h4p->flags;
		pack_id = h4p->request_extra;
		if (h4p->din_xfer_len && h4p->dout_xfer_len) {
			return ERR_PTR(-EOPNOTSUPP);
		} else if (h4p->din_xfer_len) {
			dxfr_len = h4p->din_xfer_len;
			dir = SG_DXFER_FROM_DEV;
		} else if (h4p->dout_xfer_len) {
			dxfr_len = h4p->dout_xfer_len;
			dir = SG_DXFER_TO_DEV;
		}
		/* Only allow SGV4_FLAG_DO_ON_OTHER on mrq */
		if (!cwrp->mrq && (rq_flags & SGV4_FLAG_DO_ON_OTHER))
			return ERR_PTR(-ERANGE);
	} else {		/* sg v3 interface so hi_p valid */
		dir = hi_p->dxfer_direction;
		dxfr_len = hi_p->dxfer_len;
		rq_flags = hi_p->flags;
		pack_id = hi_p->pack_id;
	}
	if (rq_flags & SGV4_FLAG_MULTIPLE_REQS)
		return ERR_PTR(-ERANGE);
	if (sfp->shr_fd == SG_SHARE_FD_UNUSED) {/* no sharing on this fd */
		if (rq_flags & SGV4_FLAG_SHARE)
			return ERR_PTR(-ENOMSG);
	} else {
		res = sg_share_chk_flags(sfp, rq_flags, dxfr_len, dir,
					 &sh_var);
		if (res < 0)
			return ERR_PTR(res);
	}
	if (dxfr_len >= SZ_256M)
		return ERR_PTR(-EINVAL);

	srp = sg_add_request(sfp, dxfr_len, cwrp->sync, sh_var);
	if (IS_ERR(srp))
		return srp;
	srp->rq_flags = rq_flags;
	srp->pack_id = pack_id;
	prev_v4_active = test_bit(SG_FRQ_IS_V4I, srp->frq_bm);
	if (prev_v4_active) {
		if (hi_p)
			clear_bit(SG_FRQ_IS_V4I, srp->frq_bm);
	} else if (h4p)
		set_bit(SG_FRQ_IS_V4I, srp->frq_bm);

	if (h4p) {
		memset(&srp->s_hdr4, 0, sizeof(srp->s_hdr4));
		srp->s_hdr4.usr_ptr = h4p->usr_ptr;
		srp->s_hdr4.sbp = uptr64(h4p->response);
		srp->s_hdr4.max_sb_len = h4p->max_response_len;
		srp->s_hdr4.cmd_len = h4p->request_len;
		srp->s_hdr4.dir = dir;
		cmd_len = h4p->request_len;
	} else {	/* v3 interface active */
		cmd_len = hi_p->cmd_len;
		memcpy(&srp->s_hdr3, hi_p, sizeof(srp->s_hdr3));
	}
	srp->cmd_opcode = cwrp->cmnd[0];/* hold opcode of command for debug */
	SG_LOG(4, sdp, "%s: opcode=0x%02x, cdb_sz=%d, pack_id=%d\n", __func__,
	       (int)cwrp->cmnd[0], cmd_len, pack_id);

	res = sg_start_req(srp, cwrp->cmnd, cmd_len, h4p, dir);
	if (res < 0)		/* probably out of space --> -ENOMEM */
		goto err_out;
	SG_TRC(__func__, "after sg_start_req", SG_TRC_REQ, 1, srp, true, true);
	if (unlikely(atomic_read(&sdp->detaching))) {
		res = -ENODEV;
		goto err_out;
	}
	if (srp->rq)
		srp->rq->timeout = cwrp->timeout;
	else {
		res = -EIDRM;	/* this failure unexpected but observed */
		goto err_out;
	}

	if (sh_var == SG_SHR_SL_RQ &&
	    test_bit(SG_FRQ_DEFER_XFR, srp->frq_bm)) {
		res = sg_process_swait(sfp, srp);
		if (!res)
			return srp;
		else if (res < 0)
			goto err_out;
		/* fallthrough when res is SZ_SG_IO_V4 to execute now */
	}
	sg_execute_cmd(sfp, srp);
	return srp;
err_out:
	sg_finish_scsi_blk_rq(srp);
	sg_deact_request(sfp, srp);
	return ERR_PTR(res);
}

/*
 * read(2) related functions follow. They are shown after write(2) related
 * functions. Apart from read(2) itself, ioctl(SG_IORECEIVE) and the second
 * half of the ioctl(SG_IO) share code with read(2).
 */

/*
 * This function is called by wait_event_interruptible in sg_read() and
 * sg_ctl_ioreceive(). wait_event_interruptible will return if this one
 * returns true (or an event like a signal (e.g. control-C) occurs).
 */
static inline bool
sg_get_ready_srp(struct sg_fd *sfp, struct sg_request **srpp, int pack_id,
		 int tag, bool use_tag)
{
	struct sg_request *srp;

	if (unlikely(atomic_read(&sfp->parentdp->detaching))) {
		*srpp = NULL;
		return true;
	}
	srp = sg_get_rq_pack_id(sfp, pack_id, tag, use_tag);
	*srpp = srp;
	return !!srp;
}

/*
 * Returns number of bytes copied to user space provided sense buffer or
 * negated errno value.
 */
static int
sg_copy_sense(struct sg_request *srp, bool v4_active)
{
	int sb_len_wr = 0;
	int scsi_stat;

	/* If need be, copy the sense buffer to the user space */
	scsi_stat = srp->rq_result & 0xff;
	if ((scsi_stat & SAM_STAT_CHECK_CONDITION) ||
	    (driver_byte(srp->rq_result) & DRIVER_SENSE)) {
		int sb_len = min_t(int, SCSI_SENSE_BUFFERSIZE, srp->sense_len);
		int mx_sb_len;
		void __user *up;

		if (v4_active) {
			up = uptr64(srp->s_hdr4.sbp);
			mx_sb_len = srp->s_hdr4.max_sb_len;
		} else {
			up = (void __user *)srp->s_hdr3.sbp;
			mx_sb_len = srp->s_hdr3.mx_sb_len;
		}
		if (up && mx_sb_len > 0 && srp->sense_bp) {
			sb_len = min_t(int, sb_len, mx_sb_len);
			/* Additional sense length field */
			sb_len_wr = 8 + (int)srp->sense_bp[7];
			sb_len_wr = min_t(int, sb_len, sb_len_wr);
			if (copy_to_user(up, srp->sense_bp, sb_len_wr))
				sb_len_wr = -EFAULT;
		}
		if (srp->sense_bp) {
			mempool_free(srp->sense_bp, sg_sense_pool);
			srp->sense_bp = NULL;
		}
	}
	return sb_len_wr;
}

#if IS_ENABLED(CONFIG_SCSI_LOGGING)
static void
sg_rep_rq_state_fail(struct sg_device *sdp, enum sg_rq_state exp_old_st,
		     enum sg_rq_state want_st, enum sg_rq_state act_old_st)
{
	const char *eors = "expected old rq_st: ";
	const char *aors = "actual old rq_st: ";

	if (IS_ENABLED(CONFIG_SCSI_PROC_FS))
		SG_LOG(1, sdp, "%s: %s%s, %s%s, wanted rq_st: %s\n", __func__,
		       eors, sg_rq_st_str(exp_old_st, false),
		       aors, sg_rq_st_str(act_old_st, false),
		       sg_rq_st_str(want_st, false));
	else
		pr_info("sg: %s: %s%d, %s%d, wanted rq_st: %d\n", __func__,
			eors, (int)exp_old_st, aors, (int)act_old_st,
			(int)want_st);
}
#else
static void
sg_rep_rq_state_fail(struct sg_device *sdp, enum sg_rq_state exp_old_st,
		     enum sg_rq_state want_st, enum sg_rq_state act_old_st)
{
}
#endif

static int
sg_rec_v3v4_state(struct sg_fd *sfp, struct sg_request *srp, bool v4_active)
{
	int err = 0;
	int sb_len_wr;
	u32 rq_res = srp->rq_result;
	enum sg_shr_var sh_var = srp->sh_var;

	sb_len_wr = sg_copy_sense(srp, v4_active);
	if (sb_len_wr < 0)
		return sb_len_wr;
	if (rq_res & SG_ML_RESULT_MSK)
		srp->rq_info |= SG_INFO_CHECK;
	if (test_bit(SG_FRQ_ABORTING, srp->frq_bm))
		srp->rq_info |= SG_INFO_ABORTED;

	if (sh_var == SG_SHR_SL_RQ && sfp->share_sfp) {
		struct sg_request *ma_srp = sfp->share_sfp->rsv_srp;
		enum sg_rq_state mar_st = atomic_read(&ma_srp->rq_st);

		switch (mar_st) {
		case SG_RS_SHR_SLAVE:
			/* already on master's free list, make re-usable */
			ma_srp->tag = SG_TAG_WILDCARD;
			ma_srp->sh_var = SG_SHR_NONE;
			err = sg_rstate_chg(ma_srp, mar_st, SG_RS_INACTIVE);
			break;
		case SG_RS_SHR_SWAP:
		case SG_RS_INACTIVE:
		case SG_RS_AWAIT_RD:
			sfp->share_sfp->slave_srp = NULL;
			break;	/* nothing to do */
		default:
			err = -EPROTO;	/* Logic error */
			SG_LOG(1, sfp->parentdp,
			       "%s: SHR_SL_ACTIVE, bad master state: %s\n",
			       __func__, sg_rq_st_str(mar_st, true));
			break;	/* nothing to do */
		}
	}
	if (unlikely(atomic_read(&sfp->parentdp->detaching)))
		srp->rq_info |= SG_INFO_DEVICE_DETACHING;
	return err;
}

static int
sg_v3v4_complete(struct sg_fd *sfp, struct sg_request *srp, bool other_err)
{
	int err = 0;
	enum sg_rq_state sr_st = atomic_read(&srp->rq_st);

	/* advance state machine, send signal to slave if appropriate */
	switch (srp->sh_var) {
	case SG_SHR_MA_RQ:
		{
			int poll_type = POLL_OUT;

			if ((srp->rq_result & SG_ML_RESULT_MSK) || other_err) {
				set_bit(SG_FFD_MASTER_ERR, sfp->ffd_bm);
				err = sg_rstate_chg(srp, sr_st, SG_RS_DONE_RD);
				poll_type = POLL_HUP;	/* "Hang-UP flag */
			} else if (sr_st != SG_RS_SHR_SWAP) {
				err = sg_rstate_chg(srp, sr_st,
						    SG_RS_SHR_SWAP);
			}
			if (sfp->share_sfp)
				kill_fasync(&sfp->share_sfp->async_qp,
					    SIGPOLL, poll_type);
		}
		break;
	case SG_SHR_SL_RQ:	/* cleanup both on slave completion */
		{
			struct sg_fd *ma_sfp = sfp->share_sfp;

			if (ma_sfp) {
				ma_sfp->slave_srp = NULL;
				if (ma_sfp->rsv_srp)
					ma_sfp->rsv_srp->sh_var =
							 SG_SHR_MA_FD_NOT_RQ;
			}
		}
		srp->sh_var = SG_SHR_SL_FD_NOT_RQ;
		srp->sgatp = &srp->sgat_h;
		err = sg_rstate_chg(srp, sr_st, SG_RS_DONE_RD);
		break;
	default:
		err = sg_rstate_chg(srp, sr_st, SG_RS_DONE_RD);
		break;
	}
	return err;
}

static int
sg_v4_receive(struct sg_fd *sfp, struct sg_request *srp, void __user *p,
	      struct sg_io_v4 *h4p)
{
	int err, err2;
	u32 rq_result = srp->rq_result;

	SG_LOG(3, sfp->parentdp, "%s: p=%s, h4p=%s\n", __func__,
	       (p ? "given" : "NULL"), (h4p ? "given" : "NULL"));
	err = sg_rec_v3v4_state(sfp, srp, true);
	h4p->guard = 'Q';
	h4p->protocol = 0;
	h4p->subprotocol = 0;
	h4p->device_status = rq_result & 0xff;
	h4p->driver_status = driver_byte(rq_result);
	h4p->transport_status = host_byte(rq_result);
	h4p->response_len = srp->sense_len;
	h4p->info = srp->rq_info;
	h4p->flags = srp->rq_flags;
	h4p->duration = srp->duration;
	switch (srp->s_hdr4.dir) {
	case SG_DXFER_FROM_DEV:
		h4p->din_xfer_len = srp->sgatp->dlen;
		break;
	case SG_DXFER_TO_DEV:
		h4p->dout_xfer_len = srp->sgatp->dlen;
		break;
	default:
		break;
	}
	h4p->din_resid = srp->in_resid;
	h4p->dout_resid = srp->s_hdr4.out_resid;
	h4p->usr_ptr = srp->s_hdr4.usr_ptr;
	h4p->response = (u64)srp->s_hdr4.sbp;
	h4p->request_extra = srp->pack_id;
	if (p) {
		if (copy_to_user(p, h4p, SZ_SG_IO_V4))
			err = err ? err : -EFAULT;
	}
	err2 = sg_v3v4_complete(sfp, srp, err < 0);
	if (err2)
		err = err ? err : err2;
	sg_finish_scsi_blk_rq(srp);
	sg_deact_request(sfp, srp);
	return err < 0 ? err : 0;
}

/*
 * Returns negative on error including -ENODATA if there are no mrqs submitted
 * nor waiting. Otherwise it returns the number of elements written to
 * rsp_arr, which may be 0 if mrqs submitted but none waiting
 */
static int
sg_mrq_iorec_complets(struct sg_fd *sfp, bool non_block, int max_mrqs,
		      struct sg_io_v4 *rsp_arr)
{
	int k;
	int res = 0;
	struct sg_request *srp;

	SG_LOG(3, sfp->parentdp, "%s: max_mrqs=%d\n", __func__, max_mrqs);
	for (k = 0; k < max_mrqs; ++k) {
		if (!sg_mrq_get_ready_srp(sfp, &srp))
			break;
		if (!srp)
			return -EPROTO;
		if (IS_ERR(srp))
			return k ? k : PTR_ERR(srp);
		res = sg_v4_receive(sfp, srp, NULL, rsp_arr + k);
		if (res)
			return res;
		rsp_arr[k].info |= SG_INFO_MRQ_FINI;
	}
	if (non_block)
		return k;

	for ( ; k < max_mrqs; ++k) {
		res = wait_event_interruptible(sfp->read_wait,
				sg_mrq_get_ready_srp(sfp, &srp));
		if (res)        /* -ERESTARTSYS as signal hit process */
			return res;
		if (!srp)
			return -EPROTO;
		if (IS_ERR(srp))
			return k ? k : PTR_ERR(srp);
		res = sg_v4_receive(sfp, srp, NULL, rsp_arr + k);
		if (res)
			return res;
		rsp_arr[k].info |= SG_INFO_MRQ_FINI;
	}
	return k;
}

static int
sg_mrq_ioreceive(struct sg_fd *sfp, void __user *p, bool non_block)
{
	int res = 0;
	u32 len, n;
	struct sg_io_v4 *cv4p;
	struct sg_io_v4 *rsp_v4_arr;
	struct sg_io_v4 cv4;
	void __user *pp;

	SG_LOG(3, sfp->parentdp, "%s: non_block=%d\n", __func__, !!non_block);
	cv4p = &cv4;
	memset(cv4p, 0, sizeof(*cv4p));
	if (!access_ok(p, sizeof(*cv4p)))
		return -EFAULT;
	if (__copy_from_user(cv4p, p, sizeof(*cv4p)))
		return -EFAULT;
	n = cv4p->din_xfer_len;
	if (n > SG_MAX_MULTI_REQ_SZ)
		return -E2BIG;
	if (!cv4p->din_xferp || n < SZ_SG_IO_V4 || (n % SZ_SG_IO_V4))
		return -ERANGE;
	n /= SZ_SG_IO_V4;
	len = n * SZ_SG_IO_V4;
	SG_LOG(3, sfp->parentdp, "%s: %s, num_reqs=%u\n", __func__,
	       (non_block ? "IMMED" : "blocking"), n);
	rsp_v4_arr = kcalloc(n, SZ_SG_IO_V4, GFP_KERNEL);
	if (!rsp_v4_arr)
		return -ENOMEM;

	// sg_sgv4_out_zero(cv4p);
	res = sg_mrq_iorec_complets(sfp, non_block, n, rsp_v4_arr);
	if (res < 0)
		goto fini;
	cv4p->info = res;
	res = 0;
	pp = uptr64(cv4p->din_xferp);
	if (pp) {
		if (!access_ok(pp, len))
			res = -EFAULT;
		else if (__copy_to_user(pp, rsp_v4_arr, len))
			res = -EFAULT;
	} else
		pr_info("%s: cv4p->din_xferp==NULL ??\n", __func__);
fini:
	kfree(rsp_v4_arr);
	return res;
}

/*
 * Called when ioctl(SG_IORECEIVE) received. Expects a v3 or v4 interface.
 * Checks if O_NONBLOCK file flag given, if not checks given v3_v4::flags
 * to see if SGV4_FLAG_IMMED is set. Either of these implies non blocking.
 * When non-blocking and there is no request waiting, yields EAGAIN;
 * otherwise it waits.
 */
static int
sg_ctl_ioreceive(struct file *filp, struct sg_fd *sfp, void __user *p)
{
	bool non_block;
	bool use_tag = false;
	bool v4_active = false;
	int res;
	int pack_id = SG_PACK_ID_WILDCARD;
	int tag = SG_TAG_WILDCARD;
	int my_flags = 0;
	int ret = 0;
	u8 v4_holder[SZ_SG_IO_V4];
	struct sg_io_v4 *h4p = (struct sg_io_v4 *)v4_holder;
	struct sg_device *sdp = sfp->parentdp;
	struct sg_io_v4 __user *h4_up = NULL;
	struct sg_io_hdr __user *h3_up = NULL;
	struct sg_request *srp;

	if (!sdp)
		return -EPROTO;
	if (unlikely(atomic_read(&sdp->detaching)))
		return -ENODEV;
	non_block = !!(filp->f_flags & O_NONBLOCK);
	if (!p)
		return -EPROTO;
	if (!access_ok(p, 3 * sizeof(int)))
		return -EFAULT;
	/* Get first three 32 bit integers: guard, proto+subproto */
	if (__copy_from_user(h4p, p, 3 * sizeof(int)))
		return -EFAULT;
	/* for v4: protocol=0 --> SCSI;  subprotocol=0 --> SPC++ */
	if (h4p->guard == 'Q') {
		if (h4p->protocol != 0 || h4p->subprotocol != 0)
			return -EPERM;
		h4_up = (struct  __user sg_io_v4 *)p;
		ret = get_user(my_flags, &h4_up->flags);
		v4_active = true;
	} else if (h4p->guard == 'S') {
		h3_up = (struct  __user sg_io_hdr *)p;
		ret = get_user(my_flags, &h3_up->flags);
	} else {
		pr_info_once("sg: %s: v3 or v4 interface only here\n",
			     __func__);
		return -EPERM;
	}
	if (ret)
		return ret;
	if (my_flags & SGV4_FLAG_IMMED)
		non_block = true;	/* set by either this or O_NONBLOCK */
	SG_LOG(3, sdp, "%s: non_block(+IMMED)=%d\n", __func__, non_block);
	if (my_flags & SGV4_FLAG_MULTIPLE_REQS) {
		if (!v4_active)
			return -EINVAL;
		return sg_mrq_ioreceive(sfp, p, non_block);
	}
	/* read in part of v3 or v4 header for pack_id or tag based find */
	if (test_bit(SG_FFD_FORCE_PACKID, sfp->ffd_bm)) {
		use_tag = test_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm);
		ret = 0;
		if (v4_active && use_tag)
			ret = get_user(tag, &h4_up->request_tag);
		else if (v4_active && !use_tag)
			ret = get_user(pack_id, &h4_up->request_extra);
		else if (!use_tag)
			ret = get_user(pack_id, &h3_up->pack_id);
		if (ret)
			return ret;
	}

	srp = sg_get_rq_pack_id(sfp, pack_id, tag, use_tag);
	if (!srp) {     /* nothing available so wait on packet or */
		if (unlikely(atomic_read(&sdp->detaching)))
			return -ENODEV;
		if (non_block)
			return -EAGAIN;
		res = wait_event_interruptible(sfp->read_wait,
				sg_get_ready_srp(sfp, &srp, pack_id, tag,
						 use_tag));
		if (unlikely(atomic_read(&sdp->detaching)))
			return -ENODEV;
		if (res)	/* -ERESTARTSYS as signal hit process */
			return res;
	}	/* now srp should be valid */
	if (test_bit(SG_FRQ_IS_V4I, srp->frq_bm))
		return sg_v4_receive(sfp, srp, p, h4p);
	return sg_v3_receive(sfp, srp, p);
}

static int
sg_rd_v1v2(void __user *buf, int count, struct sg_fd *sfp,
	   struct sg_request *srp)
{
	int res = 0;
	u32 rq_result = srp->rq_result;
	struct sg_header *h2p;
	struct sg_slice_hdr3 *sh3p;
	struct sg_header a_v2hdr;

	h2p = &a_v2hdr;
	memset(h2p, 0, SZ_SG_HEADER);
	sh3p = &srp->s_hdr3;
	h2p->reply_len = (int)sh3p->timeout;
	h2p->pack_len = h2p->reply_len; /* old, strange behaviour */
	h2p->pack_id = sh3p->pack_id;
	h2p->twelve_byte = (srp->cmd_opcode >= 0xc0 && sh3p->cmd_len == 12);
	h2p->target_status = status_byte(rq_result);
	h2p->host_status = host_byte(rq_result);
	h2p->driver_status = driver_byte(rq_result);
	if ((CHECK_CONDITION & status_byte(rq_result)) ||
	    (DRIVER_SENSE & driver_byte(rq_result))) {
		if (srp->sense_bp) {
			memcpy(h2p->sense_buffer, srp->sense_bp,
			       sizeof(h2p->sense_buffer));
			mempool_free(srp->sense_bp, sg_sense_pool);
			srp->sense_bp = NULL;
		}
	}
	switch (host_byte(rq_result)) {
	/*
	 * This foolowing setting of 'result' is for backward compatibility
	 * and is best ignored by the user who should use target, host and
	 * driver status.
	 */
	case DID_OK:
	case DID_PASSTHROUGH:
	case DID_SOFT_ERROR:
		h2p->result = 0;
		break;
	case DID_NO_CONNECT:
	case DID_BUS_BUSY:
	case DID_TIME_OUT:
		h2p->result = EBUSY;
		break;
	case DID_BAD_TARGET:
	case DID_ABORT:
	case DID_PARITY:
	case DID_RESET:
	case DID_BAD_INTR:
		h2p->result = EIO;
		break;
	case DID_ERROR:
		h2p->result = (srp->sense_bp &&
				status_byte(rq_result) == GOOD) ? 0 : EIO;
		break;
	default:
		h2p->result = EIO;
		break;
	}

	/* Now copy the result back to the user buffer.  */
	if (count >= SZ_SG_HEADER) {
		if (!access_ok(buf, count))
			return -EFAULT;
		if (__copy_to_user(buf, h2p, SZ_SG_HEADER))
			return -EFAULT;
		buf += SZ_SG_HEADER;
		if (count > h2p->reply_len)
			count = h2p->reply_len;
		if (count > SZ_SG_HEADER) {
			if (sg_rd_append(srp, buf, count - SZ_SG_HEADER))
				return -EFAULT;
		}
	} else
		res = (h2p->result == 0) ? 0 : -EIO;
	atomic_set(&srp->rq_st, SG_RS_DONE_RD);
	sg_finish_scsi_blk_rq(srp);
	sg_deact_request(sfp, srp);
	return res;
}

/*
 * This is the read(2) system call entry point (see sg_fops) for this driver.
 * Accepts v1, v2 or v3 type headers (not v4). Returns count or negated
 * errno; if count is 0 then v3: returns -EINVAL; v1+v2: 0 when no other
 * error detected or -EIO.
 */
static ssize_t
sg_read(struct file *filp, char __user *p, size_t count, loff_t *ppos)
{
	bool could_be_v3;
	bool non_block = !!(filp->f_flags & O_NONBLOCK);
	int req_pack_id = SG_PACK_ID_WILDCARD;
	int hlen, ret;
	struct sg_device *sdp = NULL;
	struct sg_fd *sfp;
	struct sg_request *srp = NULL;
	struct sg_header *h2p = NULL;
	struct sg_io_hdr a_sg_io_hdr;

	/*
	 * This could cause a response to be stranded. Close the associated
	 * file descriptor to free up any resources being held.
	 */
	ret = sg_check_file_access(filp, __func__);
	if (ret)
		return ret;

	sfp = filp->private_data;
	if (IS_ERR_OR_NULL(sfp)) {
		pr_warn("sg: %s: sfp is NULL or error\n", __func__);
		return IS_ERR(sfp) ? PTR_ERR(sfp) : -ENXIO;
	}
	sdp = sfp->parentdp;
	SG_LOG(3, sdp, "%s: read() count=%d\n", __func__, (int)count);
	if (IS_ERR_OR_NULL(sdp))
		return IS_ERR(sdp) ? PTR_ERR(sdp) : -ENXIO;
	could_be_v3 = (count >= SZ_SG_IO_HDR);
	hlen = could_be_v3 ? SZ_SG_IO_HDR : SZ_SG_HEADER;
	h2p = (struct sg_header *)&a_sg_io_hdr;

	if (test_bit(SG_FFD_FORCE_PACKID, sfp->ffd_bm) && (int)count >= hlen) {
		/*
		 * Even though this is a user space read() system call, this
		 * code is cheating to fetch the pack_id. VERIFY_READ/WRITE
		 * direction is from the kernel/driver's perspective.
		 */
		if (!access_ok(p, hlen))
			return -EFAULT;
		/* Only need the first three 32 bit integers */
		if (__copy_from_user(h2p, p, 3 * sizeof(int)))
			return -EFAULT;
		if (h2p->reply_len < 0 && could_be_v3) {
			struct sg_io_hdr *v3_hdr = (struct sg_io_hdr *)h2p;

			if (v3_hdr->interface_id == 'S') {/* v3, new sanity */
				struct sg_io_hdr __user *h3_up;

				h3_up = (struct sg_io_hdr __user *)p;
				ret = get_user(req_pack_id, &h3_up->pack_id);
				if (ret)
					return ret;
				if (!non_block) {
					int flgs;

					ret = get_user(flgs, &h3_up->flags);
					if (ret)
						return ret;
					if (flgs & SGV4_FLAG_IMMED)
						non_block = true;
				}
			} else if (v3_hdr->interface_id == 'Q') {
				pr_info_once("sg: %s: v4 interface%s here\n",
					     __func__, " disallowed");
				return -EPERM;
			} else
				return -EPERM;
		} else	/* for v1+v2 interfaces, this is the 3rd integer */
			req_pack_id = h2p->pack_id;
	}
	srp = sg_get_rq_pack_id(sfp, req_pack_id, 0, false);
	if (!srp) {     /* nothing available so wait on packet to arrive or */
		if (unlikely(atomic_read(&sdp->detaching)))
			return -ENODEV;
		if (non_block) /* O_NONBLOCK or v3::flags & SGV4_FLAG_IMMED */
			return -EAGAIN;
		ret = wait_event_interruptible(sfp->read_wait,
				sg_get_ready_srp(sfp, &srp, req_pack_id,
						 0, false));
		if (unlikely(atomic_read(&sdp->detaching)))
			return -ENODEV;
		if (ret)	/* -ERESTARTSYS as signal hit process */
			return ret;
		/* otherwise srp should be valid */
	}
	if (srp->s_hdr3.interface_id == '\0')
		ret = sg_rd_v1v2(p, (int)count, sfp, srp);
	else {
		if ((int)count < SZ_SG_IO_HDR)
			return -EINVAL;
		ret = sg_v3_receive(sfp, srp, p);
	}
	if (ret < 0)
		SG_LOG(1, sdp, "%s: negated errno: %d\n", __func__, ret);
	return ret < 0 ? ret : (int)count;
}

/*
 * Completes a v3 request/command. Called from sg_read {v2 or v3},
 * ioctl(SG_IO) {for v3}, or from ioctl(SG_IORECEIVE) when its
 * completing a v3 request/command.
 */
static int
sg_v3_receive(struct sg_fd *sfp, struct sg_request *srp, void __user *p)
{
	int err, err2;
	int rq_result = srp->rq_result;
	struct sg_io_hdr hdr3;
	struct sg_io_hdr *hp = &hdr3;

	SG_LOG(3, sfp->parentdp, "%s: sh_var: %s srp=0x%p\n", __func__,
	       sg_shr_str(srp->sh_var, false), srp);

	/* v3 completion below this point */
	err = sg_rec_v3v4_state(sfp, srp, false);
	memset(hp, 0, sizeof(*hp));
	memcpy(hp, &srp->s_hdr3, sizeof(srp->s_hdr3));
	hp->sb_len_wr = srp->sense_len;
	hp->info = srp->rq_info;
	hp->resid = srp->in_resid;
	hp->duration = srp->duration;
	hp->status = rq_result & 0xff;
	hp->masked_status = status_byte(rq_result);
	hp->msg_status = msg_byte(rq_result);
	hp->host_status = host_byte(rq_result);
	hp->driver_status = driver_byte(rq_result);
	/* copy_*_user() [without leading underscores] checks access */
	if (copy_to_user(p, hp, SZ_SG_IO_HDR))
		err = err ? err : -EFAULT;
	err2 = sg_v3v4_complete(sfp, srp, err < 0);
	if (err2)
		err = err ? err : err2;
	sg_finish_scsi_blk_rq(srp);
	sg_deact_request(sfp, srp);
	return err ? err : 0;
}

static int
max_sectors_bytes(struct request_queue *q)
{
	unsigned int max_sectors = queue_max_sectors(q);

	max_sectors = min_t(unsigned int, max_sectors, INT_MAX >> 9);
	return max_sectors << 9;
}

/*
 * Calculates sg_device::max_sgat_elems and sg_device::max_sgat_sz. It uses
 * the device's request queue. If q not available sets max_sgat_elems to 1
 * and max_sgat_sz to PAGE_SIZE. If potential max_sgat_sz is greater than
 * 2^30 scales down the implied max_segment_size so the product of the
 * max_segment_size and max_sgat_elems is less than or equal to 2^30 .
 */
static void
sg_calc_sgat_param(struct sg_device *sdp)
{
	int sz;
	u64 m;
	struct scsi_device *sdev = sdp->device;
	struct request_queue *q = sdev ? sdev->request_queue : NULL;

	if (!q) {
		sdp->max_sgat_elems = 1;
		sdp->max_sgat_sz = PAGE_SIZE;
		return;
	}
	sdp->max_sgat_elems = queue_max_segments(q);
	m = (u64)queue_max_segment_size(q) * queue_max_segments(q);
	if (m < PAGE_SIZE) {
		sdp->max_sgat_elems = 1;
		sdp->max_sgat_sz = PAGE_SIZE;
		return;
	}
	sz = (int)min_t(u64, m, 1 << 30);
	if (sz == (1 << 30))	/* round down so: sz = elems * elem_sz */
		sz = ((1 << 30) / sdp->max_sgat_elems) * sdp->max_sgat_elems;
	sdp->max_sgat_sz = sz;
}

/*
 * Only valid for shared file descriptors, else -EINVAL. Should only be
 * called after a master side request has successfully completed so that
 * there is valid data in reserve buffer. If fini1_again0 is true then
 * master is taken out of the state waiting for a slave request and the
 * master is put in the inactive state. If fini1_again0 is false (0) then
 * the master (assuming it is inactive) is put in a state waiting for
 * a slave request. This function is called when the write mask is set on
 * ioctl(SG_SET_GET_EXTENDED(SG_CTL_FLAGM_MASTER_FINI)).
 */
static int
sg_change_after_master_rq(struct sg_fd *sfp, bool fini1_again0)
{
	int res = 0;
	unsigned long iflags;
	enum sg_rq_state sr_st;
	struct sg_request *ma_rsv_srp = NULL;

	read_lock_irqsave(&sfp->rq_list_lock, iflags);
	if (sfp->shr_fd == SG_SHARE_FD_MASTER)		/* from master */
		ma_rsv_srp = sfp->rsv_srp;
	else if (sfp->shr_fd == SG_SHARE_FD_UNUSED)
		res = -EINVAL;
	else {						/* from slave */
		if (sfp->share_sfp)
			ma_rsv_srp = sfp->share_sfp->rsv_srp;
		else
			res = -EINVAL;
	}
	if (res == 0 && ma_rsv_srp) {
		spin_lock(&ma_rsv_srp->req_lck);
		sr_st = atomic_read(&ma_rsv_srp->rq_st);
		if (fini1_again0) {
			switch (sr_st) {
			case SG_RS_SHR_SWAP:
				ma_rsv_srp->sh_var = SG_SHR_MA_FD_NOT_RQ;
				ma_rsv_srp = NULL;
				res = sg_rstate_chg(ma_rsv_srp, sr_st,
						    SG_RS_INACTIVE);
				break;
			case SG_RS_SHR_SLAVE:	/* too late, slave rq active */
			case SG_RS_BUSY:
				res = -EAGAIN;
				break;
			default:	/* master in SG_RS_SHR_SWAIT is bad */
				res = -EINVAL;
				break;
			}
		} else if (sr_st == SG_RS_INACTIVE) {
			ma_rsv_srp->sh_var = SG_SHR_MA_RQ;
			res = sg_rstate_chg(ma_rsv_srp, sr_st, SG_RS_SHR_SWAP);
		} else {
			res = -EBUSY;	/* master busy doing something else */
		}
		spin_unlock(&ma_rsv_srp->req_lck);
	}
	read_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	if (res) {
		SG_LOG(1, sfp->parentdp, "%s: err=%d\n", __func__, -res);
	} else {
		SG_LOG(6, sfp->parentdp, "%s: okay, fini1_again0=%d\n",
		       __func__, fini1_again0);
	}
	return res;
}

static inline void
sg_unshare_fds(struct sg_fd *ma_sfp, struct sg_fd *sl_sfp)
{
	if (ma_sfp) {
		ma_sfp->share_sfp = NULL;
		ma_sfp->slave_srp = NULL;
		ma_sfp->shr_fd = SG_SHARE_FD_UNUSED;
	}
	if (sl_sfp) {
		sl_sfp->share_sfp = NULL;
		sl_sfp->shr_fd = SG_SHARE_FD_UNUSED;
	}
}

/* Placed here with other share/unshare processing functions */
static void
sg_remove_sfp_share(struct sg_fd *sfp)
{
	bool o_sfp_is_master = false;
	int res = 0;
	enum sg_rq_state sr_st;
	struct sg_fd *o_sfp = sfp->share_sfp;
	struct sg_request *rsv_srp = NULL;

	SG_LOG(3, sfp->parentdp, "%s: sfp=0x%p, o_sfp=0x%p%s\n", __func__,
	       sfp, o_sfp,
	       ((sfp->shr_fd == SG_SHARE_FD_MASTER) ? " master" : ""));
	if (!o_sfp)
		return;
	if (sfp->shr_fd == SG_SHARE_FD_MASTER)		/* close on master */
		rsv_srp = sfp->rsv_srp;
	else if (sfp->shr_fd == SG_SHARE_FD_UNUSED)
		return;
	else {						/* close on slave */
		o_sfp_is_master = true;
		rsv_srp = o_sfp->rsv_srp;
	}

	if (res == 0) {
		write_lock(&o_sfp->rq_list_lock);
		sg_unshare_fds(o_sfp, NULL);
		write_unlock(&o_sfp->rq_list_lock);
		if (!rsv_srp) {
			res = -EPROTO;
			goto fini;
		}
		spin_lock(&rsv_srp->req_lck);
		if (o_sfp_is_master) {
			if (rsv_srp->sh_var == SG_SHR_MA_RQ) {
				sr_st = atomic_read(&rsv_srp->rq_st);
				switch (sr_st) {
				case SG_RS_SHR_SLAVE:
				case SG_RS_SHR_SWAP:
					res = sg_rstate_chg(rsv_srp, sr_st,
							    SG_RS_INACTIVE);
					break;
				case SG_RS_BUSY:
					res = -EBUSY;
				default:
					break;
				}
				rsv_srp->sh_var = SG_SHR_NONE;
			}
		} else {
			rsv_srp->sh_var = SG_SHR_NONE;
		}
		spin_unlock(&rsv_srp->req_lck);
	}
fini:
	if (res) {
		SG_LOG(1, sfp->parentdp, "%s: internal err=%d\n", __func__,
		       -res);
	}
}

/*
 * Active when writing 1 to ioctl(SG_SET_GET_EXTENDED(CTL_FLAGS(UNSHARE))),
 * writing 0 has no effect. Undoes the configuration that has done by
 * ioctl(SG_SET_GET_EXTENDED(SHARE_FD)).
 */
static void
sg_chk_unshare(struct sg_fd *sfp, bool unshare_val)
{
	bool retry;
	int retry_count = 0;
	unsigned long iflags;
	struct sg_request *ma_rsv_srp;
	struct sg_fd *ma_sfp;
	struct sg_fd *sl_sfp;
	struct sg_fd *o_sfp = sfp->share_sfp;/* other side of existing share */

	if (sfp->shr_fd == SG_SHARE_FD_UNUSED || !o_sfp)
		return;	/* no share to undo */
	if (!unshare_val)
		return;
again:
	retry = false;
	write_lock_irqsave(&sfp->rq_list_lock, iflags);
	if (sfp->shr_fd == SG_SHARE_FD_MASTER) {  /* called on master fd */
		ma_sfp = sfp;
		sl_sfp = o_sfp;
		ma_rsv_srp = ma_sfp->rsv_srp;
		if (ma_rsv_srp && !SG_MA_THIS_RQ(ma_rsv_srp)) {
			if (!write_trylock(&sl_sfp->rq_list_lock)) {
				if (++retry_count > SG_ADD_RQ_MAX_RETRIES)
					SG_LOG(1, sfp->parentdp,
					       "%s: cannot get slave lock\n",
					       __func__);
				else
					retry = true;
				goto fini;
			}
			sg_unshare_fds(ma_sfp, sl_sfp);
			write_unlock(&sl_sfp->rq_list_lock);
		}
	} else {			/* called on slave fd */
		ma_sfp = o_sfp;
		sl_sfp = sfp;
		if (!write_trylock(&ma_sfp->rq_list_lock)) {
			if (++retry_count > SG_ADD_RQ_MAX_RETRIES)
				SG_LOG(1, sfp->parentdp,
				       "%s: cannot get master lock\n",
				       __func__);
			else
				retry = true;
			goto fini;
		}
		ma_rsv_srp = ma_sfp->rsv_srp;
		if (!SG_MA_THIS_RQ(ma_rsv_srp))
			sg_unshare_fds(ma_sfp, sl_sfp);
		write_unlock(&ma_sfp->rq_list_lock);
	}
fini:
	write_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	if (retry) {
		cpu_relax();
		goto again;
	}
}

/*
 * For backward compatibility the duration in nanoseconds is placed in a 32 bit
 * unsigned integer. This limits the maximum duration that can be represented
 * (without wrapping) to about 4.3 seconds. If that is exceeded return
 * equivalent of 3.999.. secs as it is more eye catching than the real
 * number. Negative durations should not be possible but if they occur set
 * duration to an unlikely 2 nanosec. Stalls in a request setup will have
 * ts0==KTIME_MAX and will return 1 for an unlikely 1 nanosecond duration.
 */
static u32
sg_calc_req_time(const struct sg_request *srp, bool time_in_ns)
{
	u32 ms;

	if (time_in_ns) {
		ktime_t now_ts = ktime_get_with_offset(TK_OFFS_BOOT);
		ktime_t ts0 = srp->start_kt;
		s64 diff;

		if (ts0 == KTIME_MAX)
			return 1;
		if (ts0 > now_ts)
			return 2;
		diff = ktime_to_ns(ktime_sub(now_ts, ts0));
		return (diff > (s64)U32_MAX) ? 3999999999U : (u32)diff;
	}
	ms = jiffies_to_msecs(jiffies);
	return (ms > srp->start_ms) ? (ms - srp->start_ms) : 0;
}

static void
sg_fill_request_element(struct sg_fd *sfp, struct sg_request *srp,
			struct sg_req_info *rip)
		__must_hold(&sfp->rq_list_lock)
{
	enum sg_rq_state sr_st;

	spin_lock(&srp->req_lck);
	sr_st = atomic_read(&srp->rq_st);
	switch (sr_st) {
	case SG_RS_INFLIGHT:
		rip->duration = sg_calc_req_time(srp,
			test_bit(SG_FFD_TIME_IN_NS, sfp->ffd_bm));
		break;
	case SG_RS_AWAIT_RD:
	case SG_RS_DONE_RD:
	case SG_RS_SHR_SWAP:
	case SG_RS_SHR_SLAVE:
		rip->duration = srp->duration;
		break;
	case SG_RS_INACTIVE:
	case SG_RS_BUSY:
	case SG_RS_SHR_SWAIT:
	default:
		rip->duration = 0;
		break;
	}
	rip->orphan = test_bit(SG_FRQ_IS_ORPHAN, srp->frq_bm);
	rip->sg_io_owned = test_bit(SG_FRQ_SYNC_INVOC, srp->frq_bm);
	rip->problem = !!(srp->rq_result & SG_ML_RESULT_MSK);
	rip->pack_id = test_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm) ?
				srp->tag : srp->pack_id;
	rip->usr_ptr = test_bit(SG_FRQ_IS_V4I, srp->frq_bm) ?
			uptr64(srp->s_hdr4.usr_ptr) : srp->s_hdr3.usr_ptr;
	spin_unlock(&srp->req_lck);
}

static inline bool
sg_rq_landed(struct sg_device *sdp, struct sg_request *srp)
{
	return atomic_read(&srp->rq_st) != SG_RS_INFLIGHT ||
	       unlikely(atomic_read(&sdp->detaching));
}

/*
 * This is a blocking wait for a specific srp. When h4p is non-NULL, it is
 * the blocking multiple request case
 */
static int
sg_wait_event_srp(struct file *filp, struct sg_fd *sfp, void __user *p,
		  struct sg_io_v4 *h4p, struct sg_request *srp)
{
	int res;
	enum sg_rq_state sr_st;
	struct sg_device *sdp = sfp->parentdp;

	SG_LOG(3, sdp, "%s: start\n", __func__);
	/* usually will be woken up by sg_rq_end_io() callback */
	res = wait_event_interruptible(sfp->read_wait,
				       sg_rq_landed(sdp, srp));
	if (unlikely(res)) { /* -ERESTARTSYS because signal hit thread */
		set_bit(SG_FRQ_IS_ORPHAN, srp->frq_bm);
		/* orphans harvested when sfp->keep_orphan is false */
		atomic_set(&srp->rq_st, SG_RS_INFLIGHT);
		SG_LOG(1, sdp, "%s:  wait_event_interruptible gave %d\n",
		       __func__, res);
		return res;
	}
	if (unlikely(atomic_read(&sdp->detaching))) {
		atomic_set(&srp->rq_st, SG_RS_INACTIVE);
		return -ENODEV;
	}
	sr_st = atomic_read(&srp->rq_st);
	if (unlikely(sr_st != SG_RS_AWAIT_RD))
		return -EPROTO;         /* Logic error */
	res = sg_rstate_chg(srp, sr_st, SG_RS_BUSY);
	if (unlikely(res))
		return res;
	if (test_bit(SG_FRQ_IS_V4I, srp->frq_bm))
		res = sg_v4_receive(sfp, srp, p, h4p);
	else
		res = sg_v3_receive(sfp, srp, p);
	return (res < 0) ? res : 0;
}

/*
 * Handles ioctl(SG_IO) for blocking (sync) usage of v3 or v4 interface.
 * Returns 0 on success else a negated errno.
 */
static int
sg_ctl_sg_io(struct file *filp, struct sg_device *sdp, struct sg_fd *sfp,
	       void __user *p)
{
	int res;
	struct sg_request *srp = NULL;
	u8 hu8arr[SZ_SG_IO_V4];		/* v4 header larger than v3 */
	struct sg_io_hdr *h3p = (struct sg_io_hdr *)hu8arr;
	struct sg_io_v4 *h4p = (struct sg_io_v4 *)hu8arr;

	SG_LOG(3, sdp, "%s:  SG_IO\n", __func__);
	if (atomic_read(&sdp->detaching))
		return -ENODEV;
	if (!scsi_block_when_processing_errors(sdp->device))
		return -ENXIO;
	if (!access_ok(p, SZ_SG_IO_HDR))
		return -EFAULT;
	if (__copy_from_user(h3p, p, SZ_SG_IO_HDR))
		return -EFAULT;
	if (h3p->interface_id == 'Q') {
		if (!access_ok(p, SZ_SG_IO_V4))
			return -EFAULT;
		/* copy in rest of sg_io_v4 object */
		if (__copy_from_user(hu8arr + SZ_SG_IO_HDR,
				     ((u8 __user *)p) + SZ_SG_IO_HDR,
				     SZ_SG_IO_V4 - SZ_SG_IO_HDR))
			return -EFAULT;
		res = sg_v4_submit(filp, sfp, p, h4p, true, &srp);
	} else if (h3p->interface_id == 'S') {
		res = sg_v3_submit(filp, sfp, h3p, true, &srp);
	} else {
		pr_info_once("sg: %s: v3 or v4 interface only here\n",
			     __func__);
		return -EPERM;
	}
	if (unlikely(res < 0))
		return res;
	if (!srp)	/* mrq case: already processed all responses */
		return res;
	res = sg_wait_event_srp(filp, sfp, p, h4p, srp);
	if (res) {
		SG_LOG(1, sdp, "%s: %s=0x%p  state: %s, share: %s\n",
		       __func__, "unexpected srp", srp,
		       sg_rq_st_str(atomic_read(&srp->rq_st), false),
		       sg_shr_str(srp->sh_var, false));
	}
	return res;
}

/* When use_tag is true then id is a tag, else it is a pack_id. */
static struct sg_request *
sg_match_request(struct sg_fd *sfp, bool use_tag, int id)
{
	bool found = false;
	struct sg_request *srp;

	if (list_empty(&sfp->rq_list))
		return NULL;
	if (id == SG_PACK_ID_WILDCARD)	/* both wildcards same value: -1 */
		return list_first_entry(&sfp->rq_list, struct sg_request,
					rq_entry);
	list_for_each_entry(srp, &sfp->rq_list, rq_entry) {
		if (id == (use_tag ? srp->tag : srp->pack_id)) {
			found = true;
			break;
		}
	}
	return found ? srp : NULL;
}

static int
sg_ctl_abort(struct sg_device *sdp, struct sg_fd *sfp, void __user *p)
{
	bool use_tag;
	int res, pack_id, tag, id;
	unsigned long iflags;
	struct sg_fd *o_sfp;
	struct sg_request *srp;
	struct sg_io_v4 io_v4;
	struct sg_io_v4 *h4p = &io_v4;

	if (!access_ok(p, SZ_SG_IO_V4))
		return -EFAULT;
	if (__copy_from_user(h4p, p, SZ_SG_IO_V4))
		return -EFAULT;
	if (h4p->guard != 'Q' || h4p->protocol != 0 || h4p->subprotocol != 0)
		return -EPERM;
	pack_id = h4p->request_extra;
	tag = h4p->request_tag;
	use_tag = test_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm);
	id = use_tag ? tag : pack_id;

	write_lock(&sfp->rq_list_lock);
	srp = sg_match_request(sfp, use_tag, id);
	if (srp) {
		spin_lock_irqsave(&srp->req_lck, iflags);
		write_unlock(&sfp->rq_list_lock);
	} else if (h4p->flags & SGV4_FLAG_DEV_SCOPE) {
		write_unlock(&sfp->rq_list_lock);
		read_lock(&sdp->sfd_llock);
		list_for_each_entry(o_sfp, &sdp->sfds, sfd_entry) {
			if (o_sfp == sfp)
				continue;	/* already checked */
			write_lock(&o_sfp->rq_list_lock);
			srp = sg_match_request(o_sfp, use_tag, id);
			if (srp) {
				spin_lock_irqsave(&srp->req_lck, iflags);
				write_unlock(&o_sfp->rq_list_lock);
				sfp = o_sfp;
				break;
			}
			write_unlock(&o_sfp->rq_list_lock);
		}
		read_unlock(&sdp->sfd_llock);
	} else {
		write_unlock(&sfp->rq_list_lock);
	}
	if (!srp)
		return -ENODATA;

	set_bit(SG_FRQ_ABORTING, srp->frq_bm);
	res = 0;
	switch (atomic_read(&srp->rq_st)) {
	case SG_RS_BUSY:
		clear_bit(SG_FRQ_ABORTING, srp->frq_bm);
		res = -EBUSY;	/* should occur often */
		break;
	case SG_RS_INACTIVE:	/* inactive on rq_list not good */
		clear_bit(SG_FRQ_ABORTING, srp->frq_bm);
		res = -EPROTO;
		break;
	case SG_RS_AWAIT_RD:	/* user should still do completion */
	case SG_RS_DONE_RD:
	case SG_RS_SHR_SWAP:
	case SG_RS_SHR_SLAVE:
		clear_bit(SG_FRQ_ABORTING, srp->frq_bm);
		break;		/* nothing to do here, return 0 */
	case SG_RS_INFLIGHT:
		srp->rq_result |= (DRIVER_SOFT << 24);
		if (srp->rq)
			blk_abort_request(srp->rq);
		break;
	case SG_RS_SHR_SWAIT:
		srp->rq_result |= (DRIVER_SOFT << 24);
		if (sg_rstate_chg(srp, SG_RS_SHR_SWAIT, SG_RS_AWAIT_RD))
			pr_warn("%s: unable to set rq_st?\n", __func__);
		/* Wake any sg_read()/ioctl(SG_IORECEIVE) awaiting this req */
		wake_up_interruptible(&sfp->read_wait);
		kill_fasync(&sfp->async_qp, SIGPOLL, POLL_IN);
		break;
	default:
		clear_bit(SG_FRQ_ABORTING, srp->frq_bm);
		break;
	}
	spin_unlock_irqrestore(&srp->req_lck, iflags);
	return res;
}

static int
sg_idr_max_id(int id, void *p, void *data)
	__must_hold(&sg_index_lock)
{
	int *k = data;

	if (*k < id)
		*k = id;
	return 0;
}

/*
 * Scans sg driver object tree looking for search_for. Returns valid pointer
 * if found; returns negated errno twisted by ERR_PTR(); or return NULL if
 * not found (and no error).
 */
static struct sg_fd *
sg_find_sfp_by_fd(const struct file *search_for, int search_fd,
		  struct sg_fd *from_sfp, bool from_is_master)
{
	bool found = false;
	int k, num_d;
	int res = 0;
	unsigned long iflags;
	struct sg_fd *sfp;
	struct sg_device *sdp;

	num_d = -1;
	read_lock_irqsave(&sg_index_lock, iflags);
	idr_for_each(&sg_index_idr, sg_idr_max_id, &num_d);
	++num_d;
	for (k = 0; k < num_d; ++k) {
		sdp = idr_find(&sg_index_idr, k);
		if (!sdp || unlikely(atomic_read(&sdp->detaching)))
			continue;
		read_lock(&sdp->sfd_llock);
		list_for_each_entry(sfp, &sdp->sfds, sfd_entry) {
			if (test_bit(SG_FFD_RELEASE, sfp->ffd_bm))
				continue;
			write_lock(&sfp->rq_list_lock);
			if (search_for != sfp->filp) {
				write_unlock(&sfp->rq_list_lock);
				continue;       /* not this one */
			}
			if (sfp->shr_fd != SG_SHARE_FD_UNUSED)
				res = -EADDRNOTAVAIL;/* already sharing */
			else if (SG_RS_ACTIVE(sfp->rsv_srp))
				res = -EBUSY;/* master reserve rq busy */
			else if (sdp->exclude)  /* O_EXCL used on this dev */
				res = -EPERM;
			else {
				res = 0;
				found = true;
				if (write_trylock(&from_sfp->rq_list_lock)) {
					from_sfp->share_sfp = sfp;
					from_sfp->shr_fd = from_is_master ?
						SG_SHARE_FD_MASTER : search_fd;
					write_unlock(&from_sfp->rq_list_lock);
					sfp->share_sfp = from_sfp;
					sfp->shr_fd = from_is_master ?
						search_fd : SG_SHARE_FD_MASTER;
				} else  /* ?? borrow EPROBE_DEFER + encore */
					res = -EPROBE_DEFER;
			}
			write_unlock(&sfp->rq_list_lock);
			if (res || found)
				break;
		}       /* end of loop of all fd_s in current device */
		read_unlock(&sdp->sfd_llock);
		if (res || found)
			break;
	}       /* end of loop of all sg devices */
	read_unlock_irqrestore(&sg_index_lock, iflags);
	if (res < 0)
		return ERR_PTR(res);
	return found ? sfp : NULL;
}

/*
 * After checking the proposed master-slave relationship is unique and valid,
 * sets up pointers between master and slave sg_fd objects. Returns 0 on
 * success or negated errno value. From ioctl(EXTENDED(SG_SEIM_SHARE_FD)).
 */
static int
sg_fd_share(struct sg_fd *sl_sfp, int m_fd)
{
	bool found = false;
	int res = 0;
	int retry_count = 0;
	struct file *fp;
	struct sg_fd *ma_sfp;

	SG_LOG(3, sl_sfp->parentdp, "%s:  SHARE: master_fd: %d\n", __func__,
	       m_fd);
	if (!capable(CAP_SYS_ADMIN) || !capable(CAP_SYS_RAWIO))
		return -EACCES;
	if (m_fd < 0)
		return -EBADF;
	if (sl_sfp->shr_fd != SG_SHARE_FD_UNUSED)
		return -EADDRINUSE;  /* don't allow chain of shares */
	/* Alternate approach: fcheck_files(current->files, m_fd) */
	fp = fget(m_fd);
	if (!fp)
		return -ENOENT;
	if (sl_sfp->filp == fp) {/* share with self confusing */
		res = -ELOOP;
		goto fini;
	}
	SG_LOG(6, sl_sfp->parentdp, "%s: master fd okay, scan for filp=0x%p\n",
	       __func__, fp);
again:
	ma_sfp = sg_find_sfp_by_fd(fp, m_fd, sl_sfp, false);
	if (IS_ERR(ma_sfp)) {
		res = PTR_ERR(ma_sfp);
		if (res == -EPROBE_DEFER) {
			if (++retry_count > SG_ADD_RQ_MAX_RETRIES) {
				res = -EBUSY;
			} else {
				res = 0;
				cpu_relax();
				goto again;
			}
		}
	} else {
		found = !!ma_sfp;
	}
fini:
	/* paired with fp=fget(m_fd) above */
	fput(fp);
	if (res)
		return res;
	return found ? 0 : -ENOTSOCK; /* ENOTSOCK for fd exists but not sg */
}

/*
 * After checking the proposed master-slave relationship is unique and
 * valid, sets up pointers between master and slave sg_fd objects. Return
 * 0 on success or negated errno value.
 */
static int
sg_fd_reshare(struct sg_fd *ma_sfp, int new_sl_fd)
{
	bool found = false;
	int res = 0;
	int retry_count = 0;
	unsigned long iflags;
	struct file *fp;
	struct sg_fd *sl_sfp = ma_sfp->share_sfp;
	enum sg_rq_state rq_st;

	SG_LOG(3, sl_sfp->parentdp, "%s:  new_slave_fd: %d\n", __func__,
	       new_sl_fd);
	if (!capable(CAP_SYS_ADMIN) || !capable(CAP_SYS_RAWIO))
		return -EACCES;
	if (new_sl_fd < 0)
		return -EBADF;
	if (ma_sfp->shr_fd != SG_SHARE_FD_MASTER || !sl_sfp)
		return -EINVAL;
	if (!ma_sfp->rsv_srp)
		res = -EPROTO;	/* Internal error */
	read_lock_irqsave(&ma_sfp->rq_list_lock, iflags);
	rq_st = atomic_read(&ma_sfp->rsv_srp->rq_st);
	if (!(rq_st == SG_RS_INACTIVE || rq_st == SG_RS_SHR_SWAP))
		res = -EBUSY;		/* master reserve buffer busy */
	if (ma_sfp->slave_srp)
		res = -EBUSY;	/* previous slave request not finished */
	read_unlock_irqrestore(&ma_sfp->rq_list_lock, iflags);
	if (res)
		return res;

	/* Alternate approach: fcheck_files(current->files, m_fd) */
	fp = fget(new_sl_fd);
	if (!fp)
		return -ENOENT;
	if (ma_sfp->filp == fp) {/* share with self confusing */
		res = -ELOOP;
		goto fini;
	}
	SG_LOG(6, sl_sfp->parentdp, "%s: slave fd ok, scan for filp=0x%p\n",
	       __func__, fp);
	/* cleanup up old slave side */
	write_lock_irqsave(&sl_sfp->rq_list_lock, iflags);
	sl_sfp->share_sfp = NULL;
	sl_sfp->shr_fd = SG_SHARE_FD_UNUSED;
	write_unlock_irqrestore(&sl_sfp->rq_list_lock, iflags);

again:
	sl_sfp = sg_find_sfp_by_fd(fp, new_sl_fd, ma_sfp, true);
	if (IS_ERR(sl_sfp)) {
		res = PTR_ERR(sl_sfp);
		if (res == -EPROBE_DEFER) {
			if (++retry_count > SG_ADD_RQ_MAX_RETRIES) {
				res = -EBUSY;
			} else {
				res = 0;
				cpu_relax();
				goto again;
			}
		}
	} else {
		found = !!sl_sfp;
	}
fini:
	/* paired with fp=fget(new_sl_fd) above */
	fput(fp);
	if (res)
		return res;
	return found ? 0 : -ENOTSOCK; /* ENOTSOCK for fd exists but not sg */
}

/*
 * First normalize want_rsv_sz to be >= sfp->sgat_elem_sz and
 * <= max_segment_size. Exit if that is the same as old size; otherwise
 * create a new candidate request of the new size. Then decide whether to
 * re-use an existing free list request (least buflen >= required size) or
 * use the new candidate. If new one used, leave old one but it is no longer
 * the reserved request. Returns 0 on success, else a negated errno value.
 */
static int
sg_set_reserved_sz(struct sg_fd *sfp, int want_rsv_sz)
{
	bool use_new_srp = false;
	int res = 0;
	int new_sz, blen;
	unsigned long iflags;
	struct sg_request *o_srp;       /* prior reserve sg_request */
	struct sg_request *n_srp;       /* new sg_request, may be used */
	struct sg_request *t_srp;       /* other fl entries */
	struct sg_device *sdp = sfp->parentdp;

	if (sfp->shr_fd != SG_SHARE_FD_UNUSED)
		return -EBUSY;	/* this fd can't be either side of share */
	read_lock_irqsave(&sfp->rq_list_lock, iflags);
	o_srp = sfp->rsv_srp;
	if (!o_srp) {
		read_unlock_irqrestore(&sfp->rq_list_lock, iflags);
		return -EPROTO;
	}
	new_sz = min_t(int, want_rsv_sz, sdp->max_sgat_sz);
	new_sz = max_t(int, new_sz, sfp->sgat_elem_sz);
	blen = o_srp->sgatp->buflen;
	read_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	SG_LOG(3, sdp, "%s: was=%d, ask=%d, new=%d (sgat_elem_sz=%d)\n",
	       __func__, blen, want_rsv_sz, new_sz, sfp->sgat_elem_sz);
	if (blen == new_sz)
		return 0;
	n_srp = sg_mk_srp_sgat(sfp, true /* can take time */, new_sz);
	if (IS_ERR(n_srp))
		return PTR_ERR(n_srp);
	/* new sg_request object, sized correctly is now available */
	write_lock_irqsave(&sfp->rq_list_lock, iflags);
	o_srp = sfp->rsv_srp;
	if (!o_srp) {
		res = -EPROTO;
		goto wr_unlock;
	}
	if (SG_RS_ACTIVE(o_srp) ||
	    test_bit(SG_FFD_MMAP_CALLED, sfp->ffd_bm)) {
		res = -EBUSY;
		goto wr_unlock;
	}
	use_new_srp = true;
	list_for_each_entry(t_srp, &sfp->rq_fl, fl_entry) {
		if (t_srp != o_srp && new_sz <= t_srp->sgatp->buflen &&
		    !SG_RS_ACTIVE(t_srp)) {
			/* good candidate on free list, use */
			use_new_srp = false;
			sfp->rsv_srp = t_srp;
			break;
		}
	}
	if (use_new_srp) {
		sfp->rsv_srp = n_srp;
		/* add to front of free list */
		list_add(&n_srp->fl_entry, &sfp->rq_fl);
		SG_LOG(6, sdp, "%s: new rsv srp=0x%p ++\n", __func__, n_srp);
	}
wr_unlock:
	write_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	if (!use_new_srp) {
		sg_remove_sgat(n_srp);
		kfree(n_srp);   /* no-one else has seen n_srp, so safe */
	}
	return res;
}

static bool
sg_any_persistent_orphans(struct sg_fd *sfp)
{
	bool res = false;
	unsigned long iflags;
	struct sg_request *srp;

	if (!test_bit(SG_FFD_KEEP_ORPHAN, sfp->ffd_bm))
		return false;
	read_lock_irqsave(&sfp->rq_list_lock, iflags);
	list_for_each_entry(srp, &sfp->rq_list, rq_entry) {
		if (test_bit(SG_FRQ_IS_ORPHAN, srp->frq_bm)) {
			res = true;
			break;
		}
	}
	read_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	return res;
}

/*
 * Processing of ioctl(SG_SET_GET_EXTENDED(SG_SEIM_CTL_FLAGS)) which is a set
 * of boolean flags. Access abbreviations: [rw], read-write; [ro], read-only;
 * [wo], write-only; [raw], read after write; [rbw], read before write.
 */
static void
sg_extended_bool_flags(struct sg_fd *sfp, struct sg_extended_info *seip)
{
	u32 c_flgs_wm = seip->ctl_flags_wr_mask;
	u32 c_flgs_rm = seip->ctl_flags_rd_mask;
	u32 *c_flgsp = &seip->ctl_flags;
	struct sg_fd *ma_sfp = sfp->share_sfp;
	struct sg_device *sdp = sfp->parentdp;

	/* TIME_IN_NS boolean, [raw] time in nanoseconds (def: millisecs) */
	if (c_flgs_wm & SG_CTL_FLAGM_TIME_IN_NS)
		assign_bit(SG_FFD_TIME_IN_NS, sfp->ffd_bm,
			   !!(*c_flgsp & SG_CTL_FLAGM_TIME_IN_NS));
	if (c_flgs_rm & SG_CTL_FLAGM_TIME_IN_NS) {
		if (test_bit(SG_FFD_TIME_IN_NS, sfp->ffd_bm))
			*c_flgsp |= SG_CTL_FLAGM_TIME_IN_NS;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_TIME_IN_NS;
	}
	/* TAG_FOR_PACK_ID boolean, [raw] search by tag or pack_id (def) */
	if (c_flgs_wm & SG_CTL_FLAGM_TAG_FOR_PACK_ID)
		assign_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm,
			   !!(*c_flgsp & SG_CTL_FLAGM_TIME_IN_NS));
	if (c_flgs_rm & SG_CTL_FLAGM_TAG_FOR_PACK_ID) {
		if (test_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm))
			*c_flgsp |= SG_CTL_FLAGM_TAG_FOR_PACK_ID;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_TAG_FOR_PACK_ID;
	}
	/* ORPHANS boolean, [ro] does this fd have any orphan requests? */
	if (c_flgs_rm & SG_CTL_FLAGM_ORPHANS) {
		if (sg_any_persistent_orphans(sfp))
			*c_flgsp |= SG_CTL_FLAGM_ORPHANS;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_ORPHANS;
	}
	/* OTHER_OPENS boolean, [ro] any other sg open fds on this dev? */
	if (c_flgs_rm & SG_CTL_FLAGM_OTHER_OPENS) {
		if (sdp->open_cnt > 1)
			*c_flgsp |= SG_CTL_FLAGM_OTHER_OPENS;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_OTHER_OPENS;
	}
	/* Q_TAIL boolean, [raw] 1: queue at tail; 0: head (def: depends) */
	if (c_flgs_wm & SG_CTL_FLAGM_Q_TAIL)
		assign_bit(SG_FFD_Q_AT_TAIL, sfp->ffd_bm,
			   !!(*c_flgsp & SG_CTL_FLAGM_Q_TAIL));
	if (c_flgs_rm & SG_CTL_FLAGM_Q_TAIL) {
		if (test_bit(SG_FFD_Q_AT_TAIL, sfp->ffd_bm))
			*c_flgsp |= SG_CTL_FLAGM_Q_TAIL;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_Q_TAIL;
	}
	/*
	 * UNSHARE boolean: when reading yields zero. When writing true, set
	 * unshare flag and if shared command inflight breaks connection.
	 */
	if (c_flgs_wm & SG_CTL_FLAGM_UNSHARE)
		sg_chk_unshare(sfp, !!(*c_flgsp & SG_CTL_FLAGM_UNSHARE));
	if (c_flgs_rm & SG_CTL_FLAGM_UNSHARE)
		*c_flgsp &= ~SG_CTL_FLAGM_UNSHARE;	/* clear bit */
	/* IS_SHARE boolean: [ro] true if fd may be master or slave share */
	if (c_flgs_rm & SG_CTL_FLAGM_IS_SHARE) {
		if (sfp->shr_fd == SG_SHARE_FD_UNUSED)
			*c_flgsp &= ~SG_CTL_FLAGM_IS_SHARE;
		else
			*c_flgsp |= SG_CTL_FLAGM_IS_SHARE;
	}
	/* IS_MASTER boolean: [ro] true if this fd may be a master share */
	if (c_flgs_rm & SG_CTL_FLAGM_IS_MASTER) {
		if (sfp->shr_fd == SG_SHARE_FD_MASTER)
			*c_flgsp |= SG_CTL_FLAGM_IS_MASTER;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_IS_MASTER;
	}
	/*
	 * MASTER_FINI boolean, [rbw] should be called by slave; when
	 * reading: master is finished, awaiting action by slave;
	 * when written: 1 --> slave doesn't want to continue
	 */
	if (c_flgs_rm & SG_CTL_FLAGM_MASTER_FINI) {
		if (ma_sfp && ma_sfp->rsv_srp) {
			struct sg_request *res_srp = ma_sfp->rsv_srp;

			if (atomic_read(&res_srp->rq_st) == SG_RS_SHR_SWAP)
				*c_flgsp |= SG_CTL_FLAGM_MASTER_FINI;
			else
				*c_flgsp &= ~SG_CTL_FLAGM_MASTER_FINI;
		} else
			*c_flgsp &= ~SG_CTL_FLAGM_MASTER_FINI;
	}
	if (c_flgs_wm & SG_CTL_FLAGM_MASTER_FINI) {
		bool ma_fini_wm = !!(*c_flgsp & SG_CTL_FLAGM_MASTER_FINI);

		sg_change_after_master_rq(sfp, ma_fini_wm);
	}
	/* MASTER_ERR boolean, [ro] share: master finished with error */
	if (c_flgs_rm & SG_CTL_FLAGM_MASTER_ERR) {
		if (ma_sfp && test_bit(SG_FFD_MASTER_ERR, ma_sfp->ffd_bm))
			*c_flgsp |= SG_CTL_FLAGM_MASTER_ERR;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_MASTER_ERR;
	}
}

static void
sg_extended_read_value(struct sg_fd *sfp, struct sg_extended_info *seip)
{
	u32 uv;
	unsigned long iflags;
	struct sg_fd *a_sfp;
	struct sg_device *sdp = sfp->parentdp;
	struct sg_request *srp;

	switch (seip->read_value) {
	case SG_SEIRV_INT_MASK:
		seip->read_value = SG_SEIM_ALL_BITS;
		break;
	case SG_SEIRV_BOOL_MASK:
		seip->read_value = SG_CTL_FLAGM_ALL_BITS;
		break;
	case SG_SEIRV_VERS_NUM:
		seip->read_value = sg_version_num;
		break;
	case SG_SEIRV_FL_RQS:
		uv = 0;
		read_lock_irqsave(&sfp->rq_list_lock, iflags);
		list_for_each_entry(srp, &sfp->rq_fl, fl_entry)
			++uv;
		read_unlock_irqrestore(&sfp->rq_list_lock, iflags);
		seip->read_value = uv;
		break;
	case SG_SEIRV_DEV_FL_RQS:
		uv = 0;
		read_lock_irqsave(&sdp->sfd_llock, iflags);
		list_for_each_entry(a_sfp, &sdp->sfds, sfd_entry) {
			read_lock(&a_sfp->rq_list_lock);
			list_for_each_entry(srp, &a_sfp->rq_fl, fl_entry)
				++uv;
			read_unlock(&a_sfp->rq_list_lock);
		}
		read_unlock_irqrestore(&sdp->sfd_llock, iflags);
		seip->read_value = uv;
		break;
	case SG_SEIRV_TRC_SZ:
		uv = 0;
		if (IS_ENABLED(SG_PROC_AND_DEBUG)) {
			if (sg_trc_ctl_level > 0 && sg_trc_cb_valid) {
				struct circ_buf *bp = &sg_trc_cb;

				mutex_lock(&sg_trc_cb_lck);
				uv = (u32)CIRC_CNT(bp->head, bp->tail,
						   sg_trc_cb_sz);
				mutex_unlock(&sg_trc_cb_lck);
			}
		}
		seip->read_value = uv;
		break;
	case SG_SEIRV_TRC_MAX_SZ:
		uv = 0;
		if (IS_ENABLED(SG_PROC_AND_DEBUG)) {
			if (sg_trc_ctl_level > 0 && sg_trc_cb_valid)
				uv = sg_trc_cb_sz;
		}
		seip->read_value = uv;
		break;
	case SG_SEIRV_SUBMITTED:  /* counts all non-blocking on active list */
		seip->read_value = (u32)atomic_read(&sfp->submitted);
		break;
	case SG_SEIRV_DEV_SUBMITTED: /* sum(submitted) on all fd's siblings */
		uv = 0;
		read_lock_irqsave(&sdp->sfd_llock, iflags);
		list_for_each_entry(a_sfp, &sdp->sfds, sfd_entry)
			uv += (u32)atomic_read(&a_sfp->submitted);
		read_unlock_irqrestore(&sdp->sfd_llock, iflags);
		seip->read_value = uv;
		break;
	default:
		SG_LOG(6, sdp, "%s: can't decode %d --> read_value\n",
		       __func__, seip->read_value);
		seip->read_value = 0;
		break;
	}
}

/* Called when processing ioctl(SG_SET_GET_EXTENDED) */
static int
sg_ctl_extended(struct sg_fd *sfp, void __user *p)
{
	int result = 0;
	int ret = 0;
	int n, j, s_wr_mask, s_rd_mask;
	u32 uv, or_masks;
	struct sg_device *sdp = sfp->parentdp;
	struct sg_extended_info *seip;
	struct sg_extended_info sei;

	seip = &sei;
	if (!access_ok(p, SZ_SG_EXTENDED_INFO))
		return -EFAULT;
	if (__copy_from_user(seip, p, SZ_SG_EXTENDED_INFO))
		return -EFAULT;
	s_wr_mask = seip->sei_wr_mask;
	s_rd_mask = seip->sei_rd_mask;
	or_masks = s_wr_mask | s_rd_mask;
	if (or_masks == 0) {
		SG_LOG(2, sdp, "%s: both masks 0, do nothing\n", __func__);
		return 0;
	}
	SG_LOG(3, sdp, "%s: wr_mask=0x%x rd_mask=0x%x\n", __func__, s_wr_mask,
	       s_rd_mask);
	/* tot_fd_thresh (u32), [raw] [sum of active cmd dlen_s] */
	if (or_masks & SG_SEIM_TOT_FD_THRESH) {
		if (s_wr_mask & SG_SEIM_TOT_FD_THRESH) {
			uv = seip->tot_fd_thresh;
			if (uv > 0 && uv < PAGE_SIZE)
				uv = PAGE_SIZE;
			sfp->tot_fd_thresh = uv;
		}
		if (s_rd_mask & SG_SEIM_TOT_FD_THRESH)
			seip->tot_fd_thresh = sfp->tot_fd_thresh;
	}
	/* check all boolean flags for either wr or rd mask set in or_mask */
	if (or_masks & SG_SEIM_CTL_FLAGS)
		sg_extended_bool_flags(sfp, seip);
	/* yields minor_index (type: u32) [ro] */
	if (or_masks & SG_SEIM_MINOR_INDEX) {
		if (s_wr_mask & SG_SEIM_MINOR_INDEX) {
			SG_LOG(2, sdp, "%s: writing to minor_index ignored\n",
			       __func__);
		}
		if (s_rd_mask & SG_SEIM_MINOR_INDEX)
			seip->minor_index = sdp->index;
	}
	if ((s_rd_mask & SG_SEIM_READ_VAL) && (s_wr_mask & SG_SEIM_READ_VAL))
		sg_extended_read_value(sfp, seip);
	/* slave side gives fd of master to share with [rbw] */
	if (or_masks & SG_SEIM_SHARE_FD) {
		mutex_lock(&sfp->f_mutex);
		n = sfp->shr_fd;        /* hold prior shr_fd for later read */
		if (s_wr_mask & SG_SEIM_SHARE_FD) {
			result = sg_fd_share(sfp, (int)seip->share_fd);
			if (ret == 0 && result)
				ret = result;
		}
		if (s_rd_mask & SG_SEIM_SHARE_FD)
			seip->share_fd = (unsigned int)n;
		mutex_unlock(&sfp->f_mutex);
	}
	/* master side is given shr_fd of new slave [rbw] */
	if (or_masks & SG_SEIM_CHG_SHARE_FD) {
		mutex_lock(&sfp->f_mutex);
		n = sfp->shr_fd;        /* hold prior sl_fd for later read */
		if (s_wr_mask & SG_SEIM_CHG_SHARE_FD) {
			result = sg_fd_reshare(sfp, (int)seip->share_fd);
			if (ret == 0 && result)
				ret = result;
		}
		if (s_rd_mask & SG_SEIM_CHG_SHARE_FD)
			seip->share_fd = (unsigned int)n;
		mutex_unlock(&sfp->f_mutex);
	}
	/* override scatter gather element size [rbw] (def: SG_SCATTER_SZ) */
	if (or_masks & SG_SEIM_SGAT_ELEM_SZ) {
		n = sfp->sgat_elem_sz;
		if (s_wr_mask & SG_SEIM_SGAT_ELEM_SZ) {
			j = (int)seip->sgat_elem_sz;
			if (j != (1 << ilog2(j)) || j < (int)PAGE_SIZE) {
				SG_LOG(1, sdp, "%s: %s not power of 2, %s\n",
				       __func__, "sgat element size",
				       "or less than PAGE_SIZE");
				ret = -EINVAL;
			} else
				sfp->sgat_elem_sz = j;
		}
		if (s_rd_mask & SG_SEIM_SGAT_ELEM_SZ)
			seip->sgat_elem_sz = n; /* prior value if rw */
	}
	/* reserved_sz [raw], since may be reduced by other limits */
	if (s_wr_mask & SG_SEIM_RESERVED_SIZE) {
		mutex_lock(&sfp->f_mutex);
		result = sg_set_reserved_sz(sfp, (int)seip->reserved_sz);
		if (ret == 0 && result)
			ret = result;
		mutex_unlock(&sfp->f_mutex);
	}
	if (s_rd_mask & SG_SEIM_RESERVED_SIZE)
		seip->reserved_sz = (u32)min_t(int,
					       sfp->rsv_srp->sgatp->buflen,
					       sdp->max_sgat_sz);
	/* copy to user space if int or boolean read mask non-zero */
	if (s_rd_mask || seip->ctl_flags_rd_mask) {
		if (access_ok(p, SZ_SG_EXTENDED_INFO))
			result = __copy_to_user(p, seip, SZ_SG_EXTENDED_INFO);
		else
			result = -EFAULT;
		if (ret == 0 && result)
			ret = result;
	}
	return ret;
}

/*
 * For backward compatibility, output SG_MAX_QUEUE sg_req_info objects. First
 * fetch from the active list then, if there is still room, from the free
 * list. Some of the trailing elements may be empty which is indicated by all
 * fields being zero. Any requests beyond SG_MAX_QUEUE are ignored.
 */
static int
sg_ctl_req_tbl(struct sg_fd *sfp, void __user *p)
{
	int k, len, result, val, n;
	unsigned long iflags;
	struct sg_request *srp;
	struct sg_req_info *rinfop;
	struct sg_req_info *rip;

	SG_LOG(3, sfp->parentdp, "%s:    SG_GET_REQUEST_TABLE\n", __func__);
	k = SG_MAX_QUEUE;
	len = SZ_SG_REQ_INFO * k;
	rinfop = kcalloc(k, SZ_SG_REQ_INFO, GFP_KERNEL);
	if (!rinfop)
		return -ENOMEM;
	val = 0;
	read_lock_irqsave(&sfp->rq_list_lock, iflags);
	list_for_each_entry(srp, &sfp->rq_list, rq_entry)
		++val;

	if (val > 0) {
		int max_num = min_t(int, k, val);

		n = 0;
		list_for_each_entry(srp, &sfp->rq_list, rq_entry) {
			if (n >= max_num)
				break;
			rip = &rinfop[n];
			sg_fill_request_element(sfp, srp, rip);
			n++;
		}
		list_for_each_entry(srp, &sfp->rq_fl, fl_entry) {
			if (n >= max_num)
				break;
			rip = &rinfop[n];
			sg_fill_request_element(sfp, srp, rip);
			n++;
		}
	}
	read_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	result = copy_to_user(p, rinfop, len);
	result = result ? -EFAULT : 0;
	kfree(rinfop);
	return result;
}

static long
sg_ioctl(struct file *filp, unsigned int cmd_in, unsigned long arg)
{
	void __user *p = uptr64(arg);
	int __user *ip = p;
	struct sg_device *sdp;
	struct sg_fd *sfp;
	struct sg_request *srp;
	struct scsi_device *sdev;
	__maybe_unused const char *pmlp = ", pass to mid-level";
	bool read_only = O_RDWR != (filp->f_flags & O_ACCMODE);
	bool check_detach = false;
	bool prefer_tag;
	int val;
	int result = 0;
	unsigned long iflags;

	sfp = filp->private_data;
	if (IS_ERR_OR_NULL(sfp)) {
		pr_warn("sg: %s: sfp is NULL or error\n", __func__);
		return IS_ERR(sfp) ? PTR_ERR(sfp) : -ENXIO;
	}
	sdp = sfp->parentdp;
	SG_LOG(6, sdp, "%s: cmd=0x%x, O_NONBLOCK=%d\n", __func__, cmd_in,
	       !!(filp->f_flags & O_NONBLOCK));
	if (IS_ERR_OR_NULL(sdp))
		return IS_ERR(sdp) ? PTR_ERR(sdp) : -ENXIO;
	sdev = sdp->device;

	switch (cmd_in) {
	case SG_IO:
		return sg_ctl_sg_io(filp, sdp, sfp, p);
	case SG_IOSUBMIT:
		SG_LOG(3, sdp, "%s:    SG_IOSUBMIT\n", __func__);
		return sg_ctl_iosubmit(filp, sfp, p);
	case SG_IORECEIVE:
		SG_LOG(3, sdp, "%s:    SG_IORECEIVE\n", __func__);
		return sg_ctl_ioreceive(filp, sfp, p);
	case SG_IOABORT:
		SG_LOG(3, sdp, "%s:    SG_IOABORT\n", __func__);
		if (read_only)
			return -EPERM;
		mutex_lock(&sfp->f_mutex);
		result = sg_ctl_abort(sdp, sfp, p);
		mutex_unlock(&sfp->f_mutex);
		return result;
	case SG_SET_GET_EXTENDED:
		SG_LOG(3, sdp, "%s:    SG_SET_GET_EXTENDED\n", __func__);
		result = sg_ctl_extended(sfp, p);
		return result;
	case SG_SET_TIMEOUT:
		SG_LOG(3, sdp, "%s:    SG_SET_TIMEOUT\n", __func__);
		result = get_user(val, ip);
		if (result)
			return result;
		if (val < 0)
			return -EIO;
		if (val >= mult_frac((s64)INT_MAX, USER_HZ, HZ))
			val = min_t(s64, mult_frac((s64)INT_MAX, USER_HZ, HZ),
				    INT_MAX);
		sfp->timeout_user = val;
		sfp->timeout = mult_frac(val, HZ, USER_HZ);

		return 0;
	case SG_GET_TIMEOUT:    /* N.B. User receives timeout as return value */
				/* strange ..., for backward compatibility */
		SG_LOG(3, sdp, "%s:    SG_GET_TIMEOUT\n", __func__);
		return sfp->timeout_user;
	case SG_SET_FORCE_LOW_DMA:
		/*
		 * N.B. This ioctl never worked properly, but failed to
		 * return an error value. So returning '0' to keep
		 * compatibility with legacy applications.
		 */
		SG_LOG(3, sdp, "%s:    SG_SET_FORCE_LOW_DMA\n", __func__);
		return 0;
	case SG_GET_LOW_DMA:
		SG_LOG(3, sdp, "%s:    SG_GET_LOW_DMA\n", __func__);
		return put_user((int)sdev->host->unchecked_isa_dma, ip);
	case SG_GET_SCSI_ID:
		SG_LOG(3, sdp, "%s:    SG_GET_SCSI_ID\n", __func__);
		if (!access_ok(p, sizeof(struct sg_scsi_id)))
			return -EFAULT;
		else {
			struct sg_scsi_id __user *sg_idp = p;
			struct scsi_lun lun8b;

			if (unlikely(atomic_read(&sdp->detaching)))
				return -ENODEV;
			__put_user((int)sdev->host->host_no,
				   &sg_idp->host_no);
			__put_user((int)sdev->channel, &sg_idp->channel);
			__put_user((int)sdev->id, &sg_idp->scsi_id);
			__put_user((int)sdev->lun, &sg_idp->lun);
			__put_user((int)sdev->type, &sg_idp->scsi_type);
			__put_user((short)sdev->host->cmd_per_lun,
				   &sg_idp->h_cmd_per_lun);
			__put_user((short)sdev->queue_depth,
				   &sg_idp->d_queue_depth);
			int_to_scsilun(sdev->lun, &lun8b);
			__copy_to_user(sg_idp->scsi_lun, lun8b.scsi_lun, 8);
			return 0;
		}
	case SG_SET_FORCE_PACK_ID:
		SG_LOG(3, sdp, "%s:    SG_SET_FORCE_PACK_ID\n", __func__);
		result = get_user(val, ip);
		if (result)
			return result;
		assign_bit(SG_FFD_FORCE_PACKID, sfp->ffd_bm, !!val);
		return 0;
	case SG_GET_PACK_ID:    /* or tag of oldest "read"-able, -1 if none */
		read_lock_irqsave(&sfp->rq_list_lock, iflags);
		val = -1;
		prefer_tag = test_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm);
		list_for_each_entry(srp, &sfp->rq_list, rq_entry) {
			if (SG_RS_AWAIT_READ(srp) &&
			    !test_bit(SG_FRQ_SYNC_INVOC, srp->frq_bm)) {
				val = prefer_tag ? srp->tag : srp->pack_id;
				break;
			}
		}
		read_unlock_irqrestore(&sfp->rq_list_lock, iflags);
		if (!access_ok(ip, sizeof(int)))
			return -EFAULT;
		__put_user(val, ip);
		SG_LOG(3, sdp, "%s:    SG_GET_PACK_ID=%d\n", __func__, val);
		return 0;
	case SG_GET_NUM_WAITING:
		/* SG_GET_NUM_WAITING + num_inflight == SG_SEIRV_SUBMITTED */
		val = atomic_read(&sfp->waiting);
		SG_LOG(3, sdp, "%s:    SG_GET_NUM_WAITING=%d\n", __func__,
		       val);
		return put_user(val, ip);
	case SG_GET_SG_TABLESIZE:
		SG_LOG(3, sdp, "%s:    SG_GET_SG_TABLESIZE=%d\n", __func__,
		       sdp->max_sgat_elems);
		return put_user(sdp->max_sgat_elems, ip);
	case SG_SET_RESERVED_SIZE:
		mutex_lock(&sfp->f_mutex);
		result = get_user(val, ip);
		if (!result) {
			if (val >= 0 && val <= (1024 * 1024 * 1024)) {
				result = sg_set_reserved_sz(sfp, val);
			} else {
				SG_LOG(3, sdp, "%s: invalid size\n", __func__);
				result = -EINVAL;
			}
		}
		mutex_unlock(&sfp->f_mutex);
		return result;
	case SG_GET_RESERVED_SIZE:
		mutex_lock(&sfp->f_mutex);
		val = min_t(int, sfp->rsv_srp->sgatp->buflen,
			    sdp->max_sgat_sz);
		SG_LOG(3, sdp, "%s:    SG_GET_RESERVED_SIZE=%d\n",
		       __func__, val);
		result = put_user(val, ip);
		mutex_unlock(&sfp->f_mutex);
		return result;
	case SG_SET_COMMAND_Q:
		SG_LOG(3, sdp, "%s:    SG_SET_COMMAND_Q\n", __func__);
		result = get_user(val, ip);
		if (result)
			return result;
		assign_bit(SG_FFD_CMD_Q, sfp->ffd_bm, !!val);
		return 0;
	case SG_GET_COMMAND_Q:
		SG_LOG(3, sdp, "%s:    SG_GET_COMMAND_Q\n", __func__);
		return put_user(test_bit(SG_FFD_CMD_Q, sfp->ffd_bm), ip);
	case SG_SET_KEEP_ORPHAN:
		SG_LOG(3, sdp, "%s:    SG_SET_KEEP_ORPHAN\n", __func__);
		result = get_user(val, ip);
		if (result)
			return result;
		assign_bit(SG_FFD_KEEP_ORPHAN, sfp->ffd_bm, !!val);
		return 0;
	case SG_GET_KEEP_ORPHAN:
		SG_LOG(3, sdp, "%s:    SG_GET_KEEP_ORPHAN\n", __func__);
		return put_user(test_bit(SG_FFD_KEEP_ORPHAN, sfp->ffd_bm),
				ip);
	case SG_NEXT_CMD_LEN:   /* active only in v2 interface */
		SG_LOG(3, sdp, "%s:    SG_NEXT_CMD_LEN\n", __func__);
		result = get_user(val, ip);
		if (result)
			return result;
		if (val > SG_MAX_CDB_SIZE)
			return -ENOMEM;
		mutex_lock(&sfp->f_mutex);
		sfp->next_cmd_len = max_t(int, val, 0);
		mutex_unlock(&sfp->f_mutex);
		return 0;
	case SG_GET_VERSION_NUM:
		SG_LOG(3, sdp, "%s:    SG_GET_VERSION_NUM\n", __func__);
		return put_user(sg_version_num, ip);
	case SG_GET_ACCESS_COUNT:
		SG_LOG(3, sdp, "%s:    SG_GET_ACCESS_COUNT\n", __func__);
		/* faked - we don't have a real access count anymore */
		val = (sdev ? 1 : 0);
		return put_user(val, ip);
	case SG_GET_REQUEST_TABLE:
		return sg_ctl_req_tbl(sfp, p);
	case SG_EMULATED_HOST:
		SG_LOG(3, sdp, "%s:    SG_EMULATED_HOST\n", __func__);
		if (unlikely(atomic_read(&sdp->detaching)))
			return -ENODEV;
		return put_user(sdev->host->hostt->emulated, ip);
	case SCSI_IOCTL_SEND_COMMAND:
		SG_LOG(3, sdp, "%s:    SCSI_IOCTL_SEND_COMMAND\n", __func__);
		if (unlikely(atomic_read(&sdp->detaching)))
			return -ENODEV;
		return sg_scsi_ioctl(sdev->request_queue, NULL,
				     filp->f_mode, p);
	case SG_SET_DEBUG:
		SG_LOG(3, sdp, "%s:    SG_SET_DEBUG\n", __func__);
		result = get_user(val, ip);
		if (result)
			return result;
		sdp->sgdebug = (u8)val;
		return 0;
	case BLKSECTGET:
		SG_LOG(3, sdp, "%s:    BLKSECTGET\n", __func__);
		return put_user(max_sectors_bytes(sdev->request_queue), ip);
	case BLKTRACESETUP:
		SG_LOG(3, sdp, "%s:    BLKTRACESETUP\n", __func__);
		return blk_trace_setup(sdev->request_queue,
				       sdp->disk->disk_name,
				       MKDEV(SCSI_GENERIC_MAJOR, sdp->index),
				       NULL, p);
	case BLKTRACESTART:
		SG_LOG(3, sdp, "%s:    BLKTRACESTART\n", __func__);
		return blk_trace_startstop(sdev->request_queue, 1);
	case BLKTRACESTOP:
		SG_LOG(3, sdp, "%s:    BLKTRACESTOP\n", __func__);
		return blk_trace_startstop(sdev->request_queue, 0);
	case BLKTRACETEARDOWN:
		SG_LOG(3, sdp, "%s:    BLKTRACETEARDOWN\n", __func__);
		return blk_trace_remove(sdev->request_queue);
	case SCSI_IOCTL_GET_IDLUN:
		SG_LOG(3, sdp, "%s:    SCSI_IOCTL_GET_IDLUN %s\n", __func__,
		       pmlp);
		check_detach = true;
		break;
	case SCSI_IOCTL_GET_BUS_NUMBER:
		SG_LOG(3, sdp, "%s:    SCSI_IOCTL_GET_BUS_NUMBER%s\n",
		       __func__, pmlp);
		check_detach = true;
		break;
	case SCSI_IOCTL_PROBE_HOST:
		SG_LOG(3, sdp, "%s:    SCSI_IOCTL_PROBE_HOST%s\n", __func__,
		       pmlp);
		check_detach = true;
		break;
	case SG_GET_TRANSFORM:
		SG_LOG(3, sdp, "%s:    SG_GET_TRANSFORM%s\n", __func__, pmlp);
		check_detach = true;
		break;
	case SG_SET_TRANSFORM:
		SG_LOG(3, sdp, "%s:    SG_SET_TRANSFORM%s\n", __func__, pmlp);
		check_detach = true;
		break;
	case SG_SCSI_RESET:
		SG_LOG(3, sdp, "%s:    SG_SCSI_RESET\n", __func__);
		check_detach = true;
		break;
	default:
		SG_LOG(3, sdp, "%s:    unrecognized ioctl [0x%x]%s\n",
		       __func__, cmd_in, pmlp);
		if (read_only)
			return -EPERM;  /* don't know, so take safer approach */
		break;
	}

	if (check_detach) {
		if (unlikely(atomic_read(&sdp->detaching)))
			return -ENODEV;
	}
	result = scsi_ioctl_block_when_processing_errors(sdev, cmd_in,
						 filp->f_flags & O_NDELAY);
	if (result)
		return result;
	/* ioctl that reach here are forwarded to the mid-level */
	return scsi_ioctl(sdev, cmd_in, p);
}

#if IS_ENABLED(CONFIG_COMPAT)
static long
sg_compat_ioctl(struct file *filp, unsigned int cmd_in, unsigned long arg)
{
	struct sg_device *sdp;
	struct sg_fd *sfp;
	struct scsi_device *sdev;

	sfp = filp->private_data;
	if (IS_ERR_OR_NULL(sfp)) {
		pr_warn("sg: %s: sfp is NULL or error\n", __func__);
		return IS_ERR(sfp) ? PTR_ERR(sfp) : -ENXIO;
	}
	sdp = sfp->parentdp;
	SG_LOG(3, sdp, "%s: cmd=0x%x\n", __func__, (int)cmd_in);
	if (IS_ERR_OR_NULL(sdp))
		return IS_ERR(sdp) ? PTR_ERR(sdp) : -ENXIO;

	sdev = sdp->device;
	if (sdev->host->hostt->compat_ioctl) {
		int ret;

		ret = sdev->host->hostt->compat_ioctl(sdev, cmd_in,
						      uptr64(arg));
		return ret;
	}
	return -ENOIOCTLCMD;
}
#endif

/*
 * Implements the poll(2) system call for this driver. Returns various EPOLL*
 * flags OR-ed together.
 */
static __poll_t
sg_poll(struct file *filp, poll_table * wait)
{
	__poll_t p_res = 0;
	struct sg_device *sdp;
	struct sg_fd *sfp;

	sfp = filp->private_data;
	if (IS_ERR_OR_NULL(sfp))
		return EPOLLERR;
	sdp = sfp->parentdp;
	if (IS_ERR_OR_NULL(sdp))
		return EPOLLERR;
	poll_wait(filp, &sfp->read_wait, wait);
	if (atomic_read(&sfp->waiting) > 0)
		p_res = EPOLLIN | EPOLLRDNORM;

	if (unlikely(atomic_read(&sdp->detaching)))
		p_res |= EPOLLHUP;
	else if (likely(test_bit(SG_FFD_CMD_Q, sfp->ffd_bm)))
		p_res |= EPOLLOUT | EPOLLWRNORM;
	else if (atomic_read(&sfp->submitted) == 0)
		p_res |= EPOLLOUT | EPOLLWRNORM;
	SG_LOG(3, sdp, "%s: p_res=0x%x\n", __func__, (__force u32)p_res);
	return p_res;
}

/* Entry point for fasync() related to fcntl(SET_FL(flags | O_ASYNC)) call */
static int
sg_fasync(int fd, struct file *filp, int mode)
{
	struct sg_device *sdp;
	struct sg_fd *sfp;

	sfp = filp->private_data;
	if (IS_ERR_OR_NULL(sfp)) {
		pr_warn("sg: %s: sfp is NULL or error\n", __func__);
		return IS_ERR(sfp) ? PTR_ERR(sfp) : -ENXIO;
	}
	sdp = sfp->parentdp;
	SG_LOG(3, sdp, "%s: mode(%s)\n", __func__, (mode ? "add" : "remove"));
	if (IS_ERR_OR_NULL(sdp))
		return IS_ERR(sdp) ? PTR_ERR(sdp) : -ENXIO;
	return fasync_helper(fd, filp, mode, &sfp->async_qp);
}

/* Note: the error return: VM_FAULT_SIGBUS causes a "bus error" */
static vm_fault_t
sg_vma_fault(struct vm_fault *vmf)
{
	int k, length;
	unsigned long offset, len, sa, iflags;
	struct vm_area_struct *vma = vmf->vma;
	struct sg_scatter_hold *rsv_schp;
	struct sg_request *srp;
	struct sg_device *sdp;
	struct sg_fd *sfp;
	const char *nbp = "==NULL, bad";

	if (!vma) {
		pr_warn("%s: vma%s\n", __func__, nbp);
		goto out_err;
	}
	sfp = vma->vm_private_data;
	if (IS_ERR_OR_NULL(sfp)) {
		pr_warn("%s: sfp%s\n", __func__, nbp);
		goto out_err;
	}
	sdp = sfp->parentdp;
	if (sdp && unlikely(atomic_read(&sdp->detaching))) {
		SG_LOG(1, sdp, "%s: device deatching\n", __func__);
		goto out_err;
	}
	srp = sfp->rsv_srp;
	if (!srp) {
		SG_LOG(1, sdp, "%s: srp%s\n", __func__, nbp);
		goto out_err;
	}
	spin_lock_irqsave(&srp->req_lck, iflags);
	rsv_schp = srp->sgatp;
	offset = vmf->pgoff << PAGE_SHIFT;
	if (offset >= (unsigned int)rsv_schp->buflen) {
		SG_LOG(1, sdp, "%s: offset[%lu] >= rsv.buflen\n", __func__,
		       offset);
		goto out_err_unlock;
	}
	sa = vma->vm_start;
	SG_LOG(3, sdp, "%s: vm_start=0x%lx, off=%lu\n", __func__, sa, offset);
	length = 1 << (PAGE_SHIFT + rsv_schp->page_order);
	for (k = 0; k < rsv_schp->num_sgat && sa < vma->vm_end; ++k) {
		len = vma->vm_end - sa;
		len = min_t(int, len, (int)length);
		if (offset < len) {
			struct page *page;
			struct page *pp;

			pp = rsv_schp->pages[k];
			spin_unlock_irqrestore(&srp->req_lck, iflags);
			page = nth_page(pp, offset >> PAGE_SHIFT);
			get_page(page); /* increment page count */
			vmf->page = page;
			return 0; /* success */
		}
		sa += len;
		offset -= len;
	}
out_err_unlock:
	spin_unlock_irqrestore(&srp->req_lck, iflags);
out_err:
	return VM_FAULT_SIGBUS;
}

static const struct vm_operations_struct sg_mmap_vm_ops = {
	.fault = sg_vma_fault,
};

/* Entry point for mmap(2) system call */
static int
sg_mmap(struct file *filp, struct vm_area_struct *vma)
{
	int k, length;
	int ret = 0;
	unsigned long req_sz, len, sa, iflags;
	struct sg_scatter_hold *rsv_schp;
	struct sg_fd *sfp;
	struct sg_request *srp;

	if (!filp || !vma)
		return -ENXIO;
	sfp = filp->private_data;
	if (IS_ERR_OR_NULL(sfp)) {
		pr_warn("sg: %s: sfp is NULL or error\n", __func__);
		return IS_ERR(sfp) ? PTR_ERR(sfp) : -ENXIO;
	}
	req_sz = vma->vm_end - vma->vm_start;
	SG_LOG(3, sfp->parentdp, "%s: vm_start=%p, len=%d\n", __func__,
	       (void *)vma->vm_start, (int)req_sz);
	if (vma->vm_pgoff || IS_ERR_OR_NULL(sfp->parentdp))
		return -EINVAL; /* only an offset of 0 accepted */
	/* Check reserve request is inactive and has large enough buffer */
	mutex_lock(&sfp->f_mutex);
	srp = sfp->rsv_srp;
	spin_lock_irqsave(&srp->req_lck, iflags);
	if (SG_RS_ACTIVE(srp)) {
		ret = -EBUSY;
		goto out;
	}
	rsv_schp = srp->sgatp;
	if (req_sz > (unsigned long)rsv_schp->buflen) {
		ret = -ENOMEM;
		goto out;
	}
	sa = vma->vm_start;
	length = 1 << (PAGE_SHIFT + rsv_schp->page_order);
	for (k = 0; k < rsv_schp->num_sgat && sa < vma->vm_end; ++k) {
		len = vma->vm_end - sa;
		len = min_t(unsigned long, len, (unsigned long)length);
		sa += len;
	}

	set_bit(SG_FFD_MMAP_CALLED, sfp->ffd_bm);
	vma->vm_flags |= VM_IO | VM_DONTEXPAND | VM_DONTDUMP;
	vma->vm_private_data = sfp;
	vma->vm_ops = &sg_mmap_vm_ops;
out:
	spin_unlock_irqrestore(&srp->req_lck, iflags);
	mutex_unlock(&sfp->f_mutex);
	return ret;
}

/*
 * This user context function is called from sg_rq_end_io() in 2 situations.
 * The first case is when a slave was in SWAIT state and sg_rq_end_io() has
 * just been called on the corresponding master request.
 * The second case is an orphaned request that needs to be cleaned up (e.g.
 * when control C is typed while an ioctl(SG_IO) is active).
 */
static void
sg_rq_end_io_usercontext(struct work_struct *work)
{
	struct sg_request *srp = container_of(work, struct sg_request,
					      ew_orph.work);
	struct sg_fd *sfp;
	struct sg_request *slrp;

	if (!srp) {
		WARN_ONCE("s: srp unexpectedly NULL\n", __func__);
		return;
	}
	sfp = srp->parentfp;
	if (!sfp) {
		WARN_ONCE(1, "%s: sfp unexpectedly NULL\n", __func__);
		return;
	}
	slrp = sfp->slave_srp;
	SG_LOG(3, sfp->parentdp, "%s: %s srp=0x%p, rq_st: %s, share: %s\n",
	       __func__, (slrp ? "swait" : "clean"), srp,
	       sg_rq_st_str(atomic_read(&srp->rq_st), false),
	       sg_shr_str(srp->sh_var, false));
	if (slrp) {
		unsigned long iflags;
		enum sg_rq_state rqq_st;
		struct sg_fd *slave_sfp = slrp->parentfp;

		spin_lock_irqsave(&slrp->req_lck, iflags);
		rqq_st = atomic_read(&slrp->rq_st);
		if (slave_sfp && rqq_st == SG_RS_SHR_SWAIT) {
			if (!(srp->rq_result & SG_ML_RESULT_MSK)) {
				/* master is error/sense free */
				sg_rstate_chg(slrp, rqq_st, SG_RS_INFLIGHT);
				spin_unlock_irqrestore(&slrp->req_lck, iflags);
				sg_execute_cmd(slave_sfp, slrp);
				goto chk_second;
			}
			/* end slave cmd with ::driver_status=DRIVER_SOFT */
			sg_rstate_chg(slrp, rqq_st, SG_RS_AWAIT_RD);
			spin_unlock_irqrestore(&slrp->req_lck, iflags);
			slrp->rq_result |= (DRIVER_SOFT << 24);
			if (slave_sfp) {
				wake_up_interruptible(&slave_sfp->read_wait);
				kill_fasync(&slave_sfp->async_qp, SIGPOLL,
					    POLL_IN);
			}
		} else {
			spin_unlock_irqrestore(&slrp->req_lck, iflags);
		}
	}
chk_second:
	if (test_bit(SG_FRQ_DEACT_ORPHAN, srp->frq_bm)) {
		sg_finish_scsi_blk_rq(srp);	/* clean up orphan case */
		sg_deact_request(sfp, srp);
	}
	kref_put(&sfp->f_ref, sg_remove_sfp);
	return;
}

static void
sg_check_sense(struct sg_device *sdp, struct sg_request *srp, int sense_len)
{
	int driver_stat;
	u32 rq_res = srp->rq_result;
	struct scsi_request *scsi_rp = scsi_req(srp->rq);
	u8 *sbp = scsi_rp ? scsi_rp->sense : NULL;

	if (!sbp)
		return;
	driver_stat = driver_byte(rq_res);
	if (driver_stat & DRIVER_SENSE) {
		struct scsi_sense_hdr ssh;

		if (scsi_normalize_sense(sbp, sense_len, &ssh)) {
			if (!scsi_sense_is_deferred(&ssh)) {
				if (ssh.sense_key == UNIT_ATTENTION) {
					if (sdp->device->removable)
						sdp->device->changed = 1;
				}
			}
		}
	}
	if (sdp->sgdebug > 0) {
		int scsi_stat = rq_res & 0xff;

		if (scsi_stat == SAM_STAT_CHECK_CONDITION ||
		    scsi_stat == SAM_STAT_COMMAND_TERMINATED)
			__scsi_print_sense(sdp->device, __func__, sbp,
					   sense_len);
	}
}

/*
 * This "bottom half" (soft interrupt) handler is called by the mid-level
 * when a request has completed or failed. This callback is registered in a
 * blk_execute_rq_nowait() call in the sg_common_write(). For ioctl(SG_IO)
 * (sync) usage, sg_ctl_sg_io() waits to be woken up by this callback.
 */
static void
sg_rq_end_io(struct request *rq, blk_status_t status)
{
	struct sg_request *srp = rq->end_io_data;
	struct scsi_request *scsi_rp = scsi_req(rq);
	struct sg_device *sdp;
	struct sg_fd *sfp;
	int a_resid, slen;
	enum sg_rq_state rqq_state = SG_RS_AWAIT_RD;

	if (WARN_ON(atomic_read(&srp->rq_st) != SG_RS_INFLIGHT)) {
		if (srp)
			pr_warn("%s: bad rq_st=%d\n", __func__,
				atomic_read(&srp->rq_st));
		else
			pr_warn("%s: srp is NULL\n", __func__);
		return;
	}
	sfp = srp->parentfp;
	if (unlikely(!sfp)) {
		WARN_ONCE(1, "%s: sfp unexpectedly NULL", __func__);
		return;
	}
	SG_TRC(__func__, NULL, SG_TRC_REQ, 2, srp, true, false);
	sdp = sfp->parentdp;
	if (unlikely(atomic_read(&sdp->detaching)))
		pr_info("%s: device detaching\n", __func__);

	srp->rq_result = scsi_rp->result;
	slen = min_t(int, scsi_rp->sense_len, SCSI_SENSE_BUFFERSIZE);
	a_resid = scsi_rp->resid_len;

	if (a_resid) {
		if (test_bit(SG_FRQ_IS_V4I, srp->frq_bm)) {
			if (rq_data_dir(rq) == READ)
				srp->in_resid = a_resid;
			else
				srp->s_hdr4.out_resid = a_resid;
		} else {
			srp->in_resid = a_resid;
		}
	}
	if (test_bit(SG_FRQ_ABORTING, srp->frq_bm) && srp->rq_result == 0)
		srp->rq_result |= (DRIVER_HARD << 24);

	SG_LOG(6, sdp, "%s: pack_id=%d, tag=%d, res=0x%x\n", __func__,
	       srp->pack_id, srp->tag, srp->rq_result);
	srp->duration = sg_calc_req_time(srp,
				test_bit(SG_FFD_TIME_IN_NS, sfp->ffd_bm));
	if (unlikely((srp->rq_result & SG_ML_RESULT_MSK) && slen > 0))
		sg_check_sense(sdp, srp, slen);
	if (slen > 0) {
		if (scsi_rp->sense) {
			srp->sense_bp = mempool_alloc(sg_sense_pool,
						      GFP_ATOMIC);
			if (srp->sense_bp) {
				memcpy(srp->sense_bp, scsi_rp->sense, slen);
				if (slen < SCSI_SENSE_BUFFERSIZE) {
					int d = SCSI_SENSE_BUFFERSIZE - slen;

					memset(srp->sense_bp + slen, 0, d);
				}
			} else {
				pr_warn("%s: bad mempool_alloc\n", __func__);
				slen = 0;
			}
		} else {
			pr_warn("%s: scsi_request::sense==NULL\n", __func__);
			slen = 0;
		}
	}
	srp->sense_len = slen;
	if (unlikely(test_bit(SG_FRQ_IS_ORPHAN, srp->frq_bm))) {
		spin_lock(&srp->req_lck);
		if (test_bit(SG_FFD_KEEP_ORPHAN, sfp->ffd_bm)) {
			clear_bit(SG_FRQ_SYNC_INVOC, srp->frq_bm);
		} else {
			rqq_state = SG_RS_BUSY;
			set_bit(SG_FRQ_DEACT_ORPHAN, srp->frq_bm);
		}
		spin_unlock(&srp->req_lck);
	}
	if (!test_bit(SG_FRQ_SYNC_INVOC, srp->frq_bm))
		atomic_inc(&sfp->waiting);
	if (unlikely(sg_rstate_chg(srp, SG_RS_INFLIGHT, rqq_state)))
		pr_warn("%s: can't set rq_st\n", __func__);
	/*
	 * Free the mid-level resources apart from the bio (if any). The bio's
	 * blk_rq_unmap_user() can be called later from user context.
	 */
	srp->rq = NULL;
	scsi_req_free_cmd(scsi_rp);
	blk_put_request(rq);

	if (likely(rqq_state == SG_RS_AWAIT_RD)) {
		/* Wake any sg_read()/ioctl(SG_IORECEIVE) awaiting this req */
		wake_up_interruptible(&sfp->read_wait);
		kill_fasync(&sfp->async_qp, SIGPOLL, POLL_IN);
		if (sfp->slave_srp &&	/* schedule waiting slave rq */
		    atomic_read(&sfp->slave_srp->rq_st) == SG_RS_SHR_SWAIT) {
			INIT_WORK(&srp->ew_orph.work,
				  sg_rq_end_io_usercontext);
			schedule_work(&srp->ew_orph.work);
		} else {
			kref_put(&sfp->f_ref, sg_remove_sfp);
		}
	} else {        /* clean up orphaned request that aren't being kept */
		INIT_WORK(&srp->ew_orph.work, sg_rq_end_io_usercontext);
		schedule_work(&srp->ew_orph.work);
	}
}


static const struct file_operations sg_fops = {
	.owner = THIS_MODULE,
	.read = sg_read,
	.write = sg_write,
	.poll = sg_poll,
	.unlocked_ioctl = sg_ioctl,
#if IS_ENABLED(CONFIG_COMPAT)
	.compat_ioctl = sg_compat_ioctl,
#endif
	.open = sg_open,
	.mmap = sg_mmap,
	.release = sg_release,
	.fasync = sg_fasync,
	.llseek = no_llseek,
};

static struct class *sg_sysfs_class;

static int sg_sysfs_valid = 0;

/* Returns valid pointer to sg_device or negated errno twisted by ERR_PTR */
static struct sg_device *
sg_alloc(struct gendisk *disk, struct scsi_device *scsidp)
{
	struct sg_device *sdp;
	int error;
	u32 k;
	unsigned long iflags;

	sdp = kzalloc(sizeof(struct sg_device), GFP_KERNEL);
	if (!sdp)
		return ERR_PTR(-ENOMEM);

	idr_preload(GFP_KERNEL);
	write_lock_irqsave(&sg_index_lock, iflags);

	error = idr_alloc(&sg_index_idr, sdp, 0, SG_MAX_DEVS, GFP_NOWAIT);
	if (error < 0) {
		if (error == -ENOSPC) {
			sdev_printk(KERN_WARNING, scsidp,
				    "Unable to attach sg device type=%d, minor number exceeds %d\n",
				    scsidp->type, SG_MAX_DEVS - 1);
			error = -ENODEV;
		} else {
			sdev_printk(KERN_WARNING, scsidp,
				"%s: idr allocation sg_device failure: %d\n",
				    __func__, error);
		}
		goto out_unlock;
	}
	k = error;

	SCSI_LOG_TIMEOUT(3, sdev_printk(KERN_INFO, scsidp,
			 "%s: dev=%d, sdp=0x%p ++\n", __func__, k, sdp));
	sprintf(disk->disk_name, "sg%d", k);
	disk->first_minor = k;
	sdp->disk = disk;
	sdp->device = scsidp;
	mutex_init(&sdp->open_rel_lock);
	INIT_LIST_HEAD(&sdp->sfds);
	init_waitqueue_head(&sdp->open_wait);
	atomic_set(&sdp->detaching, 0);
	rwlock_init(&sdp->sfd_llock);
	sg_calc_sgat_param(sdp);
	sdp->index = k;
	kref_init(&sdp->d_ref);
	error = 0;

out_unlock:
	write_unlock_irqrestore(&sg_index_lock, iflags);
	idr_preload_end();

	if (error) {
		kfree(sdp);
		return ERR_PTR(error);
	}
	return sdp;
}

static int
sg_add_device(struct device *cl_dev, struct class_interface *cl_intf)
{
	struct scsi_device *scsidp = to_scsi_device(cl_dev->parent);
	struct gendisk *disk;
	struct sg_device *sdp = NULL;
	struct cdev * cdev = NULL;
	int error;
	unsigned long iflags;

	disk = alloc_disk(1);
	if (!disk) {
		pr_warn("%s: alloc_disk failed\n", __func__);
		return -ENOMEM;
	}
	disk->major = SCSI_GENERIC_MAJOR;

	error = -ENOMEM;
	cdev = cdev_alloc();
	if (!cdev) {
		pr_warn("%s: cdev_alloc failed\n", __func__);
		goto out;
	}
	cdev->owner = THIS_MODULE;
	cdev->ops = &sg_fops;

	sdp = sg_alloc(disk, scsidp);
	if (IS_ERR(sdp)) {
		pr_warn("%s: sg_alloc failed\n", __func__);
		error = PTR_ERR(sdp);
		goto out;
	}

	error = cdev_add(cdev, MKDEV(SCSI_GENERIC_MAJOR, sdp->index), 1);
	if (error)
		goto cdev_add_err;

	sdp->cdev = cdev;
	if (sg_sysfs_valid) {
		struct device *sg_class_member;

		sg_class_member = device_create(sg_sysfs_class, cl_dev->parent,
						MKDEV(SCSI_GENERIC_MAJOR,
						      sdp->index),
						sdp, "%s", disk->disk_name);
		if (IS_ERR(sg_class_member)) {
			pr_err("%s: device_create failed\n", __func__);
			error = PTR_ERR(sg_class_member);
			goto cdev_add_err;
		}
		error = sysfs_create_link(&scsidp->sdev_gendev.kobj,
					  &sg_class_member->kobj, "generic");
		if (error)
			pr_err("%s: unable to make symlink 'generic' back "
			       "to sg%d\n", __func__, sdp->index);
	} else
		pr_warn("%s: sg_sys Invalid\n", __func__);

	sdev_printk(KERN_NOTICE, scsidp, "Attached scsi generic sg%d "
		    "type %d\n", sdp->index, scsidp->type);

	dev_set_drvdata(cl_dev, sdp);
	return 0;

cdev_add_err:
	write_lock_irqsave(&sg_index_lock, iflags);
	idr_remove(&sg_index_idr, sdp->index);
	write_unlock_irqrestore(&sg_index_lock, iflags);
	kfree(sdp);

out:
	put_disk(disk);
	if (cdev)
		cdev_del(cdev);
	return error;
}

static void
sg_device_destroy(struct kref *kref)
{
	struct sg_device *sdp = container_of(kref, struct sg_device, d_ref);
	unsigned long iflags;

	/* CAUTION!  Note that the device can still be found via idr_find()
	 * even though the refcount is 0.  Therefore, do idr_remove() BEFORE
	 * any other cleanup.
	 */

	write_lock_irqsave(&sg_index_lock, iflags);
	idr_remove(&sg_index_idr, sdp->index);
	write_unlock_irqrestore(&sg_index_lock, iflags);

	SG_LOG(3, sdp, "%s: sdp=0x%p --\n", __func__, sdp);

	put_disk(sdp->disk);
	kfree(sdp);
}

static void
sg_remove_device(struct device *cl_dev, struct class_interface *cl_intf)
{
	struct scsi_device *scsidp = to_scsi_device(cl_dev->parent);
	struct sg_device *sdp = dev_get_drvdata(cl_dev);
	unsigned long iflags;
	struct sg_fd *sfp;
	int val;

	if (!sdp)
		return;
	/* want sdp->detaching non-zero as soon as possible */
	val = atomic_inc_return(&sdp->detaching);
	if (val > 1)
		return; /* only want to do following once per device */

	SG_LOG(3, sdp, "%s: 0x%p\n", __func__, sdp);

	read_lock_irqsave(&sdp->sfd_llock, iflags);
	list_for_each_entry(sfp, &sdp->sfds, sfd_entry) {
		wake_up_interruptible_all(&sfp->read_wait);
		kill_fasync(&sfp->async_qp, SIGPOLL, POLL_HUP);
	}
	wake_up_interruptible_all(&sdp->open_wait);
	read_unlock_irqrestore(&sdp->sfd_llock, iflags);

	sysfs_remove_link(&scsidp->sdev_gendev.kobj, "generic");
	device_destroy(sg_sysfs_class, MKDEV(SCSI_GENERIC_MAJOR, sdp->index));
	cdev_del(sdp->cdev);
	sdp->cdev = NULL;

	kref_put(&sdp->d_ref, sg_device_destroy);
}

module_param_named(scatter_elem_sz, scatter_elem_sz, int, S_IRUGO | S_IWUSR);
module_param_named(def_reserved_size, def_reserved_size, int,
		   S_IRUGO | S_IWUSR);
module_param_named(allow_dio, sg_allow_dio, int, S_IRUGO | S_IWUSR);

MODULE_AUTHOR("Douglas Gilbert");
MODULE_DESCRIPTION("SCSI generic (sg) driver");
MODULE_LICENSE("GPL");
MODULE_VERSION(SG_VERSION_STR);
MODULE_ALIAS_CHARDEV_MAJOR(SCSI_GENERIC_MAJOR);

MODULE_PARM_DESC(scatter_elem_sz,
	"scatter gather element size (def: max(SG_SCATTER_SZ, PAGE_SIZE))");
MODULE_PARM_DESC(def_reserved_size, "size of request reserved for each fd");
MODULE_PARM_DESC(allow_dio, "allow direct I/O (default: 0 (disallow))");

static int __init
init_sg(void)
{
	int rc;

	/* check scatter_elem_sz module parameter, change if inappropriate */
	if (scatter_elem_sz < (int)PAGE_SIZE)
		scatter_elem_sz = PAGE_SIZE;
	else if (scatter_elem_sz != (1 << ilog2(scatter_elem_sz)))
		scatter_elem_sz = 1 << ilog2(scatter_elem_sz);
	/* scatter_elem_sz rounded down to power of 2, or PAGE_SIZE */
	if (def_reserved_size >= 0)
		sg_big_buff = def_reserved_size;
	else
		def_reserved_size = sg_big_buff;

	rc = register_chrdev_region(MKDEV(SCSI_GENERIC_MAJOR, 0),
				    SG_MAX_DEVS, "sg");
	if (rc)
		return rc;

	sg_sense_cache = kmem_cache_create("sg_sense", SCSI_SENSE_BUFFERSIZE,
					   0, 0, NULL);
	if (!sg_sense_cache) {
		pr_err("sg: can't init sense cache\n");
		rc = -ENOMEM;
		goto err_out_unreg;
	}
	sg_sense_pool = mempool_create_slab_pool(SG_MEMPOOL_MIN_NR,
						 sg_sense_cache);
	if (!sg_sense_pool) {
		pr_err("sg: can't init sense pool\n");
		rc = -ENOMEM;
		goto err_out_cache;
	}

	pr_info("Registered %s[char major=0x%x], version: %s, date: %s\n",
		"sg device ", SCSI_GENERIC_MAJOR, SG_VERSION_STR,
		sg_version_date);
	sg_sysfs_class = class_create(THIS_MODULE, "scsi_generic");
	if (IS_ERR(sg_sysfs_class)) {
		rc = PTR_ERR(sg_sysfs_class);
		goto err_out_pool;
	}
	sg_sysfs_valid = 1;
	rc = scsi_register_interface(&sg_interface);
	if (0 == rc) {
		sg_proc_init();
		return 0;
	}
	class_destroy(sg_sysfs_class);

err_out_pool:
	mempool_destroy(sg_sense_pool);
err_out_cache:
	kmem_cache_destroy(sg_sense_cache);
err_out_unreg:
	unregister_chrdev_region(MKDEV(SCSI_GENERIC_MAJOR, 0), SG_MAX_DEVS);
	return rc;
}

#if !IS_ENABLED(CONFIG_SCSI_PROC_FS)
static int
sg_proc_init(void)
{
	return 0;
}
#endif

static void __exit
exit_sg(void)
{
	if (IS_ENABLED(CONFIG_SCSI_PROC_FS)) {
		remove_proc_subtree("scsi/sg", NULL);
		if (IS_ENABLED(SG_DEBUG)) {
			kfree(sg_trc_wp);
			kfree(sg_trc_cb.buf);
		}
	}
	scsi_unregister_interface(&sg_interface);
	mempool_destroy(sg_sense_pool);
	kmem_cache_destroy(sg_sense_cache);
	class_destroy(sg_sysfs_class);
	sg_sysfs_valid = 0;
	unregister_chrdev_region(MKDEV(SCSI_GENERIC_MAJOR, 0),
				 SG_MAX_DEVS);
	idr_destroy(&sg_index_idr);
}

/*
 * Setup to move data between kernel buffers managed by this driver and a SCSI
 * device. Note that there is no corresponding 'unmap' call as is required by
 * blk_rq_map_user() . blk_rq has already been told direction of transfer.
 */
static int
sg_rq_map_kern(struct sg_request *srp, struct request_queue *q,
	       struct request *rq)
{
	int k, pg_sz, dlen, ln;
	int res = 0;
	struct sg_scatter_hold *schp = srp->sgatp;

	dlen = schp->dlen;
	pg_sz = 1 << (PAGE_SHIFT + schp->page_order);
	SG_LOG(4, srp->parentfp->parentdp, "%s: dlen=%d, pg_sz=%d\n",
	       __func__, dlen, pg_sz);
	for (k = 0; k < schp->num_sgat && dlen > 0; ++k, dlen -= ln) {
		ln = min_t(int, dlen, pg_sz);
		res = blk_rq_map_kern(q, rq, page_address(schp->pages[k]),
				      ln, GFP_ATOMIC);
		if (res)
			break;
	}
	return res;
}

static inline bool
sg_chk_dio_allowed(struct sg_device *sdp, struct sg_fd *sfp,
		   struct sg_request *srp, int iov_count, int dir)
{
	if (sg_allow_dio && (srp->rq_flags & SG_FLAG_DIRECT_IO)) {
		if (dir != SG_DXFER_UNKNOWN && !iov_count) {
			if (!sdp->device->host->unchecked_isa_dma)
				return true;
		}
	}
	return false;
}

static inline void
sg_set_map_data(const struct sg_scatter_hold *schp, bool up_valid,
		struct rq_map_data *mdp)
{
	memset(mdp, 0, sizeof(*mdp));
	mdp->pages = schp->pages;
	mdp->page_order = schp->page_order;
	mdp->nr_entries = schp->num_sgat;
	mdp->offset = 0;
	mdp->null_mapped = !up_valid;
}

static int
sg_start_req(struct sg_request *srp, u8 *cmd, int cmd_len,
	     struct sg_io_v4 *h4p, int dxfer_dir)
{
	bool reserved, us_xfer;
	int res = 0;
	int dxfer_len = 0;
	int r0w = READ;
	unsigned int iov_count = 0;
	void __user *up = NULL;
	struct request *rq = NULL;
	struct scsi_request *scsi_rp;
	struct sg_fd *sfp = srp->parentfp;
	struct sg_device *sdp;
	struct sg_scatter_hold *req_schp = NULL;
	struct request_queue *q;
	struct rq_map_data *md = (void *)srp; /* want any non-NULL value */
	u8 *long_cmdp = NULL;
	__maybe_unused const char *cp = "";
	__maybe_unused const char *c2p = "";
	__maybe_unused char b[32];
	struct rq_map_data map_data;

	sdp = sfp->parentdp;
	if (cmd_len > BLK_MAX_CDB) {	/* for longer SCSI cdb_s */
		long_cmdp = kzalloc(cmd_len, GFP_KERNEL);
		if (!long_cmdp)
			return -ENOMEM;
		SG_LOG(5, sdp, "%s: long_cmdp=0x%p ++\n", __func__, long_cmdp);
	}
	if (h4p) {
		if (dxfer_dir == SG_DXFER_TO_DEV) {
			r0w = WRITE;
			up = uptr64(h4p->dout_xferp);
			dxfer_len = (int)h4p->dout_xfer_len;
			iov_count = h4p->dout_iovec_count;
		} else if (dxfer_dir == SG_DXFER_FROM_DEV) {
			r0w = READ;
			up = uptr64(h4p->din_xferp);
			dxfer_len = (int)h4p->din_xfer_len;
			iov_count = h4p->din_iovec_count;
		}
	} else {
		struct sg_slice_hdr3 *sh3p = &srp->s_hdr3;

		up = sh3p->dxferp;
		dxfer_len = (int)sh3p->dxfer_len;
		iov_count = sh3p->iovec_count;
		r0w = dxfer_dir == SG_DXFER_TO_DEV ? WRITE : READ;
	}
	SG_LOG(4, sdp, "%s: dxfer_len=%d, data-%s\n", __func__, dxfer_len,
	       (r0w ? "OUT" : "IN"));
	q = sdp->device->request_queue;

	/*
	 * NOTE
	 *
	 * With scsi-mq enabled, there are a fixed number of preallocated
	 * requests equal in number to shost->can_queue.  If all of the
	 * preallocated requests are already in use, then blk_get_request()
	 * will sleep until an active command completes, freeing up a request.
	 * Although waiting in an asynchronous interface is less than ideal, we
	 * do not want to use BLK_MQ_REQ_NOWAIT here because userspace might
	 * not expect an EWOULDBLOCK from this condition.
	 */
	rq = blk_get_request(q, (r0w ? REQ_OP_SCSI_OUT : REQ_OP_SCSI_IN), 0);
	if (unlikely(IS_ERR(rq))) {
		kfree(long_cmdp);
		return PTR_ERR(rq);
	}
	/* current sg_request protected by SG_RS_BUSY state */
	scsi_rp = scsi_req(rq);

	if (cmd_len > BLK_MAX_CDB)
		scsi_rp->cmd = long_cmdp;
	memcpy(scsi_rp->cmd, cmd, cmd_len);
	scsi_rp->cmd_len = cmd_len;
	us_xfer = !(srp->rq_flags & SG_FLAG_NO_DXFER);
	assign_bit(SG_FRQ_NO_US_XFER, srp->frq_bm, !us_xfer);
	reserved = (sfp->rsv_srp == srp);
	srp->rq = rq;
	rq->end_io_data = srp;
	scsi_rp->retries = SG_DEFAULT_RETRIES;
	req_schp = srp->sgatp;

	if (dxfer_len <= 0 || dxfer_dir == SG_DXFER_NONE) {
		SG_LOG(4, sdp, "%s: no data xfer [0x%p]\n", __func__, srp);
		set_bit(SG_FRQ_NO_US_XFER, srp->frq_bm);
		goto fini;
	} else if (sg_chk_dio_allowed(sdp, sfp, srp, iov_count, dxfer_dir) &&
		   blk_rq_aligned(q, (unsigned long)up, dxfer_len)) {
		set_bit(SG_FRQ_DIO_IN_USE, srp->frq_bm);
		srp->rq_info |= SG_INFO_DIRECT_IO;
		md = NULL;
		if (IS_ENABLED(CONFIG_SCSI_PROC_FS))
			cp = "direct_io, ";
	} else {	/* normal IO and failed conditions for dio path */
		md = &map_data;
	}

	if (likely(md)) {	/* normal, "indirect" IO */
		if (unlikely((srp->rq_flags & SG_FLAG_MMAP_IO))) {
			/* mmap IO must use and fit in reserve request */
			if (!reserved || dxfer_len > req_schp->buflen)
				res = reserved ? -ENOMEM : -EBUSY;
		} else if (req_schp->buflen == 0) {
			int up_sz = max_t(int, dxfer_len, sfp->sgat_elem_sz);

			res = sg_mk_sgat(srp, sfp, up_sz);
		}
		if (res)
			goto fini;

		sg_set_map_data(req_schp, !!up, md);
		md->from_user = (dxfer_dir == SG_DXFER_TO_FROM_DEV);
	}

	if (unlikely(iov_count)) {
		struct iovec *iov = NULL;
		struct iov_iter i;

		res = import_iovec(r0w, up, iov_count, 0, &iov, &i);
		if (res < 0)
			goto fini;

		iov_iter_truncate(&i, dxfer_len);
		if (!iov_iter_count(&i)) {
			kfree(iov);
			res = -EINVAL;
			goto fini;
		}

		if (us_xfer)
			res = blk_rq_map_user_iov(q, rq, md, &i, GFP_ATOMIC);
		kfree(iov);
		if (IS_ENABLED(CONFIG_SCSI_PROC_FS)) {
			snprintf(b, sizeof(b), "iov_count=%d ", iov_count);
			cp = b;
		}
	} else if (us_xfer) { /* setup for transfer data to/from user space */
		res = blk_rq_map_user(q, rq, md, up, dxfer_len, GFP_ATOMIC);
		if (IS_ENABLED(CONFIG_SCSI_PROC_FS)) {
			if (res)
				c2p = " blk_rq_map_user()";
		}
	} else if (!test_bit(SG_FRQ_DEFER_XFR, srp->frq_bm)) {
		/* transfer data to/from kernel buffers */
		res = sg_rq_map_kern(srp, q, rq);
	}
fini:
	if (likely(res == 0)) {
		res = sg_rstate_chg(srp, SG_RS_BUSY, SG_RS_INFLIGHT);
		if (likely(res == 0))
			srp->bio = rq->bio;
	}
	if (unlikely(res && rq)) {		/* failure, free up resources */
		srp->rq = NULL;
		blk_put_request(rq);
	}
	SG_LOG(4, sdp, "%s: %s %s%s res=%d [0x%p]\n", __func__,
	       sg_shr_str(srp->sh_var, false), cp, c2p, res, srp);
	return res;
}

/*
 * Clean up mid-level and block layer resources of finished request. Sometimes
 * blk_rq_unmap_user() returns -4 (-EINTR) and this is why: "If we're in a
 * workqueue, the request is orphaned, so don't copy into a random user
 * address space, just free and return -EINTR so user space doesn't expect
 * any data." [block/bio.c]
 */
static void
sg_finish_scsi_blk_rq(struct sg_request *srp)
{
	int ret;
	struct sg_fd *sfp = srp->parentfp;

	SG_LOG(4, sfp->parentdp, "%s: srp=0x%p%s\n", __func__, srp,
	       (srp->parentfp->rsv_srp == srp) ? " rsv" : "");
	if (!test_bit(SG_FRQ_SYNC_INVOC, srp->frq_bm)) {
		atomic_dec(&sfp->submitted);
		atomic_dec(&sfp->waiting);
	}
	if (srp->bio) {
		bool us_xfer = !test_bit(SG_FRQ_NO_US_XFER, srp->frq_bm);

		if (us_xfer) {
			ret = blk_rq_unmap_user(srp->bio);
			if (ret) {	/* -EINTR (-4) can be ignored */
				SG_LOG(6, sfp->parentdp,
				       "%s: blk_rq_unmap_user() --> %d\n",
				       __func__, ret);
			}
		}
		srp->bio = NULL;
	}
	/* In worst case READ data returned to user space by this point */
	if (srp->rq) {
		struct request *rq = srp->rq;

		srp->rq = NULL;
		scsi_req_free_cmd(scsi_req(rq));
		blk_put_request(rq);
	}
}

static int
sg_mk_sgat(struct sg_request *srp, struct sg_fd *sfp, int minlen)
{
	int j, k, rem_sz, order, align_sz;
	int m_size = minlen;
	int rup_sz = 0;
	int mx_sgat_elems = sfp->parentdp->max_sgat_elems;
	u32 elem_sz;
	const size_t struct_page_sz = sizeof(struct page *);
	gfp_t mask_ap = GFP_ATOMIC | __GFP_COMP | __GFP_NOWARN | __GFP_ZERO;
	gfp_t mask_kz = GFP_ATOMIC | __GFP_NOWARN;
	struct sg_device *sdp = sfp->parentdp;
	struct sg_scatter_hold *schp = srp->sgatp;

	if (unlikely(m_size < 0))
		return -EFAULT;
	if (unlikely(m_size == 0))
		++m_size;	/* don't remember why */
	/* round request up to next highest SG_DEF_SECTOR_SZ byte boundary */
	align_sz = ALIGN(m_size, SG_DEF_SECTOR_SZ);

	schp->pages = kcalloc(mx_sgat_elems, struct_page_sz, mask_kz);
	SG_LOG(4, sdp, "%s: minlen=%d, align_sz=%d [sz=%zu, 0x%p ++]\n",
	       __func__, minlen, align_sz, mx_sgat_elems * struct_page_sz,
	       schp->pages);
	if (unlikely(!schp->pages))
		return -ENOMEM;

	elem_sz = sfp->sgat_elem_sz;    /* power of 2 and >= PAGE_SIZE */
	if (sdp && unlikely(sdp->device->host->unchecked_isa_dma))
		mask_ap |= GFP_DMA;
	/* PAGE_SIZE == (1 << PAGE_SHIFT) == (2 ** PAGE_SHIFT) */
	order = get_order(elem_sz); /* --> ilog2(1+((elem_sz-1)/PAGE_SIZE)) */
again:
	rup_sz = 1 << (PAGE_SHIFT + order);	/* round up size */

	for (k = 0, rem_sz = align_sz; rem_sz > 0 && k < mx_sgat_elems;
	     ++k, rem_sz -= rup_sz) {
		schp->pages[k] = alloc_pages(mask_ap, order);
		if (!schp->pages[k])
			goto err_out;
		SG_LOG(5, sdp, "%s: k=%d, rup_sz=%d [0x%p ++]\n", __func__, k,
		       rup_sz, schp->pages[k]);
	}
	schp->page_order = order;
	schp->num_sgat = k;
	SG_LOG(5, sdp, "%s: num_sgat=%d, order=%d\n", __func__, k, order);
	if (unlikely(rem_sz > 0))       /* must have failed */
		return -ENOMEM;
	schp->buflen = align_sz;
	if (sfp->tot_fd_thresh > 0)
		atomic_add(align_sz, &sfp->sum_fd_dlens);
	return 0;
err_out:
	for (j = 0; j < k; ++j)
		__free_pages(schp->pages[j], order);

	if (--order >= 0)
		goto again;
	kfree(schp->pages);
	return -ENOMEM;
}

static void
sg_remove_sgat_helper(struct sg_device *sdp, struct sg_scatter_hold *schp)
{
	int k;
	void *p;

	if (!schp->pages)
		return;
	for (k = 0; k < schp->num_sgat; ++k) {
		p = schp->pages[k];
		SG_LOG(5, sdp, "%s: pg[%d]=0x%p --\n", __func__, k, p);
		if (unlikely(!p))
			continue;
		__free_pages(p, schp->page_order);
	}
	SG_LOG(5, sdp, "%s: pg_order=%u, free pgs=0x%p --\n", __func__,
	       schp->page_order, schp->pages);
	kfree(schp->pages);
}

/* Remove the data (possibly a sgat list) held by srp, not srp itself */
static void
sg_remove_sgat(struct sg_request *srp)
{
	struct sg_scatter_hold *schp = &srp->sgat_h; /* care: remove own data */
	struct sg_fd *sfp = srp->parentfp;
	struct sg_device *sdp;

	sdp = (sfp ? sfp->parentdp : NULL);
	SG_LOG(4, sdp, "%s: num_sgat=%d%s\n", __func__, schp->num_sgat,
	       ((srp->parentfp ? (sfp->rsv_srp == srp) : false) ?
		" [rsv]" : ""));
	if (!test_bit(SG_FRQ_DIO_IN_USE, srp->frq_bm))
		sg_remove_sgat_helper(sdp, schp);

	if (sfp->tot_fd_thresh > 0) {
		/* this is a subtraction, error if it goes negative */
		if (atomic_add_negative(-schp->buflen, &sfp->sum_fd_dlens)) {
			SG_LOG(2, sfp->parentdp,
			       "%s: logic error: this dlen > %s\n",
			       __func__, "sum_fd_dlens");
			atomic_set(&sfp->sum_fd_dlens, 0);
		}
	}
	memset(schp, 0, sizeof(*schp));         /* zeros buflen and dlen */
}

/*
 * For sg v1 and v2 interface: with a command yielding a data-in buffer, after
 * it has arrived in kernel memory, this function copies it to the user space,
 * appended to given struct sg_header object.
 */
static int
sg_rd_append(struct sg_request *srp, void __user *outp, int num_xfer)
{
	int k, num, res;
	struct page *pgp;
	struct sg_scatter_hold *schp = srp->sgatp;

	SG_LOG(4, srp->parentfp->parentdp, "%s: num_xfer=%d\n", __func__,
	       num_xfer);
	if (unlikely(!outp || num_xfer <= 0))
		return (num_xfer == 0 && outp) ? 0 : -EINVAL;

	num = 1 << (PAGE_SHIFT + schp->page_order);
	for (k = 0, res = 0; k < schp->num_sgat; ++k) {
		pgp = schp->pages[k];
		if (unlikely(!pgp)) {
			res = -ENXIO;
			break;
		}
		if (num > num_xfer) {
			if (__copy_to_user(outp, page_address(pgp), num_xfer))
				res = -EFAULT;
			break;
		} else {
			if (__copy_to_user(outp, page_address(pgp), num)) {
				res = -EFAULT;
				break;
			}
			num_xfer -= num;
			if (num_xfer <= 0)
				break;
			outp += num;
		}
	}
	return res;
}

/*
 * If there are multiple requests outstanding, the speed of this function is
 * important. The pack_id is inactive when it is equal to SG_PACK_ID_WILDCARD
 * and this is typically the fast path. Same applies to the tag which is
 * used when use_tag is true. This function is only used in the async cases.
 * Returns pointer to (first) matching sg_request or NULL. If found,
 * sg_request state is moved from SG_RS_AWAIT_RD to SG_RS_BUSY.
 */
static struct sg_request *
sg_get_rq_pack_id(struct sg_fd *sfp, int pack_id, int tag, bool use_tag)
{
	__maybe_unused bool is_bad_st = false;
	__maybe_unused enum sg_rq_state bad_sr_st;
	bool search_for_1;
	enum sg_rq_state sr_st;
	int res;
	unsigned long iflags;
	struct sg_request *srp = NULL;

	search_for_1 = use_tag ? (tag != SG_TAG_WILDCARD) :
				 (pack_id != SG_PACK_ID_WILDCARD);
	read_lock_irqsave(&sfp->rq_list_lock, iflags);
	if (unlikely(search_for_1)) {
		list_for_each_entry(srp, &sfp->rq_list, rq_entry) {
			if (test_bit(SG_FRQ_SYNC_INVOC, srp->frq_bm))
				continue;
			if (use_tag) {
				if (srp->tag != tag)
					continue;
			} else {
				if (srp->pack_id != pack_id)
					continue;
			}
			sr_st = atomic_read(&srp->rq_st);
			switch (sr_st) {
			case SG_RS_AWAIT_RD:
				res = sg_rstate_chg(srp, sr_st, SG_RS_BUSY);
				if (likely(res == 0))
					goto good;
				/* else another caller got it, move on */
				if (IS_ENABLED(CONFIG_SCSI_PROC_FS)) {
					is_bad_st = true;
					bad_sr_st = atomic_read(&srp->rq_st);
				}
				break;
			case SG_RS_SHR_SLAVE:
				goto good;
			case SG_RS_INFLIGHT:
				break;
			default:
				if (IS_ENABLED(CONFIG_SCSI_PROC_FS)) {
					is_bad_st = true;
					bad_sr_st = sr_st;
				}
				break;
			}
			break;	/* problem if same pack_id on > 1 requests */
		}
	} else {        /* search for any request is more likely */
		list_for_each_entry(srp, &sfp->rq_list, rq_entry) {
			if (test_bit(SG_FRQ_SYNC_INVOC, srp->frq_bm))
				continue;
			sr_st = atomic_read(&srp->rq_st);
			if (sr_st == SG_RS_AWAIT_RD) {
				if (likely(sg_rstate_chg(srp, sr_st,
							 SG_RS_BUSY) == 0))
					goto good;
			}
		}
	}
	/* here if one of above loops does not find a match */
	read_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	if (IS_ENABLED(CONFIG_SCSI_PROC_FS)) {
		if (search_for_1) {
			struct sg_device *sdp = sfp->parentdp;
			const char *cptp = use_tag ? "tag=" : "pack_id=";

			if (is_bad_st)
				SG_LOG(1, sdp, "%s: %s%d wrong state: %s\n",
				       __func__, cptp,
				       (use_tag ? tag : pack_id),
				       sg_rq_st_str(bad_sr_st, true));
			else
				SG_LOG(6, sdp, "%s: %s%d not awaiting read\n",
				       __func__, cptp,
				       (use_tag ? tag : pack_id));
		}
	}
	return NULL;
good:
	read_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	if (search_for_1) {
		SG_LOG(6, sfp->parentdp, "%s: %s%d found [srp=0x%p]\n",
		       __func__, (use_tag ? "tag=" : "pack_id="),
		       (use_tag ? tag : pack_id), srp);
	}
	return srp;
}

/*
 * Returns pointer to first non-sync_invoc req waiting to be read. If none
 * found returns NULL unless there were no such reqs on active list, in which
 * case it returns ERR_PTR(-ENODATA).
 */
static struct sg_request *
sg_mrq_get_rq(struct sg_fd *sfp)
{
	bool any_arqs = false;
	unsigned long iflags;
	enum sg_rq_state sr_st;
	struct sg_request *srp;

	read_lock_irqsave(&sfp->rq_list_lock, iflags);
	list_for_each_entry(srp, &sfp->rq_list, rq_entry) {
		if (test_bit(SG_FRQ_SYNC_INVOC, srp->frq_bm))
			continue;
		any_arqs = true;
		sr_st = atomic_read(&srp->rq_st);
		if (sr_st == SG_RS_AWAIT_RD) {
			if (likely(sg_rstate_chg(srp, sr_st,
						 SG_RS_BUSY) == 0)) {
				read_unlock_irqrestore(&sfp->rq_list_lock,
						       iflags);
				return srp;
			}
		}
	}
	read_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	return any_arqs ? NULL : ERR_PTR(-ENODATA);
}

/* Note: *srpp may not be a valid pointer, might be ERR_PTR(-ENODATA) */
static bool
sg_mrq_get_ready_srp(struct sg_fd *sfp, struct sg_request **srpp)
{
	struct sg_request *srp;

	if (unlikely(atomic_read(&sfp->parentdp->detaching))) {
		*srpp = NULL;
		return true;
	}
	srp = sg_mrq_get_rq(sfp);
	*srpp = srp;
	return !!srp;
}

/*
 * Makes a new sg_request object. If 'first' is set then use GFP_KERNEL which
 * may take time but has improved chance of success, otherwise use GFP_ATOMIC.
 * Note that basic initialization is done but srp is not added to either sfp
 * list. On error returns twisted negated errno value (not NULL).
 */
static struct sg_request *
sg_mk_srp(struct sg_fd *sfp, bool first)
{
	struct sg_request *srp;
	int gfp =  __GFP_NOWARN;

	if (first)      /* prepared to wait if none already outstanding */
		srp = kzalloc(sizeof(*srp), gfp | GFP_KERNEL);
	else
		srp = kzalloc(sizeof(*srp), gfp | GFP_ATOMIC);
	if (srp) {
		spin_lock_init(&srp->req_lck);
		atomic_set(&srp->rq_st, SG_RS_INACTIVE);
		srp->sh_var =  SG_SHR_NONE;
		srp->parentfp = sfp;
		srp->tag = SG_TAG_WILDCARD;
		srp->sgatp = &srp->sgat_h; /* only slave share changes sgatp */
		return srp;
	} else
		return ERR_PTR(-ENOMEM);
}

static struct sg_request *
sg_mk_srp_sgat(struct sg_fd *sfp, bool first, int db_len)
{
	int res;
	struct sg_request *n_srp = sg_mk_srp(sfp, first);

	if (IS_ERR(n_srp))
		return n_srp;
	if (db_len > 0) {
		res = sg_mk_sgat(n_srp, sfp, db_len);
		if (res) {
			kfree(n_srp);
			return ERR_PTR(res);
		}
	}
	return n_srp;
}

/*
 * Irrespective of the given reserve request size, the minimum size requested
 * will be PAGE_SIZE (often 4096 bytes). Returns a pointer to reserve object or
 * a negated errno value twisted by ERR_PTR() macro. The actual number of bytes
 * allocated (maybe less than buflen) is in srp->sgatp->buflen . Note that this
 * function is only called in contexts where locking is not required.
 */
static struct sg_request *
sg_build_reserve(struct sg_fd *sfp, int buflen)
{
	bool go_out = false;
	int res;
	struct sg_request *srp;

	SG_LOG(3, sfp ? sfp->parentdp : NULL, "%s: buflen=%d\n", __func__,
	       buflen);
	srp = sg_mk_srp(sfp, list_empty(&sfp->rq_fl));
	if (IS_ERR(srp))
		return srp;
	sfp->rsv_srp = srp;
	do {
		if (buflen < (int)PAGE_SIZE) {
			buflen = PAGE_SIZE;
			go_out = true;
		}
		res = sg_mk_sgat(srp, sfp, buflen);
		if (res == 0) {
			SG_LOG(4, sfp ? sfp->parentdp : NULL,
			       "%s: final buflen=%d, srp=0x%p ++\n", __func__,
			       buflen, srp);
			return srp;
		}
		if (go_out)
			return ERR_PTR(res);
		/* failed so remove, halve buflen, try again */
		sg_remove_sgat(srp);
		buflen >>= 1;   /* divide by 2 */
	} while (true);
}


/*
 * Adds an active request (soon to carry a SCSI command) to the current file
 * descriptor by creating a new one or re-using a request from the free
 * list (fl). If successful returns a valid pointer in SG_RS_BUSY state. On
 * failure returns a negated errno value twisted by ERR_PTR() macro.
 */
static struct sg_request *
sg_add_request(struct sg_fd *sfp, int dxfr_len, bool sync,
	       enum sg_shr_var sh_var)
{
	bool act_empty = false;
	bool allow_rsv = true;
	bool mk_new_srp = true;
	bool sl_req = false;
	bool sl_swait = false;
	u32 sum_dlen;
	unsigned long iflags;
	enum sg_rq_state sr_st;
	enum sg_rq_state ma_sr_st;
	struct sg_request *r_srp = NULL;	/* request to return */
	struct sg_request *rsv_srp;	/* current fd's reserve request */
	struct sg_request *ma_rsv_srp = NULL;	/* master's reserve req */
	__maybe_unused struct sg_device *sdp;
	struct sg_fd *ma_sfp = NULL;
	__maybe_unused const char *cp = "fail";

	write_lock_irqsave(&sfp->rq_list_lock, iflags);
	sdp = sfp->parentdp;
	rsv_srp = sfp->rsv_srp;
	switch (sh_var) {
	case SG_SHR_NONE:
	case SG_SHR_SL_FD_NOT_RQ:
		break;
	case SG_SHR_MA_RQ:
		sr_st = atomic_read(&rsv_srp->rq_st);
		if (sr_st == SG_RS_INACTIVE) {
			if (likely(sg_rstate_chg(rsv_srp, sr_st,
						 SG_RS_BUSY) == 0)) {
				r_srp = rsv_srp;
				mk_new_srp = false;
				cp = "ma_rq";
				goto good_fini;
			}
		}
		r_srp = ERR_PTR(-EBUSY);
		break;
	case SG_SHR_MA_FD_NOT_RQ:
		allow_rsv = false;
		break;
	case SG_SHR_SL_RQ:
		ma_sfp = sfp->share_sfp;
		if (!ma_sfp) {
			r_srp = ERR_PTR(-EPROTO);
			break;
		}
		/*
		 * Contention here may be with another potential slave trying
		 * to pair with this master. The loser will receive an
		 * EADDRINUSE errno. The winner advances master's rq_state:
		 *     SG_RS_SHR_SWAP --> SG_RS_SHR_SLAVE
		 */
		ma_rsv_srp = ma_sfp->rsv_srp;
		ma_sr_st = atomic_read(&ma_rsv_srp->rq_st);
		switch (ma_sr_st) {
		case SG_RS_AWAIT_RD:
		case SG_RS_DONE_RD:
			if (ma_rsv_srp->rq_result & SG_ML_RESULT_MSK) {
				r_srp = ERR_PTR(-ENOSTR);
				break;
			}
			/* fallthrough */
		case SG_RS_SHR_SWAP:
			sl_req = true;
			if (ma_sr_st == SG_RS_AWAIT_RD)
				break;
			if (unlikely(sg_rstate_chg(ma_rsv_srp, ma_sr_st,
						   SG_RS_SHR_SLAVE) != 0))
				r_srp = ERR_PTR(-EADDRINUSE);
			break;
		case  SG_RS_INFLIGHT:
			sl_req = true;
			sl_swait = true;
			break;
		case SG_RS_INACTIVE:
			r_srp = ERR_PTR(-EADDRNOTAVAIL);
			break;
		case SG_RS_BUSY:
			r_srp = ERR_PTR(-EBUSY);
			break;
		case SG_RS_SHR_SLAVE:
		default:
			r_srp = ERR_PTR(-EADDRINUSE);
			break;
		}
		break;
	}
	if (IS_ERR(r_srp))
		goto err_out;
	cp = "";
	if (sl_req) {	/* slave dlen may be smaller than master's dlen */
		if (dxfr_len > ma_rsv_srp->sgatp->dlen) {
			SG_LOG(4, sdp, "%s: slave dlen [%d] > master dlen\n",
			       __func__, dxfr_len);
			r_srp = ERR_PTR(-E2BIG);
			goto err_out;
		}
		dxfr_len = 0;	/* any srp for slave will do, pick smallest */
	}

	/*
	 * Check the free list (fl) for candidates. If SG_SHR_MA_FD_NOT_RQ
	 * then do not re-use the reserve request. Pick zero data length
	 * requests from the back of the fl, the rest from the front.
	 */
	if (list_empty(&sfp->rq_fl))
		act_empty = true;
	else if (dxfr_len < 1) {    /* 0 data length requests at back of fl */
		list_for_each_entry_reverse(r_srp, &sfp->rq_fl, fl_entry) {
			if (!allow_rsv && rsv_srp == r_srp)
				continue;
			sr_st = atomic_read(&r_srp->rq_st);
			if (sr_st == SG_RS_INACTIVE) {
				if (likely(sg_rstate_chg(r_srp, sr_st,
							 SG_RS_BUSY) == 0)) {
					cp = "from back of fl";
					mk_new_srp = false;
					break;
				}
			}
		}
	} else { /*     find request with large enough dlen */
		list_for_each_entry(r_srp, &sfp->rq_fl, fl_entry) {
			if (!allow_rsv && rsv_srp == r_srp)
				continue;
			sr_st = atomic_read(&r_srp->rq_st);
			if (sr_st == SG_RS_INACTIVE &&
			    r_srp->sgatp->buflen >= dxfr_len) {
				if (likely(sg_rstate_chg(r_srp, sr_st,
							 SG_RS_BUSY) == 0)) {
					cp = "from front of fl";
					mk_new_srp = false;
					break;
				} /* else other got it, move on */
			}
		}
	}

good_fini:
	if (mk_new_srp) {	/* Need new sg_request object */
		bool allow_cmd_q = test_bit(SG_FFD_CMD_Q, sfp->ffd_bm);

		r_srp = NULL;
		if (!allow_cmd_q && !list_empty(&sfp->rq_list)) {
			r_srp = ERR_PTR(-EDOM);
			SG_LOG(6, sdp, "%s: trying 2nd req but cmd_q=false\n",
			       __func__);
		} else if (sfp->tot_fd_thresh > 0) {
			sum_dlen = atomic_read(&sfp->sum_fd_dlens) + dxfr_len;
			if (sum_dlen > (u32)sfp->tot_fd_thresh) {
				r_srp = ERR_PTR(-E2BIG);
				SG_LOG(2, sdp, "%s: sum_of_dlen(%u) > %s\n",
				       __func__, sum_dlen, "tot_fd_thresh");
			}
		}
		write_unlock_irqrestore(&sfp->rq_list_lock, iflags);
		if (IS_ERR(r_srp))        /* NULL is not an ERR here */
			goto err_no_lock;
		/* releasing rq_list_lock because next line could take time */
		r_srp = sg_mk_srp_sgat(sfp, act_empty, dxfr_len);
		if (IS_ERR(r_srp))
			goto err_no_lock;
		cp = "new";
		SG_LOG(4, sdp, "%s: mk_new_srp=0x%p ++\n", __func__, r_srp);
		atomic_set(&r_srp->rq_st, SG_RS_BUSY);
	} else	/* otherwise found srp is on fl, remove from fl */
		list_del(&r_srp->fl_entry);
	if (!mk_new_srp)
		spin_lock(&r_srp->req_lck);
	/* clear all bits in sg_request::frq_bm */
	memset(r_srp->frq_bm, 0, sizeof(r_srp->frq_bm));
	r_srp->sgatp->dlen = dxfr_len;/* must be <= r_srp->sgatp->buflen */
	assign_bit(SG_FRQ_SYNC_INVOC, r_srp->frq_bm, sync);
	r_srp->sh_var = sh_var;
	r_srp->cmd_opcode = 0xff;  /* set invalid opcode (VS), 0x0 is TUR */
	/* If setup stalls (e.g. blk_get_request()) debug shows 'elap=1 ns' */
	if (test_bit(SG_FFD_TIME_IN_NS, sfp->ffd_bm))
		r_srp->start_kt = KTIME_MAX;
	if (sl_req && ma_rsv_srp) {
		ma_sfp->slave_srp = r_srp;
		/* slave "shares" the master reserve request's data buffer */
		r_srp->sgatp = &ma_rsv_srp->sgat_h;
		assign_bit(SG_FRQ_DEFER_XFR, r_srp->frq_bm, sl_swait);
	}
	if (mk_new_srp)
		write_lock_irqsave(&sfp->rq_list_lock, iflags);
	list_add_tail(&r_srp->rq_entry, &sfp->rq_list);
	if (!mk_new_srp)
		spin_unlock(&r_srp->req_lck);
err_out:
	write_unlock_irqrestore(&sfp->rq_list_lock, iflags);
err_no_lock:
	SG_LOG(6, sdp, "%s: %s %sr_srp=0x%p\n", __func__, cp,
	       ((r_srp == rsv_srp) ? "[rsv] " : ""), r_srp);
	cp = cp;	/* suppress warning */
	return r_srp;
}

/*
 * Moves a completed sg_request object to the free list and sets it to
 * SG_RS_INACTIVE which makes it available for re-use. Requests with no data
 * associated are appended to the tail of the free list while other requests
 * are prepended to the head of the free list.
 */
static void
sg_deact_request(struct sg_fd *sfp, struct sg_request *srp)
{
	bool on_fl = false;
	int dlen, buflen;
	unsigned long iflags;
	enum sg_rq_state sr_st;
	struct sg_request *t_srp;
	struct sg_scatter_hold *schp;
	const char *cp = "head";
	char b[80];

	if (WARN_ON(!sfp || !srp))
		return;
	schp = &srp->sgat_h;	/* make sure it is own data buffer */
	b[0] = '\0';
	write_lock_irqsave(&sfp->rq_list_lock, iflags);
	sr_st = atomic_read(&srp->rq_st);
	if (sr_st != SG_RS_SHR_SWAP) /* mark _BUSY then _INACTIVE at end */
		atomic_set(&srp->rq_st, SG_RS_BUSY);
	list_del(&srp->rq_entry);
	/*
	 * N.B. sg_request object is not de-allocated (freed). The contents
	 * of the rq_list and rq_fl lists are de-allocated (freed) when
	 * the owning file descriptor is closed. The free list acts as a LIFO
	 * for same size (buflen) sg_request objects. This can improve the
	 * chance of a cache hit when the sg_request object is re-used.
	 */
	dlen = schp->dlen;
	buflen = schp->buflen;
	/* want ascending free list by dlen, but dlen==0 at end */
	if (dlen > 0) {
		list_for_each_entry(t_srp, &sfp->rq_fl, fl_entry) {
			if (buflen <= t_srp->sgat_h.buflen ||
			    t_srp->sgat_h.buflen == 0) {
				/* add srp _before_ t_srp on fl */
				list_add_tail(&srp->fl_entry,
					      &t_srp->fl_entry);
				on_fl = true;
				cp = "ascending pos";
				break;
			}
		}
	}
	if (!on_fl) { /* either empty list, dlen=0 or buflen highest */
		list_add_tail(&srp->fl_entry, &sfp->rq_fl);
		cp = "tail";
	}
	snprintf(b, sizeof(b), "%ssrp=0x%p move to fl %s",
		 ((sfp->rsv_srp == srp) ? "rsv " : ""), srp, cp);
	write_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	if (sr_st != SG_RS_SHR_SWAP) {
		atomic_set(&srp->rq_st, SG_RS_INACTIVE);
		srp->tag = SG_TAG_WILDCARD;
	}
	SG_LOG(5, sfp->parentdp, "%s: %s\n", __func__, b);
}

/* Returns pointer to sg_fd object or negated errno twisted by ERR_PTR */
static struct sg_fd *
sg_add_sfp(struct sg_device *sdp)
{
	bool reduced = false;
	int rbuf_len;
	unsigned long iflags;
	long err;
	struct sg_fd *sfp;
	struct sg_request *srp;

	sfp = kzalloc(sizeof(*sfp), GFP_ATOMIC | __GFP_NOWARN);
	if (!sfp) {
		SG_LOG(1, sdp, "%s: sfp allocation failed\n", __func__);
		return ERR_PTR(-ENOMEM);
	}
	init_waitqueue_head(&sfp->read_wait);
	rwlock_init(&sfp->rq_list_lock);
	INIT_LIST_HEAD(&sfp->rq_list);
	INIT_LIST_HEAD(&sfp->rq_fl);
	kref_init(&sfp->f_ref);
	mutex_init(&sfp->f_mutex);
	sfp->timeout = SG_DEFAULT_TIMEOUT;
	sfp->timeout_user = SG_DEFAULT_TIMEOUT_USER;
	/* other bits in sfp->ffd_bm[1] cleared by kzalloc() above */
	assign_bit(SG_FFD_FORCE_PACKID, sfp->ffd_bm, SG_DEF_FORCE_PACK_ID);
	assign_bit(SG_FFD_CMD_Q, sfp->ffd_bm, SG_DEF_COMMAND_Q);
	assign_bit(SG_FFD_KEEP_ORPHAN, sfp->ffd_bm, SG_DEF_KEEP_ORPHAN);
	assign_bit(SG_FFD_TIME_IN_NS, sfp->ffd_bm, SG_DEF_TIME_UNIT);
	assign_bit(SG_FFD_Q_AT_TAIL, sfp->ffd_bm, SG_DEFAULT_Q_AT);
	sfp->tot_fd_thresh = SG_TOT_FD_THRESHOLD;
	atomic_set(&sfp->sum_fd_dlens, 0);
	/*
	 * SG_SCATTER_SZ initializes scatter_elem_sz but different value may
	 * be given as driver/module parameter (e.g. 'scatter_elem_sz=8192').
	 * Any user provided number will be changed to be PAGE_SIZE as a
	 * minimum, otherwise it will be rounded down (if required) to a
	 * power of 2. So it will always be a power of 2.
	 */
	sfp->sgat_elem_sz = scatter_elem_sz;
	sfp->parentdp = sdp;
	if (atomic_read(&sdp->detaching)) {
		kfree(sfp);
		SG_LOG(1, sdp, "%s: detaching\n", __func__);
		return ERR_PTR(-ENODEV);
	}
	sg_unshare_fds(sfp, NULL);
	if (unlikely(sg_big_buff != def_reserved_size))
		sg_big_buff = def_reserved_size;

	rbuf_len = min_t(int, sg_big_buff, sdp->max_sgat_sz);
	if (rbuf_len > 0) {
		srp = sg_build_reserve(sfp, rbuf_len);
		if (IS_ERR(srp)) {
			kfree(sfp);
			err = PTR_ERR(srp);
			SG_LOG(1, sdp, "%s: build reserve err=%ld\n", __func__,
			       -err);
			return ERR_PTR(err);
		}
		if (srp->sgatp->buflen < rbuf_len) {
			reduced = true;
			SG_LOG(2, sdp,
			       "%s: reserve reduced from %d to buflen=%d\n",
			       __func__, rbuf_len, srp->sgatp->buflen);
		}
		/* will be first element so head or tail doesn't matter */
		list_add_tail(&srp->fl_entry, &sfp->rq_fl);
	}
	if (!reduced) {
		SG_LOG(4, sdp, "%s: built reserve buflen=%d\n", __func__,
		       rbuf_len);
	}
	write_lock_irqsave(&sdp->sfd_llock, iflags);
	list_add_tail(&sfp->sfd_entry, &sdp->sfds);
	kref_get(&sdp->d_ref);
	__module_get(THIS_MODULE);
	write_unlock_irqrestore(&sdp->sfd_llock, iflags);
	SG_LOG(3, sdp, "%s: success, sfp=0x%p ++\n", __func__, sfp);
	return sfp;
}

/*
 * A successful call to sg_release() will result, at some later time, to this
 * function being invoked. All requests associated with this file descriptor
 * should be completed or cancelled when this function is called (due to
 * sfp->f_ref). Also the file descriptor itself has not been accessible since
 * it was list_del()-ed by the preceding sg_remove_sfp() call. So no locking
 * is required. sdp should never be NULL but to make debugging more robust,
 * this function will not blow up in that case.
 */
static void
sg_remove_sfp_usercontext(struct work_struct *work)
{
	struct sg_fd *sfp = container_of(work, struct sg_fd, ew_fd.work);
	struct sg_device *sdp;
	struct sg_request *srp;
	__maybe_unused const char *cp = " srp=0x";

	if (IS_ERR_OR_NULL(sfp)) {
		pr_warn("sg: %s: sfp is NULL or error\n", __func__);
		return;
	}
	sdp = sfp->parentdp;

	/* Cleanup any responses which were never read(). */
	while (!list_empty(&sfp->rq_list)) {
		srp = list_last_entry(&sfp->rq_list, struct sg_request,
				      rq_entry);
		if (IS_ERR_OR_NULL(srp)) {
			pr_warn("sg: %s: NULL ptr on rq_list\n", __func__);
			continue;
		}
		sg_finish_scsi_blk_rq(srp);
		list_del(&srp->rq_entry);
		if (srp->sgatp->buflen > 0)
			sg_remove_sgat(srp);
		SG_LOG(6, sdp, "%s:%s%p --\n", __func__, cp, srp);
		kfree(srp);
	}
	while (!list_empty(&sfp->rq_fl)) {
		srp = list_last_entry(&sfp->rq_fl, struct sg_request,
				      fl_entry);
		if (IS_ERR_OR_NULL(srp)) {
			pr_warn("sg: %s: NULL ptr on rq_fl\n", __func__);
			continue;
		}
		list_del(&srp->fl_entry);
		if (srp->sgatp->buflen > 0)
			sg_remove_sgat(srp);
		SG_LOG(6, sdp, "%s: fl%s%p --\n", __func__, cp, srp);
		kfree(srp);
	}
	SG_LOG(3, sdp, "%s: kfree: sfp=0x%p --\n", __func__, sfp);
	kfree(sfp);

	if (sdp) {
		scsi_device_put(sdp->device);
		kref_put(&sdp->d_ref, sg_device_destroy);
	}
	module_put(THIS_MODULE);
}

static void
sg_remove_sfp(struct kref *kref)
{
	unsigned long iflags;
	struct sg_fd *sfp = container_of(kref, struct sg_fd, f_ref);
	struct sg_device *sdp = sfp->parentdp;
	struct sg_fd *o_sfp =  (sfp->shr_fd != SG_SHARE_FD_UNUSED) ?
			       sfp->share_sfp : NULL;

	if (o_sfp && !test_bit(SG_FFD_RELEASE, o_sfp->ffd_bm)) {
		mutex_lock(&o_sfp->f_mutex);
		sg_remove_sfp_share(sfp);
		mutex_unlock(&o_sfp->f_mutex);
	}
	write_lock_irqsave(&sdp->sfd_llock, iflags);
	list_del(&sfp->sfd_entry);
	write_unlock_irqrestore(&sdp->sfd_llock, iflags);

	INIT_WORK(&sfp->ew_fd.work, sg_remove_sfp_usercontext);
	schedule_work(&sfp->ew_fd.work);
}

static struct sg_device *
sg_lookup_dev(int dev)
	__must_hold(&sg_index_lock)
{
	return idr_find(&sg_index_idr, dev);
}

/*
 * Returns valid pointer to a sg_device object on success or a negated
 * errno value on failure. Does not return NULL.
 */
static struct sg_device *
sg_get_dev(int min_dev)
{
	struct sg_device *sdp;
	unsigned long iflags;

	read_lock_irqsave(&sg_index_lock, iflags);
	sdp = sg_lookup_dev(min_dev);
	if (!sdp)
		sdp = ERR_PTR(-ENXIO);
	else if (atomic_read(&sdp->detaching)) {
		/* If sdp->detaching, then the refcount may already be 0, in
		 * which case it would be a bug to do kref_get().
		 */
		sdp = ERR_PTR(-ENODEV);
	} else
		kref_get(&sdp->d_ref);
	read_unlock_irqrestore(&sg_index_lock, iflags);
	return sdp;
}

#if IS_ENABLED(CONFIG_SCSI_PROC_FS)
static const char *
sg_rq_st_str(enum sg_rq_state rq_st, bool long_str)
{
	switch (rq_st) {	/* request state */
	case SG_RS_INACTIVE:
		return long_str ? "inactive" :  "ina";
	case SG_RS_INFLIGHT:
		return long_str ? "inflight" : "act";
	case SG_RS_AWAIT_RD:
		return long_str ? "await_read" : "rcv";
	case SG_RS_DONE_RD:
		return long_str ? "done_read" : "fin";
	case SG_RS_BUSY:
		return long_str ? "busy" : "bsy";
	case SG_RS_SHR_SWAP:	/* only an active master has this */
		return long_str ? "share swap" : "s_sw";
	case SG_RS_SHR_SLAVE:	/* only an active master has this */
		return long_str ? "share slave active" : "sl_a";
	case SG_RS_SHR_SWAIT:	/* only an active slave has this */
		return long_str ? "share slave wait" : "sl_w";
	default:
		return long_str ? "unknown" : "unk";
	}
}
#else
static const char *
sg_rq_st_str(enum sg_rq_state rq_st, bool long_str)
{
	return "";
}
#endif

#if IS_ENABLED(CONFIG_SCSI_PROC_FS)
static const char *
sg_shr_str(enum sg_shr_var sh_var, bool long_str)
{
	switch (sh_var) {	/* share variety of request */
	case SG_SHR_NONE:
		return long_str ? "none" :  "-";
	case SG_SHR_MA_RQ:
		return long_str ? "master request" :  "m_rq";
	case SG_SHR_MA_FD_NOT_RQ:
		return long_str ? "master fd, not request" :  "m_nr";
	case SG_SHR_SL_RQ:
		return long_str ? "slave request" :  "s_rq";
	case SG_SHR_SL_FD_NOT_RQ:
		return long_str ? "slave fd, not request" :  "s_nr";
	default:
		return long_str ? "unknown" : "unk";
	}
}
#else
static const char *
sg_shr_str(enum sg_shr_var sh_var, bool long_str)
{
	return "";
}
#endif

#if IS_ENABLED(CONFIG_SCSI_PROC_FS)	/* long, almost to end of file */
static int sg_proc_seq_show_int(struct seq_file *s, void *v);

static int sg_proc_single_open_adio(struct inode *inode, struct file *filp);
static ssize_t sg_proc_write_adio(struct file *filp, const char __user *buffer,
				  size_t count, loff_t *off);
static const struct file_operations adio_fops = {
	.owner = THIS_MODULE,
	.open = sg_proc_single_open_adio,
	.read = seq_read,
	.llseek = seq_lseek,
	.write = sg_proc_write_adio,
	.release = single_release,
};

static int sg_proc_single_open_dressz(struct inode *inode, struct file *filp);
static ssize_t sg_proc_write_dressz(struct file *filp,
		const char __user *buffer, size_t count, loff_t *off);
static const struct file_operations dressz_fops = {
	.owner = THIS_MODULE,
	.open = sg_proc_single_open_dressz,
	.read = seq_read,
	.llseek = seq_lseek,
	.write = sg_proc_write_dressz,
	.release = single_release,
};

static int sg_proc_single_open_trc_ctl(struct inode *inode, struct file *filp);
static ssize_t sg_proc_read_trc_ctl(struct file *filp, char __user *buffer,
				    size_t count, loff_t *off);
static ssize_t sg_proc_write_trc_ctl(struct file *filp,
		const char __user *buffer, size_t count, loff_t *off);
static const struct file_operations trc_ctl_fops = {
	.owner = THIS_MODULE,
	.open = sg_proc_single_open_trc_ctl,
	.read = sg_proc_read_trc_ctl,
	.llseek = seq_lseek,
	.write = sg_proc_write_trc_ctl,
	.release = single_release,
};

static int sg_proc_seq_show_version(struct seq_file *s, void *v);
static int sg_proc_seq_show_devhdr(struct seq_file *s, void *v);
static int sg_proc_seq_show_dev(struct seq_file *s, void *v);
static int sg_proc_seq_show_trace_o(struct seq_file *s, void *v);

static void * dev_seq_start(struct seq_file *s, loff_t *pos);
static void * dev_seq_next(struct seq_file *s, void *v, loff_t *pos);
static void dev_seq_stop(struct seq_file *s, void *v);
static const struct seq_operations dev_seq_ops = {
	.start = dev_seq_start,
	.next  = dev_seq_next,
	.stop  = dev_seq_stop,
	.show  = sg_proc_seq_show_dev,
};

static int sg_proc_seq_show_devstrs(struct seq_file *s, void *v);
static const struct seq_operations devstrs_seq_ops = {
	.start = dev_seq_start,
	.next  = dev_seq_next,
	.stop  = dev_seq_stop,
	.show  = sg_proc_seq_show_devstrs,
};

static int sg_proc_seq_show_full_debug(struct seq_file *s, void *v);
static const struct seq_operations debug_seq_ops = {
	.start = dev_seq_start,
	.next  = dev_seq_next,
	.stop  = dev_seq_stop,
	.show  = sg_proc_seq_show_full_debug,
};

static int sg_proc_seq_show_red_debug(struct seq_file *s, void *v);
static const struct seq_operations red_debug_seq_ops = {
	.start = dev_seq_start,
	.next  = dev_seq_next,
	.stop  = dev_seq_stop,
	.show  = sg_proc_seq_show_red_debug,
};

static int
sg_proc_init(void)
{
	struct proc_dir_entry *p;

	p = proc_mkdir("scsi/sg", NULL);
	if (!p)
		return 1;

	proc_create("allow_dio", 0644, p, &adio_fops);
	proc_create_seq("debug", 0444, p, &debug_seq_ops);
	proc_create("def_reserved_size", 0644, p, &dressz_fops);
	proc_create_single("device_hdr", 0444, p, sg_proc_seq_show_devhdr);
	proc_create_seq("devices", 0444, p, &dev_seq_ops);
	proc_create_seq("device_strs", 0444, p, &devstrs_seq_ops);
	proc_create_seq("red_debug", 0444, p, &red_debug_seq_ops);
	proc_create("trc_ctl", 0644, p, &trc_ctl_fops);
	proc_create_single("trace_out", 0444, p, sg_proc_seq_show_trace_o);
	proc_create_single("version", 0444, p, sg_proc_seq_show_version);
	return 0;
}

static int
sg_last_dev(void)
{
	int k = -1;
	unsigned long iflags;

	read_lock_irqsave(&sg_index_lock, iflags);
	idr_for_each(&sg_index_idr, sg_idr_max_id, &k);
	read_unlock_irqrestore(&sg_index_lock, iflags);
	return k + 1;           /* origin 1 */
}

static int
sg_proc_seq_show_int(struct seq_file *s, void *v)
{
	seq_printf(s, "%d\n", *((int *)s->private));
	return 0;
}

static int
sg_proc_single_open_adio(struct inode *inode, struct file *filp)
{
	return single_open(filp, sg_proc_seq_show_int, &sg_allow_dio);
}

static ssize_t
sg_proc_write_adio(struct file *filp, const char __user *buffer,
		   size_t count, loff_t *off)
{
	int err;
	unsigned long num;

	if (!capable(CAP_SYS_ADMIN) || !capable(CAP_SYS_RAWIO))
		return -EACCES;
	err = kstrtoul_from_user(buffer, count, 0, &num);
	if (err)
		return err;
	sg_allow_dio = num ? 1 : 0;
	return count;
}

static int
sg_proc_single_open_dressz(struct inode *inode, struct file *filp)
{
	return single_open(filp, sg_proc_seq_show_int, &sg_big_buff);
}

static ssize_t
sg_proc_write_dressz(struct file *filp, const char __user *buffer,
		     size_t count, loff_t *off)
{
	int err;
	unsigned long k = ULONG_MAX;

	if (!capable(CAP_SYS_ADMIN) || !capable(CAP_SYS_RAWIO))
		return -EACCES;

	err = kstrtoul_from_user(buffer, count, 0, &k);
	if (err)
		return err;
	if (k <= 1048576) {     /* limit "big buff" to 1 MB */
		sg_big_buff = k;
		return count;
	}
	return -EDOM;
}

static int
sg_proc_seq_show_version(struct seq_file *s, void *v)
{
	seq_printf(s, "%d\t%s [%s]\n", sg_version_num, SG_VERSION_STR,
		   sg_version_date);
	return 0;
}

static int
sg_proc_single_open_trc_ctl(struct inode *inode, struct file *filp)
{
	int n = sg_trc_ctl_level > 0 ? -2 : -3;

	if (IS_ENABLED(SG_DEBUG)) {
		if (sg_trc_cb_valid) {
			struct circ_buf *cbp = &sg_trc_cb;

			mutex_lock(&sg_trc_cb_lck);
			n = CIRC_CNT(cbp->head, cbp->tail, sg_trc_cb_sz);
			mutex_unlock(&sg_trc_cb_lck);
		} else
			n = -1;
	}
	return single_open(filp, sg_proc_seq_show_int, &n);
}

#if IS_ENABLED(SG_DEBUG)
static bool
sg_trc_buf_alloc(int sz_exp)
{
	int new_sz = 1 << sz_exp;
	int gfp_flags = __GFP_NOWARN | GFP_KERNEL;

	sg_trc_cb_valid = false;
	if (!sg_trc_wp) {
		sg_trc_wp = kzalloc(SG_PROC_DEBUG_SZ, gfp_flags);
		if (!sg_trc_wp)
			return false;
	}
	sg_trc_cb.buf = kzalloc(new_sz, gfp_flags);
	if (sg_trc_cb.buf) {
		sg_trc_cb.head = 0;
		sg_trc_cb.tail = 0;
		sg_trc_cb_sz = new_sz;
		sg_trc_cb_valid = true;
		SCSI_LOG_TIMEOUT(6, pr_info("%s: cb=0x%p ++, wp=0x%p ++\n",
					    __func__, sg_trc_cb.buf,
					    sg_trc_wp));
		return true;
	}
	sg_trc_cb_sz = 0;
	return false;
}
#else
static bool
sg_trc_buf_alloc(int sz_exp)
{
	return true;
}
#endif

#if IS_ENABLED(SG_DEBUG)
static void
sg_trc_buf_remove(void)
{
	kfree(sg_trc_cb.buf);
	SCSI_LOG_TIMEOUT(6, pr_info("%s: sg_trc_cb %p --, wp %p --\n",
				    __func__, sg_trc_cb.buf, sg_trc_wp));
	sg_trc_cb.buf = NULL;
	sg_trc_cb.head = 0;
	sg_trc_cb.tail = 0;
	sg_trc_ctl_level = 0;
	kfree(sg_trc_wp);
	sg_trc_wp = NULL;
}
#else
static void
sg_trc_buf_remove(void)
{
}
#endif

/* when n < 8 sets sg_trc_ctl_level, if n >= 8 it's 2^n of circ buff sz */
static ssize_t
sg_proc_write_trc_ctl(struct file *filp, const char __user *buffer,
		      size_t count, loff_t *off)
{
	bool ok = true;
	int err;
	unsigned long n;

	if (!capable(CAP_SYS_ADMIN) || !capable(CAP_SYS_RAWIO))
		return -EACCES;
	err = kstrtoul_from_user(buffer, count, 0, &n);
	if (err)
		return err;
	if (n < 2 || (n > ilog2(SG_PROC_DEBUG_SZ) && n < 23)) {
		if (IS_ENABLED(SG_DEBUG)) {
			mutex_lock(&sg_trc_cb_lck);
			if (n < 8) {
				if (!!n != !!sg_trc_ctl_level) {
					if (n == 0)
						sg_trc_buf_remove();
					else {
						ok = sg_trc_buf_alloc(
							sg_trc_ctl_exp);
						sg_trc_ctl_level = ok ? n : 0;
					}
				}
			} else {
				sg_trc_buf_remove();
				ok = sg_trc_buf_alloc(n);
				sg_trc_ctl_level = ok ? n : 0;
				if (ok)
					sg_trc_ctl_exp = n;
			}
			mutex_unlock(&sg_trc_cb_lck);
		} else {
			if (n < 8)
				sg_trc_ctl_level = n;
			else {
				sg_trc_ctl_exp = n;
				sg_trc_ctl_level = n;
			}
		}

	} else {
		pr_info("%s: want 0 (off), 1 (on) or 14..22 which are 2^y\n",
			__func__);
		return -EDOM;
	}
	return ok ? (int)count : -EINVAL;
}

static ssize_t
sg_proc_read_trc_ctl(struct file *filp, char __user *buffer, size_t count,
		     loff_t *off)
{
	size_t len;
	char b[32];

	if (off && *off != 0)
		return 0;	/* EOF */
	snprintf(b, sizeof(b), "%d\n", sg_trc_ctl_level);
	len = strlen(b);
	if (len > count)
		return -EDOM;
	if (copy_to_user(buffer, b, len))
		return -EFAULT;
	if (off)
		*off += len;
	return len;
}

/* Circular buffer state unchanged if seq_write() overflows, get re-called */
static int
sg_proc_seq_show_trace_o(struct seq_file *s, void *v)
{
#if IS_ENABLED(SG_DEBUG)
	if (sg_trc_cb_valid && sg_trc_ctl_level > 0) {
		int j, m;
		struct circ_buf *cbp = &sg_trc_cb;

		if (!mutex_trylock(&sg_trc_cb_lck)) {
			seq_puts(s, "    Busy, please try again\n");
			return 0;
		}
		j = CIRC_CNT(cbp->head, cbp->tail, sg_trc_cb_sz);
		m = CIRC_CNT_TO_END(cbp->head, cbp->tail, sg_trc_cb_sz);
		if (j < 1)
			;
		else if (j > m) {
			seq_write(s, cbp->buf + cbp->tail, m);
			if (seq_has_overflowed(s)) {
				mutex_unlock(&sg_trc_cb_lck);
				return 0;
			}
			seq_write(s, cbp->buf + 0, j - m);
			if (seq_has_overflowed(s)) {
				mutex_unlock(&sg_trc_cb_lck);
				return 0;
			}
			cbp->tail = j - m;
		} else {
			seq_write(s, cbp->buf + cbp->tail, j);
			if (seq_has_overflowed(s)) {
				mutex_unlock(&sg_trc_cb_lck);
				return 0;
			}
			cbp->tail += j;
		}
		mutex_unlock(&sg_trc_cb_lck);
		seq_printf(s, ">>> End of %d byte trace buffer, clear\n", j);
	} else if (sg_trc_ctl_level < 1)
		seq_puts(s, "tracing is off; 'echo 1 > trc_ctl' to start\n");
	else
		seq_puts(s, "SG_DEBUG enabled, sg_trc_cb_valid is false\n");
#endif
	return 0;
}

static int
sg_proc_seq_show_devhdr(struct seq_file *s, void *v)
{
	seq_puts(s, "host\tchan\tid\tlun\ttype\topens\tqdepth\tbusy\tonline\n");
	return 0;
}

struct sg_proc_deviter {
	loff_t  index;
	size_t  max;
	int	fd_index;
};

static void *
dev_seq_start(struct seq_file *s, loff_t *pos)
{
	struct sg_proc_deviter *it = kzalloc(sizeof(*it), GFP_KERNEL);

	s->private = it;
	if (! it)
		return NULL;

	it->index = *pos;
	it->max = sg_last_dev();
	if (it->index >= (int)it->max)
		return NULL;
	return it;
}

static void *
dev_seq_next(struct seq_file *s, void *v, loff_t *pos)
{
	struct sg_proc_deviter *it = s->private;

	*pos = ++it->index;
	return (it->index < (int)it->max) ? it : NULL;
}

static void
dev_seq_stop(struct seq_file *s, void *v)
{
	kfree(s->private);
}

static int
sg_proc_seq_show_dev(struct seq_file *s, void *v)
{
	struct sg_proc_deviter *it = (struct sg_proc_deviter *)v;
	struct sg_device *sdp;
	struct scsi_device *scsidp;
	unsigned long iflags;

	read_lock_irqsave(&sg_index_lock, iflags);
	sdp = it ? sg_lookup_dev(it->index) : NULL;
	if (!sdp || !sdp->device || atomic_read(&sdp->detaching))
		seq_puts(s, "-1\t-1\t-1\t-1\t-1\t-1\t-1\t-1\t-1\n");
	else {
		scsidp = sdp->device;
		seq_printf(s, "%d\t%d\t%d\t%llu\t%d\t%d\t%d\t%d\t%d\n",
			      scsidp->host->host_no, scsidp->channel,
			      scsidp->id, scsidp->lun, (int)scsidp->type,
			      1,
			      (int)scsidp->queue_depth,
			      (int)atomic_read(&scsidp->device_busy),
			      (int)scsi_device_online(scsidp));
	}
	read_unlock_irqrestore(&sg_index_lock, iflags);
	return 0;
}

static int
sg_proc_seq_show_devstrs(struct seq_file *s, void *v)
{
	struct sg_proc_deviter *it = (struct sg_proc_deviter *)v;
	struct sg_device *sdp;
	struct scsi_device *scsidp;
	unsigned long iflags;

	read_lock_irqsave(&sg_index_lock, iflags);
	sdp = it ? sg_lookup_dev(it->index) : NULL;
	scsidp = sdp ? sdp->device : NULL;
	if (sdp && scsidp && (!atomic_read(&sdp->detaching)))
		seq_printf(s, "%8.8s\t%16.16s\t%4.4s\n",
			   scsidp->vendor, scsidp->model, scsidp->rev);
	else
		seq_puts(s, "<no active device>\n");
	read_unlock_irqrestore(&sg_index_lock, iflags);
	return 0;
}

/* Writes debug info for one sg_request in obp buffer */
static int
sg_proc_debug_sreq(struct sg_request *srp, int to, bool t_in_ns, char *obp,
		   int len)
	__must_hold(&srp->req_lck)
{
	bool is_v3v4, v4;
	int n = 0;
	enum sg_rq_state rq_st;
	const char *cp;
	const char *tp = t_in_ns ? "ns" : "ms";

	if (len < 1)
		return 0;
	v4 = test_bit(SG_FRQ_IS_V4I, srp->frq_bm);
	is_v3v4 = v4 ? true : (srp->s_hdr3.interface_id != '\0');
	if (srp->parentfp->rsv_srp == srp)
		cp = (is_v3v4 && (srp->rq_flags & SG_FLAG_MMAP_IO)) ?
				"     mmap>> " : "     rsv>> ";
	else
		cp = (srp->rq_info & SG_INFO_DIRECT_IO_MASK) ?
				"     dio>> " : "     ";
	rq_st = atomic_read(&srp->rq_st);
	n += scnprintf(obp + n, len - n, "%s%s: dlen=%d/%d id=%d", cp,
		       sg_rq_st_str(rq_st, false), srp->sgatp->dlen,
		       srp->sgatp->buflen, (int)srp->pack_id);
	if (rq_st == SG_RS_AWAIT_RD || rq_st == SG_RS_DONE_RD)
		n += scnprintf(obp + n, len - n, " dur=%u%s", srp->duration,
			      tp);
	else if (rq_st == SG_RS_INFLIGHT || rq_st == SG_RS_BUSY) {
		if ((srp->rq_flags & SGV4_FLAG_YIELD_TAG) &&
		    srp->tag != SG_TAG_WILDCARD)
			n += scnprintf(obp + n, len - n, " tag=0x%x",
				       srp->tag);
		n += scnprintf(obp + n, len - n, " t_o/elap=%us/%u%s",
			       to / 1000, sg_calc_req_time(srp, t_in_ns), tp);
	}
	if (srp->sh_var != SG_SHR_NONE)
		n += scnprintf(obp + n, len - n, " shr=%s",
			       sg_shr_str(srp->sh_var, false));
	n += scnprintf(obp + n, len - n, " sgat=%d op=0x%02x\n",
		       srp->sgatp->num_sgat, srp->cmd_opcode);
	return n;
}

/* Writes debug info for one sg fd (including its sg requests) in obp buffer */
static int
sg_proc_debug_fd(struct sg_fd *fp, char *obp, int len, bool reduced)
	__must_hold(&sfp->rq_list_lock)
{
	bool first_fl;
	int n = 0;
	const char *cp;
	struct sg_request *srp;

	if (fp->shr_fd == SG_SHARE_FD_UNUSED)
		cp = "";
	else
		cp = (fp->shr_fd == SG_SHARE_FD_MASTER) ?
			" shr_mast" : " shr_slv";
	/* sgat=-1 means unavailable */
	n += scnprintf(obp + n, len - n,
		       "timeout=%dms rsv_buflen=%d%s\n",
		       jiffies_to_msecs(fp->timeout),
		       fp->rsv_srp->sgatp->buflen, cp);
	n += scnprintf(obp + n, len - n,
		       "   cmd_q=%d f_packid=%d k_orphan=%d\n",
		       (int)test_bit(SG_FFD_CMD_Q, fp->ffd_bm),
		       (int)test_bit(SG_FFD_FORCE_PACKID, fp->ffd_bm),
		       (int)test_bit(SG_FFD_KEEP_ORPHAN, fp->ffd_bm));
	n += scnprintf(obp + n, len - n, "   mmap_called=%d sum_fd_dlens=%u\n",
		       test_bit(SG_FFD_MMAP_CALLED, fp->ffd_bm),
		       atomic_read(&fp->sum_fd_dlens));
	n += scnprintf(obp + n, len - n, "   submitted=%d waiting=%d\n",
		       atomic_read(&fp->submitted),
		       atomic_read(&fp->waiting));
	if (reduced)
		return n;
	list_for_each_entry(srp, &fp->rq_list, rq_entry) {
		spin_lock(&srp->req_lck);
		n += sg_proc_debug_sreq(srp, fp->timeout,
				test_bit(SG_FFD_TIME_IN_NS, fp->ffd_bm),
					obp + n, len - n);
		spin_unlock(&srp->req_lck);
	}
	if (list_empty(&fp->rq_list))
		n += scnprintf(obp + n, len - n, "     No requests active\n");
	first_fl = true;
	list_for_each_entry(srp, &fp->rq_fl, fl_entry) {
		if (first_fl) {
			n += scnprintf(obp + n, len - n, "   Free list:\n");
			first_fl = false;
		}
		spin_lock(&srp->req_lck);
		n += sg_proc_debug_sreq(srp, fp->timeout,
				test_bit(SG_FFD_MMAP_CALLED, fp->ffd_bm),
					obp + n, len - n);
		spin_unlock(&srp->req_lck);
	}
	return n;
}

/* Writes debug info for one sg device (including its sg fds) in obp buffer */
static int
sg_proc_debug_sdev(struct sg_device *sdp, char *obp, int len,
		   int *fd_counterp, bool reduced)
	__must_hold(&sdp->sfd_llock)
{
	int n = 0;
	int my_count = 0;
	struct scsi_device *ssdp = sdp->device;
	struct sg_fd *fp;
	char *disk_name;
	int *countp;

	countp = fd_counterp ? fd_counterp : &my_count;
	disk_name = (sdp->disk ? sdp->disk->disk_name : "??");
	n += scnprintf(obp + n, len - n, " >>> device=%s ", disk_name);
	n += scnprintf(obp + n, len - n, "%d:%d:%d:%llu ", ssdp->host->host_no,
		       ssdp->channel, ssdp->id, ssdp->lun);
	n += scnprintf(obp + n, len - n,
		       "  max_sgat_sz,elems=%d,%d excl=%d open_cnt=%d\n",
		       sdp->max_sgat_sz, sdp->max_sgat_elems, sdp->exclude,
		       sdp->open_cnt);
	list_for_each_entry(fp, &sdp->sfds, sfd_entry) {
		++*countp;
		read_lock(&fp->rq_list_lock); /* assume irqs disabled */
		n += scnprintf(obp + n, len - n, "  FD(%d): ", *countp);
		n += sg_proc_debug_fd(fp, obp + n, len - n, reduced);
		read_unlock(&fp->rq_list_lock);
	}
	return n;
}


/* Called via debug_seq_ops once for each sg device */
static int
sg_proc_seq_show_debug(struct seq_file *s, void *v, bool reduced)
{
	bool found = false;
	bool trunc = false;
	const int bp_len = SG_PROC_DEBUG_SZ;
	int n = 0;
	int k = 0;
	unsigned long iflags;
	struct sg_proc_deviter *it = (struct sg_proc_deviter *)v;
	struct sg_device *sdp;
	int *fdi_p;
	char *bp;
	char *disk_name;
	char b1[128];

	b1[0] = '\0';
	if (it && (0 == it->index))
		seq_printf(s, "max_active_device=%d  def_reserved_size=%d\n",
			   (int)it->max, def_reserved_size);
	fdi_p = it ? &it->fd_index : &k;
	bp = kzalloc(bp_len, __GFP_NOWARN | GFP_KERNEL);
	if (!bp) {
		seq_printf(s, "%s: Unable to allocate %d on heap, finish\n",
			   __func__, bp_len);
		return -1;
	}
	read_lock_irqsave(&sg_index_lock, iflags);
	sdp = it ? sg_lookup_dev(it->index) : NULL;
	if (NULL == sdp)
		goto skip;
	read_lock(&sdp->sfd_llock);
	if (!list_empty(&sdp->sfds)) {
		found = true;
		disk_name = (sdp->disk ? sdp->disk->disk_name : "??");
		if (atomic_read(&sdp->detaching))
			snprintf(b1, sizeof(b1), " >>> device=%s  %s\n",
				 disk_name, "detaching pending close\n");
		else if (sdp->device) {
			n = sg_proc_debug_sdev(sdp, bp, bp_len, fdi_p,
					       reduced);
			if (n >= bp_len - 1) {
				trunc = true;
				if (bp[bp_len - 2] != '\n')
					bp[bp_len - 2] = '\n';
			}
		} else
			snprintf(b1, sizeof(b1), " >>> device=%s  %s\n",
				 disk_name, "sdp->device==NULL, skip");
	}
	read_unlock(&sdp->sfd_llock);
skip:
	read_unlock_irqrestore(&sg_index_lock, iflags);
	if (found) {
		if (n > 0) {
			seq_puts(s, bp);
			if (seq_has_overflowed(s))
				goto s_ovfl;
			if (trunc)
				seq_printf(s, "   >> Output truncated %s\n",
					   "due to buffer size");
		} else if (b1[0]) {
			seq_puts(s, b1);
			if (seq_has_overflowed(s))
				goto s_ovfl;
		}
	}
s_ovfl:
	kfree(bp);
	return 0;
}

static int
sg_proc_seq_show_full_debug(struct seq_file *s, void *v)
{
	return sg_proc_seq_show_debug(s, v, false);
}

static int
sg_proc_seq_show_red_debug(struct seq_file *s, void *v)
{
	return sg_proc_seq_show_debug(s, v, true);
}

#if IS_ENABLED(SG_DEBUG)
static void
sg_trc_producer(const char *from, const char *extra, enum sg_trc_type tt,
		int trc_lev, void *vp, bool skip_lock, bool follow_slave)
{
	bool in_ns;
	int k, j, m, z, nd, t_o;
	int fd_count = 0;
	const int wp_len = SG_PROC_DEBUG_SZ;
	u64 now_ns;
	u32 now_secs, rem_ns;
	const char *extra_pad;
	struct sg_device *sdp;
	struct sg_fd *sfp;
	struct sg_fd *o_sfp;
	struct sg_request *srp;
	char *wp = sg_trc_wp;
	struct circ_buf *cbp;

	if (!sg_trc_cb_valid || trc_lev <= sg_trc_ctl_level)
		return;
	now_ns = ktime_to_ns(ktime_get_with_offset(TK_OFFS_BOOT));
	now_secs = now_ns / 1000000000;
	rem_ns = now_ns % 1000000000;
	cbp = &sg_trc_cb;
	if (!extra)
		extra = "";
	extra_pad = strlen(extra) ? ", " : "";
	mutex_lock(&sg_trc_cb_lck);
	switch (tt) {
	case SG_TRC_ALL:
		z = scnprintf(wp, wp_len, ">>>> %u.%09u %s: %s%s%s:\n",
			      now_secs, rem_ns, from, extra, extra_pad,
			      "All devices");
		if (!skip_lock)
			read_lock(&sg_index_lock); /* caller may have irqs */
		nd = sg_last_dev();
		for (k = 0; k < nd; ++k) {
			sdp = sg_lookup_dev(k);
			if (!sdp)
				continue;
			read_lock(&sdp->sfd_llock);
			z += sg_proc_debug_sdev(sdp, wp + z, wp_len - z,
						&fd_count, false);
			read_unlock(&sdp->sfd_llock);
		}
		if (!skip_lock)
			read_unlock(&sg_index_lock);
		break;
	case SG_TRC_DEV:
		z = scnprintf(wp, wp_len, "%s%u.%09u %s: %s%s%s [0x%p]:\n",
			      ">>>> ", now_secs, rem_ns, from, extra,
			      extra_pad, "Sg device", vp);
		sdp = vp;
		if (!skip_lock)
			read_lock(&sdp->sfd_llock);
		z += sg_proc_debug_sdev(sdp, wp + z, wp_len - z, &fd_count,
					false);
		if (!skip_lock)
			read_unlock(&sdp->sfd_llock);
		break;
	case SG_TRC_FD:
		z = scnprintf(wp, wp_len, "%s%u.%09u %s: %s%s%s [0x%p]:\n",
			      ">>>> ", now_secs, rem_ns, from, extra,
			      extra_pad, "Sg fd", vp);
		sfp = vp;
		if (!skip_lock)
			read_lock(&sfp->rq_list_lock);
		z += sg_proc_debug_fd(sfp, wp + z, wp_len - z, false);
		if (!skip_lock)
			read_unlock(&sfp->rq_list_lock);
		break;
	case SG_TRC_REQ:
		z = scnprintf(wp, wp_len, "%s%u.%09u %s: %s%s%s [0x%p]:\n",
			      ">>>> ", now_secs, rem_ns, from, extra,
			      extra_pad, "Sg request", vp);
		srp = vp;
		if (!skip_lock)
			spin_lock(&srp->req_lck);
		sfp = srp->parentfp;
		o_sfp = sfp->share_sfp;
		t_o = sfp->timeout;
		in_ns = test_bit(SG_FFD_TIME_IN_NS, sfp->ffd_bm);
		z += sg_proc_debug_sreq(srp, t_o, in_ns, wp + z, wp_len - z);
		if (follow_slave && o_sfp && o_sfp->rsv_srp &&
		    o_sfp->slave_srp == srp) {
			struct sg_request *ma_srp = o_sfp->rsv_srp;

			z += scnprintf(wp + z, wp_len - z,
				       "  >> Slave's master srp [0x%p]\n",
				       ma_srp);
			if (spin_trylock(&ma_srp->req_lck)) {
				z += sg_proc_debug_sreq(ma_srp, t_o, in_ns,
							wp + z, wp_len - z);
				spin_unlock(&ma_srp->req_lck);
			} else
				z += scnprintf(wp + z, wp_len - z,
					       "   is busy, don't wait\n");
		}
		if (!skip_lock)
			spin_unlock(&srp->req_lck);
		break;
	}
	if (z < 1)
		goto fini;
	/* no need to include null char at end */
	nd = CIRC_SPACE(cbp->head, cbp->tail, sg_trc_cb_sz);
	if (z > nd) {	/* overwrite oldest, fake consumer */
		j = z - nd;
		m = CIRC_CNT_TO_END(cbp->head, cbp->tail, sg_trc_cb_sz);
		if (j > m)
			cbp->tail = j - m;
		else
			cbp->tail += j;
	}
	m = CIRC_SPACE_TO_END(cbp->head, cbp->tail, sg_trc_cb_sz);
	if (z > m) {	/* real producer */
		memcpy(cbp->buf + cbp->head, wp, m);
		memcpy(cbp->buf + 0, wp + m, z - m);
		cbp->head = z - m;
	} else {
		memcpy(cbp->buf + cbp->head, wp, z);
		cbp->head += z;
	}
fini:
	mutex_unlock(&sg_trc_cb_lck);
}
#endif		/* SG_DEBUG */

#endif                          /* CONFIG_SCSI_PROC_FS (~800 lines back) */

module_init(init_sg);
module_exit(exit_sg);
